/*********************************************************************
	Component	: oxf
	Model Element	: RiJThread
	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2008. All Rights Reserved.	
*********************************************************************/

package com.ibm.rational.rhapsody.oxf;

import java.util.*;
//## operation getJThread() 
import java.lang.Thread;
//## operation RiJThread(String,int,Runnable) 
import java.lang.Runnable;
//## dependency timeouts 
import com.ibm.rational.rhapsody.oxf.timeouts.*;
//## dependency AnimServices 
import com.ibm.rational.rhapsody.oxfinst.AnimServices;

//----------------------------------------------------------------------------
// com/ibm/rational/rhapsody/oxf/RiJThread.java                                                                  
//----------------------------------------------------------------------------

//## package com::ibm::rational::rhapsody::oxf 


/**
The RiJThread is a class able to run in it's own thread, 
It is able to dispatch RiJEvent objects by extending RiJEventDispatcher.
The RiJThread does contain the methods stop(), suspend() and resume(), but it does them safely according to JDK documents recommendations. 
*/
//## class RiJThread 
public class RiJThread extends RiJEventDispatcher implements RiJActive {
    
    protected static final int WAIT_TIME = 10;		//## attribute WAIT_TIME 
    
    /**
    An optional string name. May be null.
    */
    protected String name;		//## attribute name 
    
    /**
    Thread priority.
    */
    protected int priority;		//## attribute priority 
    
    protected volatile Thread thread;		//## attribute thread 
    
    protected boolean threadSuspend = false;		//## attribute threadSuspend 
    
    protected boolean wasStarted = false;		//## attribute wasStarted 
    
    
    // Constructors
    
    //## operation RiJThread() 
    public  RiJThread() {
        //#[ operation RiJThread() 
        this(null);
        //#]
    }
    // Argument String name : 
    /**
    Optional thread name
    */
    //## operation RiJThread(String) 
    public  RiJThread(String name) {
        this(name, Thread.NORM_PRIORITY);;
        //#[ operation RiJThread(String) 
        //#]
    }
    // Argument int priority : 
    /**
    The thread's priority
    */
    //## operation RiJThread(String,int) 
    public  RiJThread(String name, int priority) {
        this(name, priority, null);
        //#[ operation RiJThread(String,int) 
        
        
        //#]
    }
    /**
    For backward compatibility with C++. For now this is meaningless.
    */
    //## operation RiJThread(boolean) 
    public  RiJThread(boolean wrapperThread) {
        //#[ operation RiJThread(boolean) 
        this();
        //#]
    }
    //## operation RiJThread(String,int,Runnable) 
    public  RiJThread(String name, int priority, Runnable runnable) {
        super();
        //#[ operation RiJThread(String,int,Runnable) 
        this.name = name;
        this.priority = priority;
        
        
        //#]
    }
    
    /**
    create \ replaces the underlying thread
    */
    //## operation createNewThread(Runnable) 
    protected void createNewThread(Runnable runnable) {
        //#[ operation createNewThread(Runnable) 
        // first eliminate reference to prev thread
        if (thread != null) {
        	thread = null;
        }
        
        if (runnable == null) 
        	runnable = this;
        if (name == null)
          thread = new Thread(runnable);
        else 
          thread = new Thread(runnable, name);
        thread.setPriority(priority);
        AnimServices.initAnimThread(this);
        
        //#]
    }
    
    /**
    Here is where all the non-thread issues (i.e. the functional ones) are carried out.
    */
    //## operation execute() 
    protected void execute() {
        //#[ operation execute() 
        // dispatch events 
        
        while (!eventQueue.isEmpty()) { 
            if (threadSuspend) return;  // stop processing events
            dispatchNextEvent();
            AnimServices.notifyEvent(this);
        } 
        
        
        
        //#]
    }
    
    /**
    return the underlying Java Thread.
    */
    //## operation getJThread() 
    public Thread getJThread() {
        //#[ operation getJThread() 
        return thread;
        //#]
    }
    
    //## operation getName() 
    public String getName() {
        //#[ operation getName() 
        return name;
        
        //#]
    }
    
    //## operation getPriority() 
    public int getPriority() {
        //#[ operation getPriority() 
        return priority;
        
        //#]
    }
    
    //## operation init() 
    public void init() {
        //#[ operation init() 
        //#]
    }
    
    //## operation isStarted() 
    public boolean isStarted() {
        //#[ operation isStarted() 
        return wasStarted;
        //#]
    }
    
    //## operation isSuspended() 
    public boolean isSuspended() {
        //#[ operation isSuspended() 
        return threadSuspend;
        //#]
    }
    
    /**
    Inserts an event to the queue and resumes execution of thread. 
    */
    // Argument RiJEvent i_event : 
    /**
    The event to be queued
    */
    //## operation queueEvent(RiJEvent) 
    public void queueEvent(RiJEvent i_event) {
        //#[ operation queueEvent(RiJEvent) 
        synchronized(eventQueue)
        {   
            super.queueEvent(i_event);
            // wake up event consumption
            eventQueue.notifyAll();
            
        }  
        
        //#]
    }
    
    /**
    Resumes execution.
    */
    //## operation resume() 
    public void resume() {
        //#[ operation resume() 
         threadSuspend = false;
         synchronized(this) {
           notify();
         }
         
        //#]
    }
    
    /**
    This method wraps the execute method, in which the actual event dispatching is called, and possibly other stuff.     
    */
    //## operation run() 
    public void run() {
        //#[ operation run() 
        wasStarted = true;
        Thread thisThread = Thread.currentThread();
        
        RiJTimeoutManager.instance().incNonIdleCount();
        
        while (thisThread == thread) { // as long as the thread is alive
         // if suspended wait to be resumed
         synchronized(this) 
         {
            while (threadSuspend) 
            {
               try 
               {    
                 wait();                
               }
               catch (InterruptedException e) {
               }
            }
         }
        
         RiJTimeoutManager.instance().decNonIdleCount();
        
         // wait for an event
         synchronized(eventQueue)
         {   
             while (eventQueue.isEmpty()) {   
                AnimServices.notifyIdle(this);
                try {      
                                  
                     eventQueue.wait();
        
                }                     
                catch (InterruptedException e) {
                }
             }
         }
        
         RiJTimeoutManager.instance().incNonIdleCount();
         
         // do the job
         execute();
        }
        
        //#]
    }
    
    /**
    Schedule timeout event for the given reactive
    */
    // Argument long delay : 
    /**
    Delay time in milli-seconds
    */
    // Argument long timeoutID : 
    /**
    ID of the timeout
    */
    // Argument RiJReactive reactive : 
    /**
    The reactive event consumer of the event
    */
    // Argument String stateName : 
    /**
    The state name in which this timeout was instantiated.
    */
    //## operation schedTimeout(long,long,RiJReactive,String) 
    public void schedTimeout(long delay, long timeoutID, RiJReactive reactive, String stateName) {
        //#[ operation schedTimeout(long,long,RiJReactive,String) 
        // create the timeout event
        RiJTimeout timeout = new RiJTimeout(timeoutID, reactive, delay, stateName);
        
        // schedule 
        RiJTimeoutManager tManager = RiJTimeoutManager.instance();
        tManager.schedTimeout(timeout, reactive.getItsRiJThread());
        
        //#]
    }
    
    /**
    Schedule timeout event for the given reactive
    */
    // Argument long delay : 
    /**
    Delay time in milli-seconds
    */
    // Argument long timeoutID : 
    /**
    ID of the timeout
    */
    // Argument RiJReactive reactive : 
    /**
    The reactive event consumer of the event
    */
    //## operation schedTimeout(long,long,RiJReactive) 
    public void schedTimeout(long delay, long timeoutID, RiJReactive reactive) {
        //#[ operation schedTimeout(long,long,RiJReactive) 
        schedTimeout(delay, timeoutID, reactive, "No State");
        //#]
    }
    
    // Argument String name : 
    /**
    Name to be assigned to thread.
    */
    //## operation setName(String) 
    public void setName(String name) {
        //#[ operation setName(String) 
        this.name = name;
        if (thread != null) 
          thread.setName(name);
        //#]
    }
    
    /**
    set the thread attribute
    the setting pass only if the thread is null, the method return sucess status
    */
    //## operation setThread(Thread) 
    protected synchronized boolean setThread(Thread theThread) {
        //#[ operation setThread(Thread) 
        boolean status = (thread == null);
        if (thread == null) {
        	thread = theThread;
        }
        return status;
        //#]
    }
    
    /**
    starts the thread execution, with "this" as runnable
    */
    //## operation start() 
    public void start() {
        //#[ operation start() 
        start(this);
        //#]
    }
    
    /**
    starts the thread execution
    */
    //## operation start(Runnable) 
    public void start(Runnable runnable) {
        //#[ operation start(Runnable) 
        if (thread == null || ((runnable != null) && (runnable != this)))
        {
        	createNewThread(runnable);
        }
        AnimServices.threadStart(this, runnable);    
        
        
        thread.start();
        //#]
    }
    
    /**
    The start(doFork) API is added to allow users to customize the system default active class.
    The user must implement the doFork it self (RiJMainThread.start(doFork) can be used as a reference)
    */
    //## operation start(boolean) 
    public void start(boolean doFork) {
        //#[ operation start(boolean) 
        start();
        //#]
    }
    
    //## operation started() 
    public boolean started() {
        //#[ operation started() 
        return wasStarted;
        //#]
    }
    
    //## operation stop() 
    synchronized public void stop() {
        //#[ operation stop() 
        thread = null;
        //#]
    }
    
    /**
    Suspends execuation (safe from deadlocks), 
    in case of the default implementation of run(), the current event will be processed through and then the thread will be suspended. (see execute()).
    */
    //## operation suspend() 
    synchronized public void suspend() {
        //#[ operation suspend() 
        threadSuspend = true;
        
        //#]
    }
    
    /**
    unschedules a timeout event
    */
    //## operation unschedTimeout(long,RiJEventConsumer) 
    public void unschedTimeout(long timeoutID, RiJEventConsumer dest) {
        //#[ operation unschedTimeout(long,RiJEventConsumer) 
        // dummy but equal
        RiJTimeout dummyTm = new RiJTimeout(timeoutID, dest, 0);
        RiJTimeoutManager.instance().unschedTimeout(dummyTm);
        
        //#]
    }
    
    /**
    Inserts an event to the queue in an unsynchronized way.
    To be used only by animation frameowrk.
    */
    // Argument RiJEvent i_event : 
    /**
    The event to be queued
    */
    //## operation unsynchQueueEvent(RiJEvent) 
    public void unsynchQueueEvent(RiJEvent i_event) {
        //#[ operation unsynchQueueEvent(RiJEvent) 
        
        // if thread was suspended by animation enter event to queue 
        // and go on.
        // if thread is not suspended, queue event in the 
        // notmal way
        if (AnimServices.isAnimThreadSuspended(this))
        {
            super.queueEvent(i_event);               
        }    
        else         
        {
            queueEvent(i_event);
        }    
            
        
        //#]
    }
    
}

