/*********************************************************************
	Component	: oxf
	Model Element	: RiJSemaphore
	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2008. All Rights Reserved.	
*********************************************************************/

package com.ibm.rational.rhapsody.oxf.services;

import java.util.*;
//## operation findLocker(Thread) 
import java.lang.Thread;

//----------------------------------------------------------------------------
// com/ibm/rational/rhapsody/oxf/services/RiJSemaphore.java                                                                  
//----------------------------------------------------------------------------

//## package com::ibm::rational::rhapsody::oxf::services 


/**
Semaphore implementation in Java.
*/
//## class RiJSemaphore 
public class RiJSemaphore {
    
    final public static short NOT_ALLOWED = -2;		//## attribute NOT_ALLOWED 
    
    final public static short REACHED_MAX = -1;		//## attribute REACHED_MAX 
    
    final public static short SUCCESS = 0;		//## attribute SUCCESS 
    
    /**
    number of lockers.
    */
    private int counter;		//## attribute counter 
    
    /**
    Max. Number of Lockers. 
    */
    private int maxLockers;		//## attribute maxLockers 
    
    /**
    Locker Threads
    */
    private Thread[] threads;		//## attribute threads 
    
    
    // Constructors
    
    /**
    Initializes a semaphore. Both args must be positive. maxCount must be greater or equal to initialCount, otherwise irregular behaviour will occure.
    */
    //## operation RiJSemaphore(int,int) 
    public  RiJSemaphore(int initialCount, int maxCount) {
        //#[ operation RiJSemaphore(int,int) 
        counter = initialCount;
        maxLockers = maxCount;
        threads = new Thread[maxCount+1];
        
        //#]
    }
    
    /**
    Counts number of locker threads
    */
    //## operation countLockers() 
    synchronized protected int countLockers() {
        //#[ operation countLockers() 
        int i;
        for (i=0; (i<threads.length) && (threads[i] != null); ++i);
        return i;
        //#]
    }
    
    /**
    find the lowest index free spot for a new locker.
    */
    //## operation findFreeSpot() 
    synchronized protected int findFreeSpot() {
        //#[ operation findFreeSpot() 
        for (int i=0; i<threads.length; i++) {
          if (threads[i] == null) 
             return i;
        }
        return -1; 
        //#]
    }
    
    /**
    check if thread is among those who already have a token.
    */
    //## operation findLocker(Thread) 
    synchronized protected int findLocker(Thread thread) {
        //#[ operation findLocker(Thread) 
        for (int i=0; i<threads.length; i++) {
          if (threads[i] == thread) 
             return i;
        }
        return -1;
        //#]
    }
    
    /**
    Allows an additional thread to start by increasing the counter and notifying. However, the thread is not removed and still remains a "locker".
    Only "locker" threads may increase counters.
    */
    //## operation increaseCounter() 
    synchronized public short increaseCounter() {
        //#[ operation increaseCounter() 
        if (counter + countLockers() >= maxLockers)
          return REACHED_MAX;
        if (findLocker(Thread.currentThread()) < 0)
          return NOT_ALLOWED;
        counter++;
        notify(); // this is the last thing to be done
        return SUCCESS;
        //#]
    }
    
    /**
    try to lock, but give up after timeout. timeout = 0 equivalent to infinity.
    */
    //## operation lock(long) 
    synchronized public void lock(long timeout) throws InterruptedException {
        //#[ operation lock(long) 
        Thread currentThread = Thread.currentThread();
        if (findLocker(currentThread) > -1) return;
        if (counter == 0) {
          // block
          wait(timeout);
        }
        // take it
        --counter;
        threads[findFreeSpot()] = currentThread;
        
        //#]
    }
    
    /**
    try to lock, wailt infinitly until able.
    */
    //## operation lock() 
    synchronized public void lock() throws InterruptedException {
        //#[ operation lock() 
        lock(0);
        
        //#]
    }
    
    /**
    Releases the lock on the semaphore.
    */
    //## operation release() 
    synchronized public short release() {
        //#[ operation release() 
        if (counter == maxLockers)
          return REACHED_MAX;
        if (!removeLocker(Thread.currentThread()))
          return NOT_ALLOWED;
        counter++;
        notify(); // this is the last thing to be done
        return SUCCESS;
        //#]
    }
    
    /**
    check if thread is among those who already have a token.
    */
    //## operation removeLocker(Thread) 
    synchronized protected boolean removeLocker(Thread thread) {
        //#[ operation removeLocker(Thread) 
        int pos;
        if ((pos = findLocker(thread)) > -1) {
          threads[pos] = null;
          return true;
        }
        return false;
        //#]
    }
    
}

