/*********************************************************************
	Component	: oxf
	Model Element	: RiJStateReactive
	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2008. All Rights Reserved.	
*********************************************************************/

package com.ibm.rational.rhapsody.oxf.states;

import java.util.*;
//## attribute busy 
import java.lang.Boolean;
//## dependency oxf 
import com.ibm.rational.rhapsody.oxf.*;
//## operation _takeEvent(RiJEvent) 
import com.ibm.rational.rhapsody.oxf.RiJEvent;
//## operation RiJStateReactive(RiJThread) 
import com.ibm.rational.rhapsody.oxf.RiJThread;
//## class RiJStateReactive 
import com.ibm.rational.rhapsody.oxf.RiJReactive;
//## dependency AnimServices 
import com.ibm.rational.rhapsody.oxfinst.AnimServices;
//## dependency RiJMutex 
import com.ibm.rational.rhapsody.oxf.services.RiJMutex;

//----------------------------------------------------------------------------
// com/ibm/rational/rhapsody/oxf/states/RiJStateReactive.java                                                                  
//----------------------------------------------------------------------------

//## package com::ibm::rational::rhapsody::oxf::states 


/**
RiJStateReactive is a reactive capable of "driving" a statemachine.
*/
//## class RiJStateReactive 
public class RiJStateReactive extends RiJReactive {
    
    /**
    Default value for maximum allowed null transitions.
    */
    final public static int DEF_MAX_NULL_STEPS = 32;		//## attribute DEF_MAX_NULL_STEPS 
    
    final public static int TAKE_EVENT_COMPLETE = 0;		//## attribute TAKE_EVENT_COMPLETE 
    
    final public static int TAKE_EVENT_NOT_CONSUMED = -3;		//## attribute TAKE_EVENT_NOT_CONSUMED 
    
    final public static int TAKE_EVENT_REACHED_TERMINATOR = -1;		//## attribute TAKE_EVENT_REACHED_TERMINATOR 
    
    final public static int TAKE_EVENT_UNKNOWN_FAIL = -2;		//## attribute TAKE_EVENT_UNKNOWN_FAIL 
    
    /**
    For mutual exclusion.
    */
    private Boolean busy = Boolean.FALSE;		//## attribute busy 
    
    /**
    a convenient attribute to avoid having the event transferred.
    */
    public RiJEvent event;		//## attribute event 
    
    /**
    This is for mutial exclusion.
    */
    private RiJMutex eventMutex = null;		//## attribute eventMutex 
    
    private boolean isBehaviorStarted = false;		//## attribute isBehaviorStarted 
    
    /**
    The thread running the instance.
    */
    protected RiJThread m_thread;		//## attribute m_thread 
    
    /**
    max. number of allowed null transitions.
    */
    protected int maxNullSteps = DEF_MAX_NULL_STEPS;		//## attribute maxNullSteps 
    
    /**
    If true, then a triggered operation invoked from another triggered operation is ignored (default).
    If false, it is not ignore.
    */
    protected boolean mutualExclusionOfTrigOp = true;		//## attribute mutualExclusionOfTrigOp 
    
    protected int nullStepsCounter;		//## attribute nullStepsCounter 
    
    protected boolean shouldCompleteStartBehavior = false;		//## attribute shouldCompleteStartBehavior 
    
    protected boolean terminated = false;		//## attribute terminated 
    
    protected RiJImplicitState rootState;
    
    // Constructors
    
    // Argument RiJThread thread : 
    /**
    m_thread = i_thread;
    */
    //## operation RiJStateReactive(RiJThread) 
    public  RiJStateReactive(RiJThread thread) {
        //#[ operation RiJStateReactive(RiJThread) 
        super(thread);
        maxNullSteps = DEF_MAX_NULL_STEPS;
        //#]
    }
    public  RiJStateReactive() {
    }
    
    //## operation _takeEvent(RiJEvent) 
    protected int _takeEvent(RiJEvent event) {
        //#[ operation _takeEvent(RiJEvent) 
        // mutual exclusion - 
        // the event guard is set -
        // use it to set mutual exclusion between Events and Triggered Operations
        // the m_eventGuard is set by the application when there is a guarded triggered operation
        
        //lockEventMutex();
        
        RiJEvent prevEvent = this.event;
        this.event = event;
        int res = TAKE_EVENT_NOT_CONSUMED;
        
        
        // the first recieved event, need to preform runToCompletion(),
        // if in startBehavior() shouldCompleteStartBehavior is set to true.
        // for any other event shouldCompleteStartBehavior should be false.
        if (shouldCompleteStartBehavior) {
            shouldCompleteStartBehavior = false;
            if (!mutualExclusionOfTrigOp) {
                runToCompletion();
            }
            else {
                doBusy();
                runToCompletion();
                undoBusy();
            }
        }
        
        // consume all events but the start behavior dummy event
        if (event.getID() != RiJEvent.START_BEHAVIOR_EVENT_ID) {
            if (!mutualExclusionOfTrigOp) {
            	res = consumeEvent(event);
            	// consume null transitions
            	if (shouldCompleteRun()) runToCompletion();
            }
            else {
                if (isBusy()==false)
                {  
                    doBusy();
                	res = consumeEvent(event);
                	// consume null transitions
                	if (shouldCompleteRun()) runToCompletion();
                	undoBusy();
                } 
        //        else {
        //            System.out.println("busy for " + event + " this: " + this + " thread: " + Thread.currentThread());
        //            System.out.println("prevEvent = " + prevEvent);
        //        }
            }
        }
        //freeEventMutex();
        
        return res;
        
        //#]
    }
    
    /**
    consumes the event
    */
    //## operation consumeEvent(RiJEvent) 
    protected int consumeEvent(RiJEvent event) {
        //#[ operation consumeEvent(RiJEvent) 
        int res = TAKE_EVENT_NOT_CONSUMED;
        if (terminated) { 
            return TAKE_EVENT_REACHED_TERMINATOR;
        }
        if (event.getID() == RiJEvent.NULL_EVENT_ID && shouldCompleteRun()) {
            AnimServices.notifyNullTransition(this);
        }
        res = rootState_dispatchEvent(event);
        return res;
        //#]
    }
    
    //## operation createEventMutex() 
    public void createEventMutex() {
        //#[ operation createEventMutex() 
        eventMutex = new RiJMutex();
        //#]
    }
    
    //## operation doBusy() 
    public void doBusy() {
        //#[ operation doBusy() 
        busy = Boolean.TRUE;
        
        //#]
    }
    
    //## operation freeEventMutex() 
    public short freeEventMutex() {
        //#[ operation freeEventMutex() 
        short res = RiJMutex.NOT_OWNER;
        if (eventMutex != null) {    
            //System.out.println(this + " freed  " + eventMutex);
        	res = eventMutex.free();
        }        
        return res;
        
        //#]
    }
    
    //## operation getRootState() 
    public RiJOrState getRootState() {
        //#[ operation getRootState() 
        return rootState;
        
        //#]
    }
    
    /**
    handle event that was not consumed, actual behavoir should be set by the user, by customizing the reactive class behavior
    */
    //## operation handleEventNotConsumed(RiJEvent) 
    public void handleEventNotConsumed(RiJEvent event) {
        //#[ operation handleEventNotConsumed(RiJEvent) 
        //#]
    }
    
    /**
    handle triggered operation that was not consumed, actual behavoir should be set by the user, by customizing the reactive class behavior
    */
    //## operation handleTONotConsumed(RiJEvent) 
    public void handleTONotConsumed(RiJEvent event) {
        //#[ operation handleTONotConsumed(RiJEvent) 
        //#]
    }
    
    //## operation isBusy() 
    public boolean isBusy() {
        //#[ operation isBusy() 
        boolean res = true;
        if (busy.equals(Boolean.FALSE)) {
             res = false;
        }
        return res;
        //#]
    }
    
    /**
    Returns the port through which the current event arrived.
    Returns null if event was sent directly.
    */
    //## operation isPort(Object) 
    public boolean isPort(Object port) {
        //#[ operation isPort(Object) 
        boolean res = false;
        if ((event != null) && (port != null))
          res = (event.getPort() == port);
        return res;
        //#]
    }
    
    /**
    Indicates if reached termination connector.
    */
    //## operation isTerminated() 
    public boolean isTerminated() {
        //#[ operation isTerminated() 
        return terminated;
        //#]
    }
    
    //## operation lockEventMutex() 
    public void lockEventMutex() {
        //#[ operation lockEventMutex() 
        
        if (eventMutex != null) {  
            //System.out.println(this + " locked " + eventMutex);
            try {
        	    eventMutex.lock();
            } catch(InterruptedException e) {
                System.err.println("Faild to lock");
                System.err.println("Exception : " + e);
            }
        }                                                         
        
        //#]
    }
    
    /**
    push a null transition to be taken.
    */
    //## operation popNullConfig() 
    synchronized public void popNullConfig() {
        //#[ operation popNullConfig() 
        if (nullStepsCounter == 0) {
          System.err.println("Error : NullStepsUnderflow");
          System.exit(1);
          // throw new NullStepsUnderflowException();
        }
        --nullStepsCounter;
        
        //#]
    }
    
    /**
    push a null transition to be taken.
    */
    //## operation pushNullConfig() 
    synchronized public void pushNullConfig() {
        //#[ operation pushNullConfig() 
        if (nullStepsCounter >= maxNullSteps) {
          System.err.println(nullStepsCounter  + "Error : NullStepsOverflow");
          System.exit(1);
          // throw new NullStepsOverflowException();
        }
        
        ++nullStepsCounter;
        
        //#]
    }
    
    /**
    consumes the event
    */
    //## operation rootState_dispatchEvent(RiJEvent) 
    protected int rootState_dispatchEvent(RiJEvent event) {
        //#[ operation rootState_dispatchEvent(RiJEvent) 
        
        // TEMP: this is a patch for flat implementation 
        if (rootState == null)
        {
          short id = (short)event.getID();
          return rootState_dispatchEvent(id);
        }
        
        
        
        if (rootState != null) {
          RiJState active = rootState.getItsActiveState();
          if (active != null)
            return active.takeEvent(event);
        }
        return TAKE_EVENT_UNKNOWN_FAIL;
        //#]
    }
    
    /**
    TEMP:
    DUMMY FUNCTION USED FOR FLAT. 
    */
    //## operation rootState_dispatchEvent(short) 
    public int rootState_dispatchEvent(short id) {
        //#[ operation rootState_dispatchEvent(short) 
        return TAKE_EVENT_UNKNOWN_FAIL;
        //#]
    }
    
    //## operation rootState_entDef() 
    public void rootState_entDef() throws StateEnterException {
        //#[ operation rootState_entDef() 
        if (rootState != null)
          rootState.entDef();
        //runToCompletion();
        //#]
    }
    
    /**
    Fire null events until statemachine reaches a steady state.
    */
    //## operation runToCompletion() 
    synchronized public void runToCompletion() {
        //#[ operation runToCompletion() 
        
        // this counter is used to avoid infinite loops
        short localCounter = 0;
        
        // event destination
        RiJStateReactive dest = this;
        
        // a null event to be fired until state machine is stable
        RiJEvent nullEvent = new RiJEvent(RiJEvent.NULL_EVENT_ID, dest);
        // set the null event as the current event 
        // while handling null transitions
        RiJEvent originalEvent = this.event;
        this.event = nullEvent; 
        
        while (consumeEvent(nullEvent) >= 0) // as long event consumption is well
        {
          //AnimServices.notifyNullTransition(this);
          localCounter++;
          if (localCounter > maxNullSteps) {
            // throw new RiJState
            RiJInformer.instance().printMsg(this, "Error : Infinite Loop"); 
            break;
          }
          if (terminated) break;
        }
        
        // reset the current event
        this.event = originalEvent;
        nullEvent = null; // delete event object
        
        //#]
    }
    
    /**
    Are there any null transitions to perform?
    */
    //## operation shouldCompleteRun() 
    protected boolean shouldCompleteRun() {
        //#[ operation shouldCompleteRun() 
        if (nullStepsCounter == 0) return false;
        return true;
        //#]
    }
    
    //## operation startBehavior() 
    public boolean startBehavior() {
        //#[ operation startBehavior() 
        try {
        	if (!isBehaviorStarted) {
        		isBehaviorStarted = true;
        		rootState_entDef();
        		if (shouldCompleteRun()) {
        			// generate a dummy event in case the class doesn't recieve any external events
        			// this cause the runToCompletion() after the default transition to be taken -
        			// in the class own thread (for active classes)
        			shouldCompleteStartBehavior = true;
        			RiJEvent ev = new RiJEvent(RiJEvent.START_BEHAVIOR_EVENT_ID, this);
        			_gen(ev);
        		}
        	}
        }
        catch (StateEnterException stateEE) {
        	RiJInformer.instance().printMsg(this, "Exception: " + stateEE);
        	return false;
        }
        return true;
        //#]
    }
    
    //## operation takeEvent(RiJEvent) 
    public int takeEvent(RiJEvent event) {
        //#[ operation takeEvent(RiJEvent) 
        if (terminated) {   
            return TAKE_EVENT_REACHED_TERMINATOR;
        }
        
        int res = TAKE_EVENT_NOT_CONSUMED;
        if (eventMutex != null) {
            synchronized(eventMutex) {
                return _takeEvent(event);
            }
        }
        else {
            res = _takeEvent(event);
        }
        
        // handle event that was not consumed
        if (res == TAKE_EVENT_NOT_CONSUMED) {
            if (event.getIsTriggeredOperation()) {
                handleTONotConsumed(event);
            }
            else {
                handleEventNotConsumed(event);
            }
        }
        
        return res;
        //#]
    }
    
    //## operation takeTrigger(RiJEvent) 
    public int takeTrigger(RiJEvent event) {
        //#[ operation takeTrigger(RiJEvent) 
        // mark the event as triggered operation
        event.setIsTriggeredOperation(true);
        // actually call takeEvent()
        return takeEvent(event);
        //#]
    }
    
    //## operation terminate() 
    public void terminate() {
        //#[ operation terminate() 
        terminated = true;
        //#]
    }
    
    //## operation undoBusy() 
    public void undoBusy() {
        //#[ operation undoBusy() 
          busy = Boolean.FALSE;
        
        //#]
    }
    
    public void setMaxNullSteps(int p_maxNullSteps) {
        maxNullSteps = p_maxNullSteps;
    }
    
    public boolean getMutualExclusionOfTrigOp() {
        return mutualExclusionOfTrigOp;
    }
    
    public void setMutualExclusionOfTrigOp(boolean p_mutualExclusionOfTrigOp) {
        mutualExclusionOfTrigOp = p_mutualExclusionOfTrigOp;
    }
    
    public void setRootState(RiJImplicitState p_RiJImplicitState) {
        rootState = p_RiJImplicitState;
    }
    
}

