package com.sodius.mdw.core.eval;

import java.io.File;

/**
 * Describes evaluation properties.
 * 
 * <p>Note : template and script designers should generally not have to consider this configuration,
 * which is directly handled by the workbench.</p>
 * 
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 */
public interface EvaluationConfiguration {
    
    /**
     * Returns the evaluation directory, which determines where to write relative generated files.
     * 
     * <p>This directory is used to resolve the relative path of generated files.
     * For example, if a generation output specifies a file <code>"myOutputFolder\myFile.txt"</code> and
     * if the evaluation directory is <code>"c:\results"</code>, 
     * then the absolute path will be <code>"c:\results\myOutputFolder\myFile.txt"</code>.</p>
     * 
     * <p>It is always a good pratice to specify relative paths in text templates,
     * and let the end-user having the control of where to write the generated files.</p>
     * 
     * @return the evaluation directory (default is the Java system property <code>"user.home"</code>).
     */
    public File getEvaluationDirectory();
    
    /**
     * Sets the evaluation directory.
     * This evaluation directory may not exist at this time. 
     * In that case, it will be created when the first generated file is written out.
     * 
     * @param directory the new evaluation directory.
     * @throws IllegalArgumentException if the directory is not valid (if it is a file for example)
     */
    public void setEvaluationDirectory(File directory);
    
    /**
     * Returns <code>true</code> if generated files must be written out even 
     * if their contents is the same as the one on disk.
     * 
     * <p>This options has no impact on performances and 
     * it is recommended to not overwrite unmodified files when a source configuration management system is used.</p>
     * 
     * @return <code>true</code> if generated files must always be written on disk, 
     * <code>false</code> if the engine must first check wheter the contents differs from the one on disk (default is <code>false</code>. 
     */
    public boolean isFileOverwriteForced();
    
    /**
     * Specifies wheter generated files must be written on disk even 
     * if their contents is the same as the one on disk.
     * 
     * @param value <code>true</code> if generated files must always be written on disk, 
     * <code>false</code> if the engine must first check wheter the contents differs from the one on disk.
     */
    public void setFileOverwriteForced(boolean value);
    
    /**
     * Returns <code>true</code> if a debug hierarchy must be created during the evaluation.
     * 
     * <p>A debug hierarchy keeps track of every evaluated template, rule and script, for debugging purpose.
     * This lets the developer navigate in the evaluation tree once the evaluation is completed.</p>
     * 
     * @return <code>true</code> if a debug hierarchy must be computed, <code>false</code> otherwise (default is <code>false</code>).
     * @see EvaluationResult#getDebugHierarchy()
     */
    public boolean isDebugHierarchyEnabled();
    
    /**
     * Sets wheter a debug hierarchy should be created during the evaluation.
     * 
     * <p>A debug hierarchy keeps track of every evaluated template, rule and script, for debugging purpose.
     * This lets the developer navigate in the evaluation tree once the evaluation is completed.</p>
     * 
     * <p>A debug hierarchy comes at a considerable cost in both time and space,
     * and should not be requested frivolously. The additional space is not reclaimed until the 
     * debug hierarchy and all its nodes become garbage. So it is very
     * important to not retain any of these objects longer than absolutely necessary </p>
     * 
     * @param value determines if a debug hierarchy must be computed
     */
    public void setDebugHierarchyEnabled(boolean value);

}
