package com.sodius.mdw.core.eval;

import java.util.List;

import com.sodius.mdw.core.MDWorkbench;
import com.sodius.mdw.core.OperationCanceledException;
import com.sodius.mdw.core.eval.project.Project;
import com.sodius.mdw.core.model.Model;
import com.sodius.mdw.core.util.log.Logger;
import com.sodius.mdw.core.util.progress.ProgressMonitor;

/**
 * Provides services that helps to build model transformators/generators.
 * 
 * <p>An instanceof of <code>EvaluationContext</code> is always accessible 
 * from templates and scripts through the <code>context</code> variable.</p>
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 */
public interface EvaluationContext {
    
    /**
     * Returnq the workbench in which the evaluation is performed, 
     * which may be used to access configuration properties.
     * The workbench configuration provides for example access to the command line that launched the workbench, 
     * and can determine wheter the workbench runs in headless (no user interface) mode. 
     * 
     * @return the workbench.
     */
    public MDWorkbench getWorkbench();
    
    /**
     * Returns the logger used to report errors and warnings, as well as debugging information. 
     * 
     * @return the logger.
     */
    public Logger getLogger();
    
    /**
     * Returns the progress monitor used to report progress information.
     * 
     * <p>A script designer may report progress information to the end-user by calling
     * the <code>subTask()</code> method this way :</p>
     * <p><code>
     * context.getProgressMonitor().subTask("Analysing packages");<br>
     * context.evaluateRule(...);<br><br>
     * context.getProgressMonitor().subTask("Generating some files");<br>
     * context.generate(...);<br><br>
     * </code></p>
     * 
     * @return the progress monitor used to report progress information (never <code>null</code>).
     */
    public ProgressMonitor getProgressMonitor();
    
    /**
     * Provides an easy way to interrupt the evaluation.
     * This method throws an <code>OperationCanceledException</code>, 
     * to abort the evaluation, that is silently handled by the workbench.
     * 
     * <p>This method is designed to respond to an end-user request.
     * For example, if a selection dialog box is opened and the user press the Cancel button,
     * this method helps to silently terminate the evaluation.
     * But if the evaluation cannot complete correctly because of invalid input or
     * missing configuration properties, you should rather throw an exception (like <code>EvaluationException</code>).</p>
     * 
     * @throws OperationCanceledException to interrupt the evaluation.
     */
    public void cancelEvaluation() throws OperationCanceledException;

    /**
     * Evaluates the specified text template and returns the generated contents.
     * 
     * <p>The template name can be a fully qualified name (<code>"myPackage.myTemplate"</code> for example),
     * or a simple name if the template is part of the caller package (<code>"myTemplate"</code> if called
     * from the package <code>"myPackage"</code>).</p>
     * 
     * <p>The number and type of arguments must match the parameters of the template. 
     * If the template does not declare parameters, the arguments can be an empty list or <code>null</code>.</p>
     * 
     * <p><b>Important note :</b> the text template is evaluated but the generated contents is not written out on disk.
     * While this method may be helpfull in some specific cases, you will generally prefer to use the <code>generate</code> method.</p>
     * 
     * @param templateName the name of the template.
     * @param arguments the template arguments (can be <code>null</code> if no arguments expected)
     * @return a generated code, which describes the text template output.
     * @throws EvaluationException if anything prevents the template to be evaluated or its generated contents to be persisted.
     * @see #generate(String, List)
     */
    public GeneratedCode createGeneratedCode(String templateName, List<?> arguments);
    
    /**
     * Evaluates the specified text template and write on disk the generated contents.
     * 
     * <p>The template name can be a fully qualified name (<code>"myPackage.myTemplate"</code> for example),
     * or a simple name if the template is part of the caller package (<code>"myTemplate"</code> if called
     * from the package <code>"myPackage"</code>).</p>
     * 
     * <p>The number and type of arguments must match the parameters of the template. 
     * If the template does not declare parameters, the arguments can be an empty list or <code>null</code>.</p>
     * 
     * <p>The template is evaluated and the generated contents is written out, as specified by
     * the file property of the text template. If the template does not specify a file property,
     * an <code>EvaluationException</code> if thrown.</p>
     * 
     * @param templateName the name of the template.
     * @param arguments the template arguments (can be <code>null</code> if no arguments expected)
     * @throws EvaluationException if anything prevents the template to be evaluated or its generated contents to be persisted.
     * @see GeneratedCode#write()
     */
    public void generate(String templateName, List<?> arguments);
    
    /**
     * Evaluates the specified rule and returns the evaluation result.
     * 
     * <p>The rule set name can be a fully qualified name (<code>"myPackage.myRuleSet"</code> for example),
     * or a simple name if the rule set is part of the caller package (<code>"myRuleSet"</code> if called
     * from the package <code>"myPackage"</code>).</p>
     * 
     * <p>The rule name must match a rule declared by the rule set and that is :
     * <ul>
     * <li>a top-level rule : you can't call subrules outside of the declaring rule set.</li>
     * <li>visible from the caller (the rule must no be <code>private</code>).</li>
     * </ul>
     * </p>
     * 
     * <p>The number and type of arguments must match the parameters of the rule set and of the rules. 
     * If the rule set or the rule does not declare parameters, the arguments can be an empty list or <code>null</code>.</p>
     * 
     * @param ruleSetName the name of the rule set.
     * @param ruleSetArguments the rule set arguments (can be <code>null</code> if no arguments expected)
     * @param ruleName the name of the rule.
     * @param ruleArguments the rules arguments (can be <code>null</code> if no arguments expected)
     * @return the rule evaluation result (can be <code>null</code>) 
     * @throws EvaluationException if anything prevents the rule to be evaluated.
     */
    public Object evaluateRule(String ruleSetName, List<?> ruleSetArguments, String ruleName, List<?> ruleArguments); 
    
    /**
     * Returns the transient link manager, which provides facilities to dynamically create virtual links between objects of any type.
     * These links do not exist before the evaluation and are automatically garbaged at the end of the evaluation process.
     * They do never alter the models.
     * 
     *  @return the transient link manager.
     */
    public TransientLinkManager getTransientLinks();
    
    /**
     * Return the project in which the evaluation takes place.
     * The project provides a way to dynamically discover available scripts and templates.
     * 
     * <p>Note : template and script designers should generally not have to consider this advanced facility.</p>
     * 
     * @return the project in which the evaluation takes place.
     */
    public Project getProject();
    
    /**
     * Return the configuration that define options of this evaluation.
     * 
     * <p>Note : template and script designers should generally not have to consider this configuration,
     * which is directly handled by the workbench.</p>
     * 
     * @return the evaluation configuration.
     */
    public EvaluationConfiguration getConfiguration();
    
    /**
     * Creates an empty model based on the specified metamodel.
     * 
     * @param metamodelID a metamodel unique identifier.
     * @return an empty model.
     * @throws EvaluationException if no metamodel matches this ID.
     */
    public Model createModel(String metamodelID);
    
}
