package com.sodius.mdw.core.eval;

import java.io.File;

import com.sodius.mdw.core.CoreException;
import com.sodius.mdw.core.eval.debug.DebugHierarchy;
import com.sodius.mdw.core.eval.launch.LaunchConfiguration;
import com.sodius.mdw.core.eval.project.Project;
import com.sodius.mdw.core.eval.report.Report;
import com.sodius.mdw.core.util.io.ContentWriter;
import com.sodius.mdw.core.util.progress.ProgressMonitor;

/**
 * Provides entry-point services to launch an evaluation.
 * 
 * <p>Here is a typical use of the evaluation manager :
 * <ul>
 * <li>load a launch configuration using {@link #loadLaunchConfiguration(File)},</li>
 * <li>evaluate this configuration using  {@link #evaluate(LaunchConfiguration, ProgressMonitor)}.</li>
 * </ul>
 * </p>
 * 
 * <p>If a specific project has to be evaluated more than one time in different configurations, 
 * or if the templates to be evaluated can be selected dynamically, 
 * you can have a finer control on the evaluation process. Here are the steps :
 * <ul>
 * <li>load a project using {@link #loadProject(File)},</li>
 * <li>eventually inspect the content of this project to select some templates to evaluate,</li>
 * <li>instantiate an <code>Evaluable</code> subclass, which describes what is to be evaluated,</li>
 * <li>launch the evaluation using {@link #evaluate(Project, Evaluable, ProgressMonitor)}.</li>
 * </ul></p>
 * 
 * <p>An evaluation process produces an evaluation result, which contains :
 * <ul>
 * <li>An evaluation report : describes the list of generated files and handled models.</li>
 * <li>A debug hierarchy : describes each evaluated rule and script.</li>
 * </ul>
 * </p>
 * 
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 * 
 * @see com.sodius.mdw.core.MDWorkbench#getEvaluationManager()
 */
public interface EvaluationManager {
    
    /**
     * Loads the specified project.
     * 
     * @param file the file containing a project definition.
     * @return the loaded project.
     * @throws CoreException if the file could not be read or if the project contains errors.
     */
    public Project loadProject(File file) throws CoreException;
    
    /**
     * Loads the specified evaluation report.
     * 
     * @param file the file containing a report definition.
     * @return the loaded report.
     * @throws CoreException if the file could not be read.
     */
    public Report loadReport(File file) throws CoreException;
    
    /**
     * Loads the specified launch configuration. 
     * 
     * @param file the file containing a launch configuration.
     * @return the loaded report.
     * @throws CoreException if the file could not be read.
     */
    public LaunchConfiguration loadLaunchConfiguration(File file) throws CoreException;
    
    /**
     * Loads the specified debug hierarchy. 
     * 
     * @param file the file containing a debug hierarchy.
     * @return the loaded debug hierarchy.
     * @throws CoreException if the file could not be read.
     */
    public DebugHierarchy loadDebugHierarchy(File file) throws CoreException;
    
    /**
     * Launches an evaluation based on the specified launch configuration.
     * First the project referenced in the configuration is loaded and 
     * then the evaluation process of the referenced text template or main rule is started. 
     * 
     * <p>This is a long running operation that can be monitored through the <code>monitor</code> parameter.
     * A progress monitor may be used to report progress information and 
     * to interrupt the evaluation process.</p>
     * 
     * @param configuration the configuration which describes the launch parameters.
     * @param monitor the progress monitor used to report progress information and check for cancelation, maybe <code>null</code>.
     * @return an evaluation result, containing a report and a debug hierarchy.
     * @throws CoreException if the evaluation fails.
     */
    public EvaluationResult evaluate(LaunchConfiguration configuration, ProgressMonitor monitor) throws CoreException;
    
    /**
     * Launches an evaluation based on the specified project.
     * The <code>Evaluable</code> imlementation has the full control on what is to be evaluated. 
     * 
     * <p>This is a long running operation that can be monitored through the <code>monitor</code> parameter.
     * A progress monitor may be used to report progress information and 
     * to interrupt the evaluation process.</p>
     * 
     * @param project the project which describes the available templates and scripts.
     * @param evaluable the object whose <code>evaluate</code> method is called.
     * @param monitor the progress monitor used to report progress information and check for cancelation, maybe <code>null</code>.
     * @return an evaluation result, containing a report and a debug hierarchy.
     * @throws CoreException if the evaluation fails.
     */
    public EvaluationResult evaluate(Project project, Evaluable evaluable, ProgressMonitor monitor) throws CoreException;
    
    /**
     * Returns the registered content writer, used to write on disk a text template output.
     * 
     * @return the content writer
     */
    public ContentWriter getContentWriter();
    
    /**
     * Registers a content writer, used to write on disk a text template output.
     * 
     * @param writer the new content writer to use.
     */
    public void setContentWriter(ContentWriter writer);

}
