package com.sodius.mdw.core.eval;

import com.sodius.mdw.core.eval.debug.DebugHierarchy;
import com.sodius.mdw.core.eval.report.Report;

/**
 * Provides access to the output of an evaluation. 
 * This output consist of :
 * <ul>
 * <li>an evaluation report, which details the generated file and handled models,</li>
 * <li>a debug hierarchy, which keeps track of every evaluated template, rule and script, for debugging purpose.</li>
 * </ul>
 * 
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 */
public interface EvaluationResult {
    
    /**
     * Returns the evaluation report, which details the generated file and handled models.
     * 
     * @return the evaluation report.
     */
    public Report getReport();
    
    /**
     * Returns <code>true</code> if a debug hierarchy has been created during the evaluation.
     * A debug hierarchy is computed only if requested in the evaluation options.
     * 
     * @return <code>true</code> if a debug hierarchy has been created during the evaluation, <code>false</code> otherwise.
     * @see EvaluationConfiguration#isDebugHierarchyEnabled()
     */
    public boolean isDebugHierarchyEnabled();
    
    /**
     * Returns the created debug hierarchy, or <code>null</code> if not enabled.
     * 
     * <p>A debug hierarchy keeps track of every evaluated template, rule and script, for debugging purpose.
     * This lets the developer navigate in the evaluation tree once the evaluation is completed.</p>
     * 
     * <p>A debug hierarchy comes at a considerable cost in both time and space,
     * and should not be requested frivolously. The additional space is not reclaimed until the 
     * debug hierarchy and all its nodes become garbage. So it is very
     * important to not retain any of these objects longer than absolutely necessary </p>
     * 
     * @return the created debug hierarchy, or <code>null</code> if not enabled.
     * @see #isDebugHierarchyEnabled()
     */
    public DebugHierarchy getDebugHierarchy();

}
