package com.sodius.mdw.core.eval;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.List;

import org.xml.sax.InputSource;

import com.sodius.mdw.core.util.io.InputSourceProvider;

/**
 * Describes the evaluation output of a text template.
 * The output of a text template has the following properties :
 * <ul>
 * <li>The file path (may be a relative path), as specified by the file property of the template.</li>
 * <li>The encoding to use (<code>null</code> most of time).</li>
 * <li>The generated contents.</li>
 * <li>The protected sections, which define regions in a file that can contain end-user manual code
 * and that the generator must never overwrite.</li>
 * </ul>
 * 
 * <p>A generated code is created when a text template is evaluated, 
 * and its contents can be persistent using the {@link #write()} method.</p>
 * 
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 * 
 * @see EvaluationContext#generate(String, List)
 */
public interface GeneratedCode {
    
    /**
     * Returns the file path specified in the text template.
     * This file corresponds to the path generated by the text template. 
     * At this time the path is not resolved against the current evaluation directory,
     * so it may be a relative path.
     * This file can be <code>null</code> if no the text template has no file property. 
     * 
     * @return the file path generated by the text template (can be <code>null</code>). 
     */
    public File getFile();
    
    /**
     * Sets the file, which may be relative, where the generated contents should be written out.
     * 
     * @param file the file where to write the generated contents.
     */
    public void setFile(File file);
    
    /**
     * Returns the encoding to use when writing the generated contents.
     * If this encoding is <code>null</code>, which is the general case, 
     * the platform default encoding is used. The encoding can be used for example
     * when generating XML files, since it must match the declaration of the XML header (<code>UTF-8</code> for example).
     * 
     * @return the encoding to use, <code>null</code> if no encoding specified.
     */
    public String getEncoding();
    
    /**
     * Sets the encoding to use when writing the generated contents.
     * If this encoding is <code>null</code>, which is the general case, 
     * the platform default encoding is used. The encoding can be used for example
     * when generating XML files, since it must match the declaration of the XML header (<code>UTF-8</code> for example).
     * 
     * @param encoding the encoding to use, <code>null</code> to use the platform default encoding.
     */
    public void setEncoding(String encoding);
    
    /**
     * Returns the contents generated by the text template.
     * This contents is not merged with the underlying existing file at this time.
     * 
     * @return the generated contents, <code>null</code> if not specified.
     */
    public String getContents();
    
    /**
     * Sets the generated contents.
     * 
     * @param contents the contents generated by the text template.
     */
    public void setContents(String contents);
    
    /**
     * Returns the contents generated by the text template.
     * This contents is not merged with the underlying existing file at this time.
     * 
     * @return the generated contents, <code>null</code> if not specified.
     * @since 3.2.0
     */
    public InputSource getContentSource() throws IOException;
    
    /**
     * Sets the generated contents.
     * 
     * @param contentsProvider the contents generated by the text template.
     * @since 3.2.0
     */
    public void setContents(InputSourceProvider contentsProvider);
    
    /**
     * Adds a protected section. 
     * Start and end tags can be any arbitrary (language specific) strings, including multiline strings (with '\n' characters).
     * A start delimiter must be unique in the scope of the generated file.</p>
     * 
     * @param startTag the start delimiter.
     * @param endTag the end delimiter.
     */
    public void addProtectedSection(String startTag, String endTag);
    
    /**
     * Starts a protected section. 
     * The start can be any arbitrary (language specific) string, including multiline string (with '\n' characters).
     * A start delimiter must be unique in the scope of the generated file.</p>
     * 
     * @param startTag the start delimiter.
     * @see #addEndProtectedSection(String)
     */
    public void addStartProtectedSection(String startTag);
    
    /**
     * Ends a protected section. 
     * The end tag can be any arbitrary (language specific) string, including multiline string (with '\n' characters)</p>
     * 
     * @param endTag the start delimiter.
     * @see #addStartProtectedSection(String)
     */
    public void addEndProtectedSection(String endTag);
    
    /**
     * Adds each protected section of the collection.
     * 
     * @param sections a collection of <code>ProtectedSection</code> elements.
     */
    public void addProtectedSections(Collection<ProtectedSection> sections);
    
    /**
     * Returns the list of protected sections, which define regions in a file that can contain end-user manual code
     * and that the generator must never overwrite.
     * 
     * @return an unmodifiable list of <code>ProtectedSection</code> elements, ordered the way they were added.
     */
    public List<ProtectedSection> getProtectedSections();
    
    /**
     * Writes this generated contents into the underlying file on disk.
     * 
     * <p>
     * Writing a generated code consists of the following steps :
     * <ul>
     * <li>Resolve the relative file path with the current evaluation directory to compute an absolute location.</li>
     * <li>If there are registered protected sections :
     * <ul>
     * <li>Read the existing file and disk and remind the existing end-user manual code in protected sections.
     * <li>Merge these existing end-user manual code with the contents generated by the text template.</li>
     * </ul>
     * </li>
     * <li>Write the generated contents into this file, using the requested encoding, 
     * if the contents is different from the existing file or if the force overwrite option is on.</li>
     * </ul>
     * </p>
     * 
     * <p>Note : template and script designers should generally not have to consider this method.
     * The {@link EvaluationContext#generate(String, List)} method evaluates the template and write it automaticaly on disk.
     * This method is only to be used if the template has been evaluated with the {@link EvaluationContext#createGeneratedCode(String, List)} method.</p>
     * 
     * @see EvaluationContext#generate(String, List)
     * @see EvaluationConfiguration#getEvaluationDirectory()
     * @see EvaluationConfiguration#isFileOverwriteForced()
     * @throws EvaluationException if the generated contents could not be written on disk. Reasons include :
     * <ul>
     * <li>The file is not specified or is invalid.</li>
     * <li>The encoding is not supported</li>
     * <li>An exception occured during the file writing.</li>
     * </ul>
     */
    public void write() throws EvaluationException;
    
    /**
     * Returns <code>true</code> if this generated code is discarded and must not be written.
     * A generated code is discarded when the related text template guard failed.
     * Any attempt to write a discarded generated code would fail.
     * 
     * @return <code>true</code> if the generated code is discarded, <code>false</code> otherwise.
     */
    public boolean isDiscarded();
    
    /**
     * Notifies this generated code it must be discarded.
     * A generated code is discarded when the related text template guard failed.
     */
    public void discard();

}
