package com.sodius.mdw.core.eval;

/**
 * Provides services to create transient links between pairs of elements.
 * A transient link is a named virtual link between two objects of any kind. 
 * These links do not exist before the evaluation and are automatically garbaged at the end of the evaluation process.
 * 
 * <p>Theses links are virtual, have no restriction on the type of objects to connect and do never alter the models, 
 * while modeling link are reserved to model elements and will modify the related objects.</p> 
 * 
 * <p>Transient links are particularly useful to add traceability between a source model element and a target model element.</p>
 * 
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 */
public interface TransientLinkManager {

    /**
     * Removes all registered transient links.
     */
    public void clear();
    
    /**
     * Returns <code>true</code> if this manager contains no transient link.
     * 
     * @return <code>true</code> if this manager contains no transient link, <code>false</code> otherwise.
     */
    public boolean isEmpty();
    
    /**
     * Determines whether there are transient links from the specified source object of the specified name.
     * 
     * @param name the name of the transient link.
     * @param source the source object of the transient link.
     * @return <code>true</code> if the source objects has any transient links of that name, <code>false</code> otherwise.
     */
    public boolean containsKey(String name, Object source);
    
    /**
     * Returns the list a target objects connected to the source object through a transient link of specified name.
     * 
     * <p>The returned list may be empty, when no target object is connected.
     * This list is modifiable, you can add or remove target objects directly from this list.</p>
     * 
     * @param name the name of the transient link.
     * @param source the source object of the transient link.
     * @return a modifiable list of objects connected to the source object through the specified transient link.
     */
    public TransientLinkList<Object> get(String name, Object source);
    
    /**
     * Removes any transient link of the specified name, whatever the source objects are.
     * 
     * @param name the name of the transient links to remove.
     */
    public void remove(String name);
    
    /**
     * Removes any transient link of the specified name with the specified source object.
     * 
     * @param name the name of the transient link to remove.
     * @param source the source object of the transient link.
     */
    public void remove(String name, Object source);
}
