package com.sodius.mdw.core.eval.launch;

import java.io.File;

import com.sodius.mdw.core.CoreException;
import com.sodius.mdw.core.eval.EvaluationConfiguration;

/**
 * A launch configuration defines a complete set of properties to launch
 * a text template or a main rule evaluation.
 * 
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 * 
 * @see com.sodius.mdw.core.eval.EvaluationManager#loadLaunchConfiguration(File)
 * @see com.sodius.mdw.core.eval.EvaluationManager#evaluate(LaunchConfiguration, ProgressMonitor)
 * @see com.sodius.mdw.core.eval.launch.LaunchRunner
 */
public interface LaunchConfiguration extends EvaluationConfiguration {

    /**
     * Returns the launch configuration name, which matches the Eclipse launch configuration.
     * 
     * @return the launch configuration name.
     */
    public String getName();
    
    /**
     * Returns the file where this launch configuration is saved.
     * 
     * @return the file where this launch configuration is saved, <code>null</code> if no saved in a file.
     */
    public File getFile();
    
    /**
     * Sets the launch configuration name.
     * 
     * @param name the launch configuration name.
     */
    public void setName(String name);
    
    /**
     * Returns the location of the file defining the required project classpath.
     * 
     * @return the file defining the project classpath.
     */
    public File getProjectClasspathFile();
    
    /**
     * Returns the referenced entry-point, which can either be a text template or a main rule.
     * 
     * @return the launch entry point.
     */
    public LaunchEntryPoint getEntryPoint();
    
    /**
     * Returns the file where the debug hierarchy created during the evaluation must be persisted.
     * 
     * @return the file where to store the debug hierarchy, <code>null</code> if no debug hierarchy requested.
     */
    public File getDebugHierarchyFile();
    
    /**
     * Sets the file where the debug hierarchy created during the evaluation must be persisted.
     * 
     * <p>If this file is <code>null</code>, then no debug hierarchy is created.
     * Otherwise, setting a file will override the <code>isDebugHierarchyEnabled</code> method
     * to force the workbench to create a debug hierarchy.</p>
     * 
     * @param file the file where to store the debug hierarchy, <code>null</code> if no debug hierarchy requested.
     * @see EvaluationConfiguration#isDebugHierarchyEnabled()
     */
    public void setDebugHierarchyFile(File file);
    
    /**
     * Returns the file where the report created during the evaluation must be persisted.
     * 
     * @return the file where to store the created evaluation report, <code>null</code> if the report must not be saved.
     */
    public File getReportFile();
    
    /**
     * Sets the file where the report created during the evaluation must be persisted.
     * 
     * @param file the file where to store the created evaluation report, <code>null</code> if the report must not be saved. 
     */
    public void setReportFile(File file);
    
    /**
     * Saves this launch configuration into the specified file.
     * 
     * @param file the file where to store this launch configuration.
     * @throws CoreException if the configuration could not be saved.
     */
    public void save(File file) throws CoreException;
}
