package com.sodius.mdw.core.eval.launch;

import com.sodius.mdw.internal.core.eval.launch.LaunchRunnerImpl;

/**
 * A runnable class (with a <code>main</code> method) to launch an evaluation of a launch configuration.
 * 
 * <p>The <code>main</code> method performs the following steps :
 * <ul>
 * <li>Parse the command line to determine the launch configuration file path to use.</li>
 * <li>Create an <code>MDWorkbench</code> instance.</li>
 * <li>Load the specified launch configuration.</li>
 * <li>Launch the evaluation of the configuration.</li>
 * <li>Shutdown the workbench.</li>
 * </ul>
 * </p>
 * 
 * <p>Here is a command line snippet :</p>
 * <p>
 * <code>java -classpath "<...>" com.sodius.mdw.core.eval.launch.LaunchRunner -launch "c:\myProjects\myLaunchFilename.launch"</code>
 * </p>
 * 
 * <p>You can override the evaluation directory to use:</p>
 * <p>
 * <code>java -classpath "<...>" com.sodius.mdw.core.eval.launch.LaunchRunner -launch "c:\myProjects\myLaunchFilename.launch"
 * -evaluationDirectory c:\test\generated</code>
 * </p>
 * 
 * <p>You can also override launch arguments, using a <code>-args[<i>index</i>] <i>value</i></code> notation:</p>
 * <ul>
 * <li><code>index</code> is the index of the launch argument to override</li>
 * <li><code>value</code> is the new value to set for the overriden argument.
 * For a model connector, the value must of the form <code><i>connectorName</i>|<i>uri</i></code>.</li>
 * </ul>
 * 
 * <code>java -classpath "<...>" com.sodius.mdw.core.eval.launch.LaunchRunner -launch "c:\myProjects\myLaunchFilename.launch"
 * -args[0] "XMI|c:\models\myModel.xmi" -args[2] "Hello World" -args[3] true</code>
 * </p>
 * <p>
 * This class cannot be instantiated or subclassed by clients.
 * It is just intended to run in command line mode.
 * </p>
 */
public final class LaunchRunner {
    
    /**
     * The argument name which references the application ID to use.
     * @see com.sodius.mdw.core.MDWorkbenchFactory#create(String)
     */
    public static final String ARG_APPLICATION_ID = "applicationID"; //$NON-NLS-1$
    
    /**
     * The argument name which references the file path which contains a launch configuration.
     */
    public static final String ARG_LAUNCH = "launch"; //$NON-NLS-1$
    
    /**
     * The argument name which references the directory where to generate files 
     * (overrides the one specified in the launch configuration).
     * 
     * @since 2.1.0
     */
    public static final String ARG_EVALUATION_DIRECTORY = "evaluationDirectory"; //$NON-NLS-1$
    
    /**
     * The prefix of the argument name to override a launch parameter.
     * (overrides the one specified in the launch configuration).
     * 
     * @since 2.1.0
     */
    public static final String ARG_LAUNCH_ARG_PREFIX = "args["; //$NON-NLS-1$
    
    /**
     * The suffix of the argument name to override a launch parameter.
     * 
     * @since 2.1.0
     */
    public static final String ARG_LAUNCH_ARG_SUFFIX = "]"; //$NON-NLS-1$
    
    /**
     * The character to separate the model reader from the uri (<code>"XMI|c:\models\file.xmi"</code> for example).
     * 
     * @since 2.1.0
     */
    public static final char ARG_LAUNCH_MODEL_SEPARATOR = '|';

    /**
     * Launches an evaluation of a launch configuration.
     * 
     * <p>This method performs the following steps :
     * <ul>
     * <li>Parse the command line to determine the launch configuration file path to use.</li>
     * <li>Create a <code>MDWorkbench</code> instance.</li>
     * <li>Load the specified launch configuration.</li>
     * <li>Launch the evaluation of the configuration.</li>
     * <li>Shutdown the workbench.</li>
     * </ul>
     * </p>
     * 
     * <p>Here is a command line snippet :</p>
     * <p>
     * <code>java -classpath "<...>" com.sodius.mdw.core.eval.launch.LaunchRunner -launch "c:\myProjects\myLaunchFilename.launch"</code>
     * </p>
     */
    public static void main(String[] args) {
        LaunchRunnerImpl.main(args);
    }
    
    private LaunchRunner() {
    }

}
