package com.sodius.mdw.core.eval.project;

/**
 * Enumerations of known model parameter direction kinds.
 * 
 * <p>
 * This class cannot be instantiated or subclassed by clients.
 * </p>
 * 
 * @see com.sodius.mdw.core.eval.project.ModelParameter#getDirection()
 */
public final class DirectionKind {

    /**
     * in : elements are read in the input model, no modification performed. 
     * For an entry-point rule set, this will tell the workbench a model needs to be loaded from a source location.
     */
    public static final DirectionKind IN = new DirectionKind("in"); //$NON-NLS-1$

    /**
     * out : elements are created in the output model. 
     * For an entry-point rule set, this will tell the workbench a model may be written into a target location.
     */
    public static final DirectionKind OUT = new DirectionKind("out"); //$NON-NLS-1$
    
    /**
     * inout : some elements are read and others are created in the model.
     * For an entry-point rule set, this will tell the workbench the model needs to be loaded from a resource and saved to a target location,
     * (which may or may not be the same physical resource).
     */
    public static final DirectionKind INOUT = new DirectionKind("inout"); //$NON-NLS-1$
    
    /**
     * Determines the direction represented as a <code>String</code> value.
     * 
     * @param value a <code>String</code> representing a direction.
     * @return the corresponding direction.
     * @throws IllegalArgumentException if the value is not recognized.
     */
    public static DirectionKind valueOf(String value) throws IllegalArgumentException {
        if (value.equals(IN.name))
            return IN;
        else if (value.equals(OUT.name))
            return OUT;
        else if (value.equals(INOUT.name))
            return INOUT;
        else
            throw new IllegalArgumentException(value);
    }
    
    private String name;
    
    private DirectionKind(String name) {
        this.name = name;
    }
    
    /**
     * Return <code>true</code> if the direction is <code>in</code>.
     * 
     * @return <code>true</code> if the direction is <code>in</code>.
     * @see #IN
     */
    public boolean isIn() {
        return this == IN;
    }
    
    /**
     * Return <code>true</code> if the direction is <code>out</code>.
     * 
     * @return <code>true</code> if the direction is <code>out</code>.
     * @see #OUT
     */
    public boolean isOut() {
        return this == OUT;
    }
    
    /**
     * Return <code>true</code> if the direction is <code>inout</code>.
     * 
     * @return <code>true</code> if the direction is <code>inout</code>.
     * @see #INOUT
     */
    public boolean isInOut() {
        return this == INOUT;
    }
    
    /**
     * Returns a <code>String</code> representation of this direction.
     * 
     * @return the direction as a <code>String</code>.
     */
    public String toString() {
        return name;
    }
}
