package com.sodius.mdw.core.eval.project;

import java.util.Collection;

import com.sodius.mdw.core.model.Metamodel;

/**
 * Describes a type on which scripts are defined.
 * 
 * <p>A type is defined in a metamodel and may correspond to a profile element.</p>
 * 
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 * 
 * @see Project#getTypes()
 * @see Script#getType()
 */
public interface Type {

    /**
     * The metamodel from which this type originates.
     * 
     * @return the type metamodel.
     */
    public Metamodel getMetamodel();
    
    /**
     * Returns the simple name of this type (<code>"Classifier"</code>, or <code>"Class:EJB"</code> for example).
     * 
     * @return the type simple name.
     * @see #getInstanceClass()
     */
    public String getName();
    
    /**
     * Returns the Java instance class that this type represents (<code>com.sodius.mdw.metamodel.uml21.Classifier</code> for example).
     * 
     * @return the Java instance class that this type represents.
     */
    public Class<?> getInstanceClass();
    
    /**
     * Determines whether this type is defined in a profile.
     * 
     * @return <code>true</code> if this type comes from a profile, <code>false</code> otherwise.
     */
    public boolean isProfileType();
    
    /**
     * Returns the profile name part of this type (<code>"EJB"</code> for example).
     * 
     * @return the profile name part of this type name, <code>null</code> if this is not a profile type.
     * @see #getName()
     * @see #isProfileType()
     */
    public String getProfileName();
    
    /**
     * Determines whether this is a root type, i.e. if there is no script defined 
     * on super-types in the scope of this project.
     * 
     * @return <code>true</code> if there are no super-types which define scripts in this project, <code>false</code> otherwise.
     */
    public boolean isRoot();
    
    /**
     * Returns a collection of super-types which define at least one script in the project scope.
     * 
     * @return an unmodifiable collection of <code>Type</code> elements.
     */
    public Collection<Type> getSuperTypes();
    
    /**
     * Determines whether this is a leaf type, i.e. if there is no script defined 
     * on sub-types in the scope of this project.
     * 
     * @return <code>true</code> if there are no sub-types which define scripts in this project, <code>false</code> otherwise.
     */
    public boolean isLeaf();
    
    /**
     * Returns a collection of sub-types which define at least one script in the project scope.
     * 
     * @return an unmodifiable collection of <code>Type</code> elements.
     */
    public Collection<Type> getSubTypes();
    
    /**
     * Returns the scripts defined on this type in the project scope.
     * 
     * @return an unmodifiable collection of <code>Script</code> elements defined on this type.
     */
    public Collection<Script> getScripts();
    
    /**
     * Returns all scripts defined on the super-type hierarchy of this type in the project scope.
     * The returned collection is not visibility filtered.
     * 
     * @return an unmodifiable collection of <code>Script</code> elements defined on this type or on one of its super-type.
     */
    public Collection<Script> getAllScripts();
}
