package com.sodius.mdw.core.eval.project;

/**
 * Enumerations of known rule and script visibilities.
 * 
 * <p>
 * This class cannot be instantiated or subclassed by clients.
 * </p>
 * 
 * @see Rule#getVisibility()
 * @see Script#getVisibility()
 */
public final class VisibilityKind {

    /**
     * entry : the rule is available to the end-user (only valid for rules). 
     */
    public static final VisibilityKind ENTRY_POINT = new VisibilityKind("entry", 5); //$NON-NLS-1$

    /**
     * public : the script is visible from everywhere, 
     * the rule is visible from everywhere but should not be presented to the end-user (valid for rules and scripts). 
     */
    public static final VisibilityKind PUBLIC = new VisibilityKind("public", 4); //$NON-NLS-1$
    
    /**
     * protected : the element is visible from the package in which the element is defined
     * and on sub types of the script (valid for rules and scripts).
     */
    public static final VisibilityKind PROTECTED = new VisibilityKind("protected", 3); //$NON-NLS-1$
    
    /**
     * package : the element is visible from the package in which the element is defined (valid for rules and scripts).
     */
    public static final VisibilityKind PACKAGE = new VisibilityKind("package", 2); //$NON-NLS-1$
    
    /**
     * private : the element is not visible outside of the resource (valid for rules and scripts).
     */
    public static final VisibilityKind PRIVATE = new VisibilityKind("private", 1); //$NON-NLS-1$
    
    /**
     * Determines the visibility represented as a <code>String</code> value.
     * 
     * @param value a <code>String</code> representing a visibility.
     * @return the corresponding visibility.
     * @throws IllegalArgumentException if the value is not recognized.
     */
    public static VisibilityKind valueOf(String value) throws IllegalArgumentException {
        if (value == null || value.length() == 0)
             return PACKAGE;
        else if (value.equals(ENTRY_POINT.name))
            return ENTRY_POINT;
        else if (value.equals(PUBLIC.name))
            return PUBLIC;
        else if (value.equals(PROTECTED.name))
            return PROTECTED;
        else if (value.equals(PRIVATE.name))
            return PRIVATE;
        else
            throw new IllegalArgumentException(value);
    }
    
    private String name;
    private int level;
    
    private VisibilityKind(String name, int level) {
        this.name = name;
        this.level = level;
    }
    
    /**
     * Return <code>true</code> if the visibility is <code>entry</code>.
     * 
     * @return <code>true</code> if the visibility is <code>entry</code>.
     * @see #ENTRY_POINT
     */
    public boolean isEntryPoint() {
        return this == ENTRY_POINT;
    }
    
    /**
     * Return <code>true</code> if the visibility is <code>public</code>.
     * 
     * @return <code>true</code> if the visibility is <code>public</code>.
     * @see #PUBLIC
     */
    public boolean isPublic() {
        return this == PUBLIC;
    }
    
    /**
     * Return <code>true</code> if the visibility is <code>protected</code>.
     * 
     * @return <code>true</code> if the visibility is <code>protected</code>.
     * @see #PROTECTED
     */
    public boolean isProtected() {
        return this == PROTECTED;
    }
    
    /**
     * Return <code>true</code> if the visibility is <code>package</code>.
     * 
     * @return <code>true</code> if the visibility is <code>package</code>.
     * @see #PACKAGE
     */
    public boolean isPackage() {
        return this == PACKAGE;
    }
    
    /**
     * Return <code>true</code> if the visibility is <code>private</code>.
     * 
     * @return <code>true</code> if the visibility is <code>private</code>.
     * @see #PRIVATE
     */
    public boolean isPrivate() {
        return this == PRIVATE;
    }
    
    /**
     * <p>Compares this visibility with the specified visibility for order.  Returns a
     * negative integer, zero, or a positive integer as this visibility is less
     * than, equal to, or greater than the specified visibility.</p>
     * 
     * <p>The lowest visibility is <code>PRIVATE</code>, the highest is <code>ENTRY_POINT</code></p>
     * 
     * @param visibility the visibility to compare with.
     * @return a negative integer, zero, or a positive integer as this visibility is less
     * than, equal to, or greater than the specified visibility.
     */
    public int compareTo(VisibilityKind visibility) {
        if (level == visibility.level)
            return 0;
        else if (level < visibility.level)
            return -1;
        else
            return 1;
    }
    
    /**
     * Returns a <code>String</code> representation of this visibility.
     * 
     * @return the visibility as a <code>String</code>.
     */
    public String toString() {
        return name;
    }
}
