package com.sodius.mdw.core.model;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;

/**
 * Provides access to structural features, like <code>EClass</code> does, but merges the features of the base <code>EClass</code> 
 * with the ones defined on applied metamodel extensions (if any).
 * 
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 * 
 * @see MDWObject#eExtendedClass()
 * @see MetamodelExtension
 * @since MDWorkbench 3.0.0
 */
public interface ExtendedClass {

    /**
     * Returns the metamodel this type is defined in.
     * 
     * @return the metamodel.
     */
    public Metamodel getMetamodel();
    
    /**
     * Returns the <code>EClass</code> this type is based on.
     * 
     * @return the <code>EClass</code> this type is based on.
     */
    public EClass getEClass();
    
    /**
     * <p>Returns the feature with the specified name.</p> 
     * <p>This method will lookup features defined in the base <code>EClass</code>
     * and in the applied metamodel extensions, if any.</p>
     * 
     * @param featureName the name of the feature to look for
     * @return the feature matching the specified name, <code>null</code> if none.
     */
    public EStructuralFeature getEStructuralFeature(String featureName);
    
    /**
     * Returns the features defined in the base <code>EClass</code> and in the applied metamodel extensions.
     * 
     * @return features defined in the base <code>EClass</code> and in the applied metamodel extensions.
     */
    public EList<EStructuralFeature> getEStructuralFeatures();
    
    /**
     * Returns the features defined in the base <code>EClass</code> (and its supertypes) and in the applied metamodel extensions.
     * 
     * @return features defined in the base <code>EClass</code> (and its supertypes) and in the applied metamodel extensions.
     */
    public EList<EStructuralFeature> getEAllStructuralFeatures();
    
    /**
     * Returns the attributes defined in the base <code>EClass</code> and in the applied metamodel extensions.
     * 
     * @return attributes defined in the base <code>EClass</code> and in the applied metamodel extensions.
     */
    public EList<EAttribute> getEAttributes();
    
    /**
     * Returns the attributes defined in the base <code>EClass</code> (and its supertypes) and in the applied metamodel extensions.
     * 
     * @return attributes defined in the base <code>EClass</code> (and its supertypes) and in the applied metamodel extensions.
     */
    public EList<EAttribute> getEAllAttributes();
    
    /**
     * Returns the references defined in the base <code>EClass</code> and in the applied metamodel extensions.
     * 
     * @return references defined in the base <code>EClass</code> and in the applied metamodel extensions.
     */
    public EList<EReference> getEReferences();
    
    /**
     * Returns the references defined in the base <code>EClass</code> (and its supertypes) and in the applied metamodel extensions.
     * 
     * @return references defined in the base <code>EClass</code> (and its supertypes) and in the applied metamodel extensions.
     */
    public EList<EReference> getEAllReferences();
    
    /**
     * Returns the containment references defined in the base <code>EClass</code> (and its supertypes) and in the applied metamodel extensions.
     * 
     * @return containment references defined in the base <code>EClass</code> (and its supertypes) and in the applied metamodel extensions.
     */
    public EList<EReference> getEAllContainments();
}
