package com.sodius.mdw.core.model;

import java.util.Collection;
import java.util.List;

import org.eclipse.emf.ecore.EStructuralFeature;

/**
 * Provides facilities to invoke scripts on each <code>MDWObject</code> element of this collection.
 * <code>MDWCollection</code> enables to:
 * <ul>
 * <li>concatenate script results into a <code>String</code> using {@link #concat(String scriptName)}</li>
 * <li>detect the first element that matches a condition using {@link #detect(String scriptName, Object value)}</li>
 * <li>sort <code>MDWObject</code> elements of this collection using {@link #sort(String scriptName)}</li>
 * </ul>
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 * @since MDWorkbench 3.0.0
 */

public interface MDWCollection<E> extends Collection<E> {

    /**
     * Evaluates the specified script on each <code>MDWObject</code> of this collection, 
     * and concatenates the results into a <code>String</code>.
     * 
     * <p>This collection is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     *      This script must not expect any parameter.
     * @return the concatenation the of script results. 
     * @throws ClassCastException if the collection contains anything else than <code>MDWObject</code> instances.
     */
    public String concat(String scriptName);
    
    /**
     * Evaluates the specified script on each <code>MDWObject</code> of this collection, 
     * and concatenates the results into a <code>String</code>.
     * 
     * <p>This collection is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * <p>The separator is inserted between each concatenated (non empty) script result.</p> 
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     *      This script must not expect any parameter.
     * @param separator a separator to insert between each concatenated script result
     * @return the concatenation the of script results. 
     * @throws ClassCastException if the collection contains anything else than <code>MDWObject</code> instances.
     */
    public String concat(String scriptName, String separator);
    
    /**
     * Evaluates the specified script on each <code>MDWObject</code> of this collection, 
     * and concatenates the results into a <code>String</code>.
     * 
     * <p>This collection is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     * @param arguments the script arguments.
     * @return the concatenation the of script results. 
     * @throws ClassCastException if the collection contains anything else than <code>MDWObject</code> instances.
     */
    public String concat(String scriptName, List<?> arguments);
    
    /**
     * Evaluates the specified script on each <code>MDWObject</code> of this collection, 
     * and concatenates the results into a <code>String</code>.
     * 
     * <p>This collection is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * <p>The separator is inserted between each concatenated (non empty) script result.</p> 
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     * @param arguments the script arguments.
     * @return the concatenation the of script results. 
     * @throws ClassCastException if the collection contains anything else than <code>MDWObject</code> instances.
     */
    public String concat(String scriptName, List<?> arguments, String separator);
    
    /**
     * Evaluates the specified feature on each <code>MDWObject</code> of this collection, 
     * and concatenates the results into a <code>String</code>.
     * 
     * <p>This collection is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * @param feature the feature to evaluate on each <code>MDWObject</code>.
     * @return the concatenation the of feature results. 
     * @throws ClassCastException if the collection contains anything else than <code>MDWObject</code> instances.
     */
    public String concat(EStructuralFeature feature);
    
    /**
     * Evaluates the specified feature on each <code>MDWObject</code> of this collection, 
     * and concatenates the results into a <code>String</code>.
     * 
     * <p>This collection is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * <p>The separator is inserted between each concatenated (non empty) feature result.</p> 
     * 
     * @param feature the feature to evaluate on each <code>MDWObject</code>.
     * @param separator a separator to insert between each concatenated script result
     * @return the concatenation the of feature results. 
     * @throws ClassCastException if the collection contains anything else than <code>MDWObject</code> instances.
     */
    public String concat(EStructuralFeature feature, String separator);
    
    /**
     * Returns the first <code>MDWObject</code> of this collection whose script evaluation result matches the specified value.
     * 
     * <p>This collection is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     *      This script must not expect any parameter.
     * @param value the comparison criterium (can be <code>null</code>).
     * @return the first <code>MDWObject</code> instances whose script result matches the specified value,
     *      <code>null</code> if no instance matches the value.
     * @throws ClassCastException if the collection contains anything else than <code>MDWObject</code> instances.
     */
    public <T extends MDWObject> T detect(String scriptName, Object value);

    /**
     * Returns the first <code>MDWObject</code> of this collection whose script evaluation result matches the specified value.
     * 
     * <p>This collection is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     * @param arguments the script arguments.
     * @return the first <code>MDWObject</code> instances whose script result matches the specified value, 
     *      <code>null</code> if no instance matches the value.
     * @throws ClassCastException if the collection contains anything else than <code>MDWObject</code> instances.
     */
    public <T extends MDWObject> T detect(String scriptName, List<?> arguments, Object value);
    
    /**
     * Returns the first <code>MDWObject</code> of this collection whose feature matches the specified value.
     * 
     * <p>This collection is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * @param feature the feature to evaluate on each <code>MDWObject</code>.
     * @param value the comparison criterium (can be <code>null</code>).
     * @return the first <code>MDWObject</code> instances whose feature matches the specified value,
     *      <code>null</code> if no instance matches the value.
     * @throws ClassCastException if the collection contains anything else than <code>MDWObject</code> instances.
     */
    public <T extends MDWObject> T detect(EStructuralFeature feature, Object value);

    /**
     * Evaluates the specified script on each <code>MDWObject</code> of this collection, 
     * and concatenates the results into a new list.
     * 
     * <p>The returned list is always flattened: if a script result is a collection 
     * (<code>Collection</code>, <code>Map</code>, <code>Iterator</code> or <code>array</code>),
     * we iterate on this collection and add each element into the returned list.</p>
     * 
     * <p>This collection is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     *      This script must not expect any parameter.
     * @return a modifiable list containing each script result. 
     * @throws ClassCastException if the collection contains anything else than <code>MDWObject</code> instances.
     */
    public <T> MDWList<T> collect(String scriptName); 
    
    /**
     * Evaluates the specified script on each <code>MDWObject</code> of this collection, 
     * and concatenates the results into a new list.
     * 
     * <p>The returned list is always flattened: if a script result is a collection 
     * (<code>Collection</code>, <code>Map</code>, <code>Iterator</code> or <code>array</code>),
     * we iterate on this collection and add each element into the returned list.</p>
     * 
     * <p>This collection is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     * @param arguments the script arguments.
     * @return a modifiable list containing each script result. 
     * @throws ClassCastException if the collection contains anything else than <code>MDWObject</code> instances.
     */
    public <T> MDWList<T> collect(String scriptName, List<?> arguments); 
    
    /**
     * Evaluates the specified feature on each <code>MDWObject</code> of this collection, 
     * and concatenates the results into a new list.
     * 
     * <p>The returned list is always flattened: if a feature is a collection,
     * we iterate on this collection and add each element into the returned list.</p>
     * 
     * <p>This collection is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * @param feature the feature to evaluate on each <code>MDWObject</code>.
     * @return a modifiable list containing each feature result. 
     * @throws ClassCastException if the collection contains anything else than <code>MDWObject</code> instances.
     */
    public <T> MDWList<T> collect(EStructuralFeature feature); 
    
    /**
     * Returns a sorted list into ascending order induced by the specified script.
     * 
     * <p>The script is evaluated on each <code>MDWObject</code> of this collection, 
     * the script results are then used to compare elements.
     * This sort is guaranteed to be <i>stable</i>: equal elements will not be reordered as a result of the sort.</p>
     * 
     * <p>This collection is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     *      This script must not expect any parameter.
     * @return a modifiable list containing each script result. 
     * @throws ClassCastException if the collection contains anything else than <code>MDWObject</code> instances,
     *      or if the script results that are not mutually comparable (for example, strings and integers).
     */
    public MDWList<E> sort(String scriptName); 
    
    /**
     * Returns a sorted list into ascending order induced by the specified script.
     * 
     * <p>The script is evaluated on each <code>MDWObject</code> of this collection, 
     * the script results are then used to compare elements.
     * This sort is guaranteed to be <i>stable</i>: equal elements will not be reordered as a result of the sort.</p>
     * 
     * <p>This collection is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     * @param arguments the script arguments.
     * @return a modifiable list containing each script result. 
     * @throws ClassCastException if the collection contains anything else than <code>MDWObject</code> instances,
     *      or if the script results that are not mutually comparable (for example, strings and integers).
     */
    public MDWList<E> sort(String scriptName, List<?> arguments); 
    
    /**
     * Returns a sorted list into ascending order induced by the specified feature.
     * 
     * <p>The feature is evaluated on each <code>MDWObject</code> of this collection, 
     * the feature results are then used to compare elements.
     * This sort is guaranteed to be <i>stable</i>: equal elements will not be reordered as a result of the sort.</p>
     * 
     * <p>This collection is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * @param feature the feature to evaluate on each <code>MDWObject</code>.
     * @return a modifiable list containing each feature result. 
     * @throws ClassCastException if the collection contains anything else than <code>MDWObject</code> instances.
     */
    public MDWList<E> sort(EStructuralFeature feature); 
    
}
