package com.sodius.mdw.core.model;

import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;

/**
 * Provides facilities to invoke scripts on each <code>MDWObject</code> element of this list.
 * <code>MDWList</code> enables to:
 * <ul>
 * <li>concatenate script results:
 * <ul>
 * <li>into a <code>String</code> using {@link #concat(String scriptName)}</li>
 * <li>into a <code>MDWList</code> using {@link #collect(String scriptName)}</li>
 * </ul>
 * <br>
 * <li>filter <code>MDWObject</code> elements of this list based on script results:
 * <ul>
 * <li>select all elements that matches a condition using {@link #select(String scriptName, Object value)}</li>
 * <li>detect the first element that matches a condition using {@link #detect(String scriptName, Object value)}</li>
 * <li>reject all elements that matches a condition using {@link #reject(String scriptName, Object value)}</li>
 * </ul>
 * <br>
 * <li>filter <code>MDWObject</code> elements of this list based on metatype using {@link #getInstances(String typeName)}
 * <br>
 * <li>sort <code>MDWObject</code> elements of this list using {@link #sort(String scriptName)}</li>
 * </ul>
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 * @see com.sodius.mdw.core.model.DefaultMDWList
 * </p>
 */

public interface MDWList<E> extends EList<E>, MDWCollection<E> {

    /** 
     * Returns the first element of the list, or <code>null</code> if empty. 
     * 
     * @return the first element, <code>null</code> if the list is empty.
     */
    public E first();
    
    /** 
     * Returns the last element of the list, or <code>null</code> if empty. 
     * 
     * @return the last element, <code>null</code> if the list is empty.
     */
    public E last();
    
    /**
     * Copies all of the elements of the receiver list to a newly created one.
     * 
     * <p>This method is useful when you want to traverse a list and modify its contents:
     * you can create a copy of the original list, loop on its elements, and change the contents of the original list 
     * without throwing <code>ConcurrentModificationException</code>s.</p>
     * 
     * @return a new list containining all receiver's elements.
     */
    public MDWList<E> copy();
    
    /**
     * Creates a new list containing the receiver's elements in a reverse order.
     * 
     * @return a new list containining all receiver's elements in a reverse order.
     */
    public MDWList<E> reverse();
    
    /**
     * Evaluates the specified script on each <code>MDWObject</code> of this list, 
     * and concatenates the results into a <code>String</code>.
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     *      This script must not expect any parameter.
     * @return the concatenation the of script results. 
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     */
    public String concat(String scriptName);
    
    /**
     * Evaluates the specified script on each <code>MDWObject</code> of this list, 
     * and concatenates the results into a <code>String</code>.
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * <p>The separator is inserted between each concatenated (non empty) script result.</p> 
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     *      This script must not expect any parameter.
     * @param separator a separator to insert between each concatenated script result
     * @return the concatenation the of script results. 
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     */
    public String concat(String scriptName, String separator);
    
    /**
     * Evaluates the specified script on each <code>MDWObject</code> of this list, 
     * and concatenates the results into a <code>String</code>.
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     * @param arguments the script arguments.
     * @return the concatenation the of script results. 
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     */
    public String concat(String scriptName, List<?> arguments);
    
    /**
     * Evaluates the specified script on each <code>MDWObject</code> of this list, 
     * and concatenates the results into a <code>String</code>.
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * <p>The separator is inserted between each concatenated (non empty) script result.</p> 
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     * @param arguments the script arguments.
     * @return the concatenation the of script results. 
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     */
    public String concat(String scriptName, List<?> arguments, String separator);
    
    /**
     * Evaluates the specified feature on each <code>MDWObject</code> of this list, 
     * and concatenates the results into a <code>String</code>.
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * @param feature the feature to evaluate on each <code>MDWObject</code>.
     * @return the concatenation the of feature results. 
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     * @since MDWorkbench 3.0.0
     */
    public String concat(EStructuralFeature feature);
    
    /**
     * Evaluates the specified feature on each <code>MDWObject</code> of this list, 
     * and concatenates the results into a <code>String</code>.
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * <p>The separator is inserted between each concatenated (non empty) feature result.</p> 
     * 
     * @param feature the feature to evaluate on each <code>MDWObject</code>.
     * @param separator a separator to insert between each concatenated script result
     * @return the concatenation the of feature results. 
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     * @since MDWorkbench 3.0.0
     */
    public String concat(EStructuralFeature feature, String separator);
    
    /**
     * Selects <code>MDWObject</code> instances of this list whose script evaluation result matches the specified value.
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     *      This script must not expect any parameter.
     * @param value the comparison criterium (can be <code>null</code>).
     * @return a modifiable list containing <code>MDWObject</code> instances whose script result matches the specified value. 
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     * @see #detect(String, Object)
     * @see #reject(String, Object)
     */
    public MDWList<E> select(String scriptName, Object value); 
    
    /**
     * Selects <code>MDWObject</code> instances of this list whose script evaluation result matches the specified value.
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     * @param arguments the script arguments.
     * @param value the comparison criterium (can be <code>null</code>).
     * @return a modifiable list containing <code>MDWObject</code> instances whose script result matches the specified value. 
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     * @see #detect(String, List, Object)
     * @see #reject(String, List, Object)
     */
    public MDWList<E> select(String scriptName, List<?> arguments, Object value); 
    
    /**
     * Selects <code>MDWObject</code> instances of this list whose feature matches the specified value.
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * @param feature the feature to evaluate on each <code>MDWObject</code>.
     * @param value the comparison criterium (can be <code>null</code>).
     * @return a modifiable list containing <code>MDWObject</code> instances whose feature matches the specified value. 
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     * @see #detect(EStructuralFeature, Object)
     * @see #reject(EStructuralFeature, Object)
     * @since MDWorkbench 3.0.0
     */
    public MDWList<E> select(EStructuralFeature feature, Object value); 
    
    /**
     * Rejects <code>MDWObject</code> instances of this list whose script evaluation result does not match the specified value.
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     *      This script must not expect any parameter.
     * @param value the comparison criterium (can be <code>null</code>).
     * @return a modifiable list containing <code>MDWObject</code> instances whose script result does not match the specified value. 
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     * @see #select(String, Object)
     */
    public MDWList<E> reject(String scriptName, Object value); 
    
    /**
     * Rejects <code>MDWObject</code> instances of this list whose script evaluation result does not match the specified value.
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     * @param arguments the script arguments.
     * @return a modifiable list containing <code>MDWObject</code> instances whose script result does not match the specified value. 
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     * @see #select(String, List, Object)
     */
    public MDWList<E> reject(String scriptName, List<?> arguments, Object value);
    
    /**
     * Rejects <code>MDWObject</code> instances of this list whose feature does not match the specified value.
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * @param feature the feature to evaluate on each <code>MDWObject</code>.
     * @param value the comparison criterium (can be <code>null</code>).
     * @return a modifiable list containing <code>MDWObject</code> instances whose feature does not match the specified value. 
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     * @see #select(EStructuralFeature, Object)
     * @since MDWorkbench 3.0.0
     */
    public MDWList<E> reject(EStructuralFeature feature, Object value); 
    
    /**
     * Returns the first <code>MDWObject</code> of this list whose script evaluation result matches the specified value.
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     *      This script must not expect any parameter.
     * @param value the comparison criterium (can be <code>null</code>).
     * @return the first <code>MDWObject</code> instances whose script result matches the specified value,
     *      <code>null</code> if no instance matches the value.
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     * @see #select(String, Object)
     */
    public <T extends MDWObject> T detect(String scriptName, Object value);

    /**
     * Returns the first <code>MDWObject</code> of this list whose script evaluation result matches the specified value.
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     * @param arguments the script arguments.
     * @return the first <code>MDWObject</code> instances whose script result matches the specified value, 
     *      <code>null</code> if no instance matches the value.
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     * @see #select(String, List, Object)
     */
    public <T extends MDWObject> T detect(String scriptName, List<?> arguments, Object value);
    
    /**
     * Returns the first <code>MDWObject</code> of this list whose feature matches the specified value.
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * @param feature the feature to evaluate on each <code>MDWObject</code>.
     * @param value the comparison criterium (can be <code>null</code>).
     * @return the first <code>MDWObject</code> instances whose feature matches the specified value,
     *      <code>null</code> if no instance matches the value.
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     * @since MDWorkbench 3.0.0
     */
    public <T extends MDWObject> T detect(EStructuralFeature feature, Object value);

    /**
     * Evaluates the specified script on each <code>MDWObject</code> of this list, 
     * and concatenates the results into a new list.
     * 
     * <p>The returned list is always flattened: if a script result is a collection 
     * (<code>Collection</code>, <code>Map</code>, <code>Iterator</code> or <code>array</code>),
     * we iterate on this collection and add each element into the returned list.</p>
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     *      This script must not expect any parameter.
     * @return a modifiable list containing each script result. 
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     */
    public <T> MDWList<T> collect(String scriptName); 
    
    /**
     * Evaluates the specified script on each <code>MDWObject</code> of this list, 
     * and concatenates the results into a new list.
     * 
     * <p>The returned list is always flattened: if a script result is a collection 
     * (<code>Collection</code>, <code>Map</code>, <code>Iterator</code> or <code>array</code>),
     * we iterate on this collection and add each element into the returned list.</p>
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     * @param arguments the script arguments.
     * @return a modifiable list containing each script result. 
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     */
    public <T> MDWList<T> collect(String scriptName, List<?> arguments); 
    
    /**
     * Evaluates the specified feature on each <code>MDWObject</code> of this list, 
     * and concatenates the results into a new list.
     * 
     * <p>The returned list is always flattened: if a feature is a collection,
     * we iterate on this collection and add each element into the returned list.</p>
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * @param feature the feature to evaluate on each <code>MDWObject</code>.
     * @return a modifiable list containing each feature result. 
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     * @since MDWorkbench 3.0.0
     */
    public <T> MDWList<T> collect(EStructuralFeature feature); 
    
    /**
     * Returns a sorted list into ascending order induced by the specified script.
     * 
     * <p>The script is evaluated on each <code>MDWObject</code> of this list, 
     * the script results are then used to compare elements.
     * This sort is guaranteed to be <i>stable</i>: equal elements will not be reordered as a result of the sort.</p>
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     *      This script must not expect any parameter.
     * @return a modifiable list containing each script result. 
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances,
     *      or if the script results that are not mutually comparable (for example, strings and integers).
     */
    public MDWList<E> sort(String scriptName); 
    
    /**
     * Returns a sorted list into ascending order induced by the specified script.
     * 
     * <p>The script is evaluated on each <code>MDWObject</code> of this list, 
     * the script results are then used to compare elements.
     * This sort is guaranteed to be <i>stable</i>: equal elements will not be reordered as a result of the sort.</p>
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate on each <code>MDWObject</code>.
     * @param arguments the script arguments.
     * @return a modifiable list containing each script result. 
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances,
     *      or if the script results that are not mutually comparable (for example, strings and integers).
     */
    public MDWList<E> sort(String scriptName, List<?> arguments); 
    
    /**
     * Returns a sorted list into ascending order induced by the specified feature.
     * 
     * <p>The feature is evaluated on each <code>MDWObject</code> of this list, 
     * the feature results are then used to compare elements.
     * This sort is guaranteed to be <i>stable</i>: equal elements will not be reordered as a result of the sort.</p>
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * @param feature the feature to evaluate on each <code>MDWObject</code>.
     * @return a modifiable list containing each feature result. 
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     * @since MDWorkbench 3.0.0
     */
    public MDWList<E> sort(EStructuralFeature feature); 
    
    /**
     * Converts this list to a new modifiable <code>MDWSet</code>.
     * 
     * @return a modifiable <code>MDWSet</code> containing each element of this list.
     * @since MDWorkbench 3.0.0
     */
    
    public MDWSet<E> toSet();

    /**
     * Selects <code>MDWObject</code> instances of this list of the specified type.
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * <p>The specified type name can be a fully qualified class (<code>"com.sodius.mdw.metamodel.uml21.Class"</code> for example),
     * or a simple name (<code>"Class"</code>), if this name is unique in the metamodel.</p>
     * 
     * @param typeName the type name of model elements to retrieve.
     * @return a modifiable list of <code>MDWObject</code> that matches the specified type.
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     * @see #getInstances(String, boolean)
     */
    public <T extends E> MDWList<T> getInstances(String typeName);

    /**
     * Selects <code>MDWObject</code> instances of this list of the specified type, or one of its sub types.
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * <p>The specified type name can be a fully qualified class (<code>"com.sodius.mdw.metamodel.uml21.Class"</code> for example),
     * or a simple name (<code>"Class"</code>), if this name is unique in the metamodel.</p>
     * 
     * @param typeName the type name of model elements to retrieve.
     * @param includeSubTypes determines wheter to retrieve elements based on sub types.
     * @return a modifiable list of <code>MDWObject</code> that matches the specified type, or one of its sub types.
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     * @see #getInstances(String)
     */
    public <T extends E> MDWList<T> getInstances(String typeName, boolean includeSubTypes);
    
    /**
     * Selects <code>MDWObject</code> instances of this list of the specified type.
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * @param type the type of model elements to retrieve.
     * @return a modifiable list of <code>MDWObject</code> that matches the specified type.
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     * @see #getInstances(EClass, boolean)
     * @since MDWorkbench 3.0.0
     */
    public <T extends E> MDWList<T> getInstances(EClass type);

    /**
     * Selects <code>MDWObject</code> instances of this list of the specified type, or one of its sub types.
     * 
     * <p>This list is expected to contain only <code>MDWObject</code> instances. 
     * A <code>ClassCastException</code> is thrown if any other kind of element is found 
     * (<code>null</code> values are silently ignored).</p>  
     * 
     * @param type the type of model elements to retrieve.
     * @param includeSubTypes determines wheter to retrieve elements based on sub types.
     * @return a modifiable list of <code>MDWObject</code> that matches the specified type, or one of its sub types.
     * @throws ClassCastException if the list contains anything else than <code>MDWObject</code> instances.
     * @see #getInstances(EClass)
     * @since MDWorkbench 3.0.0
     */
    public <T extends E> MDWList<T> getInstances(EClass type, boolean includeSubTypes);
    
    /**
     * Selects <code>MDWObject</code> instances of this list of the specified type, which match the specified profile name.
     * This method does not consider sub types of the specified type.
     * 
     * <p>The specified type name can be a fully qualified class (<code>"com.sodius.mdw.metamodel.uml21.Class"</code> for example),
     * or a simple name (<code>"Class"</code>), if this name is unique in the metamodel.</p>
     * 
     * @param typeName the type name of model elements to retrieve.
     * @param profileName the name of the profile type the instances must match.
     * @return a modifiable list of <code>MDWObject</code> that matches the specified type.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if this API is not called during an MDWorkbench evaluation.
     * @see #getProfiledInstances(String, String, boolean)
     * @since MDWorkbench 3.0.0
     */
    public <T extends E> MDWList<T> getProfiledInstances(String typeName, String profileName);
    
    /**
     * Selects <code>MDWObject</code> instances of this list of the specified type, or one of its sub types, 
     * which match the specified profile name.
     * 
     * <p>The specified type name can be a fully qualified class (<code>"com.sodius.mdw.metamodel.uml21.Classifier"</code> for example),
     * or a simple name (<code>"Classifier"</code>), if this name is unique in the metamodel.</p>
     * 
     * @param typeName the type name of model elements to retrieve.
     * @param profileName the name of the profile type the instances must match.
     * @param includeSubTypes determines wheter to retrieve elements based on sub types.
     * @return a modifiable list of <code>MDWObject</code> that matches the specified type, or one of its sub types.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if this API is not called during an MDWorkbench evaluation.
     * @see #getProfiledInstances(String, String)
     * @since MDWorkbench 3.0.0
     */
    public <T extends E> MDWList<T> getProfiledInstances(String typeName, String profileName, boolean includeSubTypes);
    
    /**
     * Selects <code>MDWObject</code> instances of this list of the specified type, which match the specified profile name.
     * This method does not consider sub types of the specified type.
     * 
     * @param type the type of model elements to retrieve.
     * @param profileName the name of the profile type the instances must match.
     * @return a modifiable list of <code>MDWObject</code> that matches the specified type.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if this API is not called during an MDWorkbench evaluation.
     * @see #getProfiledInstances(EClass, String, boolean)
     * @since MDWorkbench 3.0.0
     */
    public <T extends E> MDWList<T> getProfiledInstances(EClass type, String profileName);
    
    /**
     * Selects <code>MDWObject</code> instances of this list of the specified type, or one of its sub types, 
     * which match the specified profile name.
     * 
     * @param type the type of model elements to retrieve.
     * @param profileName the name of the profile type the instances must match.
     * @param includeSubTypes determines wheter to retrieve elements based on sub types.
     * @return a modifiable list of <code>MDWObject</code> that matches the specified type, or one of its sub types.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if this API is not called during an MDWorkbench evaluation.
     * @see #getProfiledInstances(EClass, String)
     * @since MDWorkbench 3.0.0
     */
    public <T extends E> MDWList<T> getProfiledInstances(EClass type, String profileName, boolean includeSubTypes);
}
