package com.sodius.mdw.core.model;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EModelElement;

/**
 * <code>MDWObject</code> is the root of all modeled objects. 
 * Most of method names start with "e" to distinguish the MDWorkbench methods from the client methods. 
 * 
 * <p>It provides support to call scripts, and gives access to the related model and the metamodel.</p>
 * 
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 */
public interface MDWObject extends Comparable<Object>, EModelElement {
    
    /**
     * Returns the containing model, or <code>null</code>.
     * 
     * <p>The returned model is the one used to create this object (<code>Model.create()</code>),
     * or the one to which the object has been added (<code>Model.add()</code>). 
     * 
     * @return the containing model, or <code>null</code> (when the object is removed).
     * @see Model#create(String)
     * @see Model#add(MDWObject)
     */
    public Model eModel();
    
    /**
     * Returns the metamodel to which this object is related.
     * 
     * @return the metamodel used by this object.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if this object is not attached to a model.
     */
    public Metamodel eMetamodel();
    
    /**
     * Returns the simple name of the meta-type (<code>"Actor"</code> or <code>"Activity"</code> for example).
     * 
     * @return the simple name of the meta-type.
     */
    public String eMetaTypeName();
    
    /**
     * Returns the ID which uniquely identifies the object, and which is constant in time.
     * 
     * <p>This ID is not generated by the framework, a <code>MDWObject</code> may not have a unique ID.
     * The returned ID is the one that has been set using <code>eSetUniqueID()</code>.</p>
     * 
     * <p>This ID must be unique during the lifetime of this object, and must be constant between multiple sessions.
     * Reloading a model must assign the same unique ID for each element.</p>
     * 
     * @return the unique ID of this object, <code>null</code> if no ID has been set.
     */
    public String eUniqueID();
    
    /**
     * Sets the unique ID of this object.
     * 
     * <p>This ID must be unique during the lifetime of this object, and must be constant between multiple sessions.
     * Reloading a model must assign the same unique ID for each element.</p>
     * 
     * @param uniqueID the unique ID of this object.
     */
    public void eSetUniqueID(String uniqueID);
    
    /**
     * Compares this object with the specified object for order using the <code>toString()</code> method.
     * 
     * <p><u>Caution</u>: while this implementation may be statisfying for a natural ordering based on <code>toString()</code>,
     * it may have no sense to compare model elements this way. 
     * It is always a good practice to sort model elements by defining your own <code>Comparator</code> 
     * which may handle specific features.</p>
     * 
     * @see Comparable
     * @see java.util.Comparator
     * @see java.util.Collections#sort(java.util.List, java.util.Comparator)
     */
    public int compareTo(Object o);
    
    /**
     * Determines wheter this object is an instance of the specified type.
     * 
     * <p>The specified type name can be a fully qualified class (<code>"com.sodius.mdw.metamodel.uml21.Classifier"</code> for example),
     * or a simple name (<code>"Classifier"</code>), if this name is unique in the metamodel.</p>
     * 
     * @param typeName the type name to compare with.
     * @return <code>true</code> if this object's type matches the specified type or one of its subtype, 
     *      <code>false</code> otherwise.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if this object is not attached to a model.
     * @throws NoSuchTypeException if the specified type does not exists
     */
    public boolean eIsInstanceOf(String typeName);
    
    /**
     * Determines wheter this object is an instance of the specified type and if it matches the specified profile name.
     * 
     * <p>The specified type name can be a fully qualified class (<code>"com.sodius.mdw.metamodel.uml21.Classifier"</code> for example),
     * or a simple name (<code>"Classifier"</code>), if this name is unique in the metamodel.</p>
     * 
     * @param typeName the type name to compare with.
     * @param profileName the name of the profile type this instance must match.
     * @return <code>true</code> if this object's type matches the specified type or one of its subtype, 
     *      <code>false</code> otherwise.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if this object is not attached to a model or
     *      if this API is not called during an MDWorkbench evaluation.
     * @throws NoSuchTypeException if the specified type does not exists
     */
    public boolean eIsInstanceOf(String typeName, String profileName);
    
    /**
     * Determines wheter this object matches the specified profile name.
     * 
     * @param profileName the name of the profile type this instance must match.
     * @param includeSubTypes determines wheter to check for sub profile types.
     * @return <code>true</code> if this object's type matches the specified profile type or one of its subtype, 
     *      <code>false</code> otherwise.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if this object is not attached to a model or
     *      if this API is not called during an MDWorkbench evaluation.
     * @throws NoSuchTypeException if the specified type does not exists
     */
    public boolean eIsProfiled(String profileName, boolean includeSubTypes);
    
    /**
     * Answers wheter this object is tagged as removed.
     * 
     * <p>When removed, an object is no longer attached to a model and you can't anymore call scripts on it.</p>
     * 
     * @return <code>true</code> if the object is removed, <code>false</code> otherwise. 
     */
    public boolean eIsRemoved();
    
    /**
     * Tag this object as removed. Has no effect if the object is already tagged as removed.
     * 
     * <p>This method will:
     * <ul>
     * <li>Removes references from other objects to this one.</li>
     * <li>Detach this object from its container or its resource (if this is a <code>EObject</code>).</li>
     * <li>Detach this object from its model</li>
     * </ul>
     * </p>
     * 
     * <p>Once the object is tagged as removed, it should not be accessible from other objects and 
     * from its model.</p>
     */
    public void eRemove();

    /**
     * Invokes the specified script.
     * 
     * <p>The specified name can reference a structural feature
     * (<code>EAttribute</code> or <code>EReference</code>) defined in the related metamodel.</p>
     * 
     * @param scriptName the name of the script to evaluate (which must not expect any parameter).
     * @return the script result, or <code>null</code> if the scripts returns nothing.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed.
     */
    public Object toObject(String scriptName);

    /**
     * Invokes the specified script.
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate.
     * @param arguments the script arguments.
     * @return the script result, or <code>null</code> if the scripts returns nothing.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed.
     */
    public Object toObject(String scriptName, List<?> arguments);
    
    /**
     * Invokes the specified script and returns a <code>String</code> representation of the evaluation result.
     * 
     * <p>The specified name can reference a structural feature
     * (<code>EAttribute</code> or <code>EReference</code>) defined in the related metamodel.</p>
     * 
     * @param scriptName the name of the script to evaluate (which must not expect any parameter).
     * @return a <code>String</code> representation of the script result, an empty <code>String</code> if the result is <code>null</code> .
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed.
     * @see String#valueOf(java.lang.Object)
     */
    public String toString(String scriptName);
    
    /**
     * Invokes the specified script and returns a <code>String</code> representation of the evaluation result.
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate.
     * @param arguments the script arguments.
     * @return a <code>String</code> representation of the script result, an empty <code>String</code> if the result is <code>null</code> .
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed.
     * @see String#valueOf(java.lang.Object)
     */
    public String toString(String scriptName, List<?> arguments);
    
    /**
     * Invokes the specified script and returns a <code>boolean</code> representation of the evaluation result.
     * 
     * <p>The specified name can reference a structural feature
     * (<code>EAttribute</code> or <code>EReference</code>) defined in the related metamodel.</p>
     * 
     * <p>If the script result is:
     * <ul>
     * <li><code>null</code>: <code>false</code> is returned.</li>
     * <li>a <code>java.lang.Boolean</code>, its <code>boolean</code> value is returned.
     * <li>a <code>java.lang.String</code>, <code>Boolean.valueOf(String)</code> is used to convert the <code>String</code> to a <code>boolean</code>.
     * <li>anything else, an exception is thrown.</li>
     * </ul>
     * </p>
     * 
     * @param scriptName the name of the script to evaluate (which must not expect any parameter).
     * @return a <code>boolean</code> representation of the script result.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed,
     *      or the script result can't be converted to a <code>boolean</code>.
     * @see Boolean#valueOf(java.lang.String)
     */
    public boolean toBoolean(String scriptName);
    
    /**
     * Invokes the specified script and returns a <code>boolean</code> representation of the evaluation result.
     * 
     * <p>If the script result is:
     * <ul>
     * <li><code>null</code>: <code>false</code> is returned.</li>
     * <li>a <code>java.lang.Boolean</code>, its <code>boolean</code> value is returned.
     * <li>a <code>java.lang.String</code>, <code>Boolean.valueOf(String)</code> is used to convert the <code>String</code> to a <code>boolean</code>.
     * <li>anything else, an exception is thrown.</li>
     * </ul>
     * </p>
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate.
     * @param arguments the script arguments.
     * @return a <code>boolean</code> representation of the script result.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed,
     *      or the script result can't be converted to a <code>boolean</code>.
     * @see Boolean#valueOf(java.lang.String)
     */
    public boolean toBoolean(String scriptName, List<?> arguments);
    
    /**
     * Invokes the specified script and returns a <code>int</code> representation of the evaluation result.
     * 
     * <p>The specified name can reference a structural feature
     * (<code>EAttribute</code> or <code>EReference</code>) defined in the related metamodel.</p>
     * 
     * <p>If the script result is:
     * <ul>
     * <li><code>null</code>: <code>0</code> is returned.</li>
     * <li>a <code>java.lang.Number</code>, its <code>int</code> value is returned.
     * <li>a <code>java.lang.String</code>, <code>Integer.parseInt(String)</code> is used to convert the <code>String</code> to a <code>int</code>.
     * <li>anything else, an exception is thrown.</li>
     * </ul>
     * </p>
     * 
     * @param scriptName the name of the script to evaluate (which must not expect any parameter).
     * @return an <code>int</code> representation of the script result.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed,
     *      or the script result can't be converted to a <code>int</code>.
     * @see Integer#parseInt(java.lang.String)
     */
    public int toInt(String scriptName);
    
    /**
     * Invokes the specified script and returns a <code>int</code> representation of the evaluation result.
     * 
     * <p>If the script result is:
     * <ul>
     * <li><code>null</code>: <code>0</code> is returned.</li>
     * <li>a <code>java.lang.Number</code>, its <code>int</code> value is returned.
     * <li>a <code>java.lang.String</code>, <code>Integer.parseInt(String)</code> is used to convert the <code>String</code> to a <code>int</code>.
     * <li>anything else, an exception is thrown.</li>
     * </ul>
     * </p>
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate.
     * @param arguments the script arguments.
     * @return an <code>int</code> representation of the script result.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed,
     *      or the script result can't be converted to a <code>int</code>.
     * @see Integer#parseInt(java.lang.String)
     */
    public int toInt(String scriptName, List<?> arguments);
    
    /**
     * Invokes the specified script and returns a <code>long</code> representation of the evaluation result.
     * 
     * <p>The specified name can reference a structural feature
     * (<code>EAttribute</code> or <code>EReference</code>) defined in the related metamodel.</p>
     * 
     * <p>If the script result is:
     * <ul>
     * <li><code>null</code>: <code>0</code> is returned.</li>
     * <li>a <code>java.lang.Number</code>, its <code>long</code> value is returned.
     * <li>a <code>java.lang.String</code>, <code>Long.parseLong(String)</code> is used to convert the <code>String</code> to a <code>long</code>.
     * <li>anything else, an exception is thrown.</li>
     * </ul>
     * </p>
     * 
     * @param scriptName the name of the script to evaluate (which must not expect any parameter).
     * @return a <code>long</code> representation of the script result.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed,
     *      or the script result can't be converted to a <code>long</code>.
     * @see Long#parseLong(java.lang.String)
     */
    public long toLong(String scriptName);
    
    /**
     * Invokes the specified script and returns a <code>long</code> representation of the evaluation result.
     * 
     * <p>If the script result is:
     * <ul>
     * <li><code>null</code>: <code>0</code> is returned.</li>
     * <li>a <code>java.lang.Number</code>, its <code>long</code> value is returned.
     * <li>a <code>java.lang.String</code>, <code>Long.parseLong(String)</code> is used to convert the <code>String</code> to a <code>long</code>.
     * <li>anything else, an exception is thrown.</li>
     * </ul>
     * </p>
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate.
     * @param arguments the script arguments.
     * @return a <code>long</code> representation of the script result.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed,
     *      or the script result can't be converted to a <code>long</code>.
     * @see Long#parseLong(java.lang.String)
     */
    public long toLong(String scriptName, List<?> arguments);
    
    /**
     * Invokes the specified script and returns a <code>short</code> representation of the evaluation result.
     * 
     * <p>The specified name can reference a structural feature
     * (<code>EAttribute</code> or <code>EReference</code>) defined in the related metamodel.</p>
     * 
     * <p>If the script result is:
     * <ul>
     * <li><code>null</code>: <code>0</code> is returned.</li>
     * <li>a <code>java.lang.Number</code>, its <code>short</code> value is returned.
     * <li>a <code>java.lang.String</code>, <code>Short.parseShort(String)</code> is used to convert the <code>String</code> to a <code>short</code>.
     * <li>anything else, an exception is thrown.</li>
     * </ul>
     * </p>
     * 
     * @param scriptName the name of the script to evaluate (which must not expect any parameter).
     * @return a <code>short</code> representation of the script result.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed,
     *      or the script result can't be converted to a <code>short</code>.
     * @see Short#parseShort(java.lang.String)
     */
    public short toShort(String scriptName);

    /**
     * Invokes the specified script and returns a <code>short</code> representation of the evaluation result.
     * 
     * <p>If the script result is:
     * <ul>
     * <li><code>null</code>: <code>0</code> is returned.</li>
     * <li>a <code>java.lang.Number</code>, its <code>short</code> value is returned.
     * <li>a <code>java.lang.String</code>, <code>Short.parseShort(String)</code> is used to convert the <code>String</code> to a <code>short</code>.
     * <li>anything else, an exception is thrown.</li>
     * </ul>
     * </p>
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate.
     * @param arguments the script arguments.
     * @return a <code>short</code> representation of the script result.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed,
     *      or the script result can't be converted to a <code>short</code>.
     * @see Short#parseShort(java.lang.String)
     */
    public short toShort(String scriptName, List<?> arguments);
    
    /**
     * Invokes the specified script and returns a <code>float</code> representation of the evaluation result.
     * 
     * <p>The specified name can reference a structural feature
     * (<code>EAttribute</code> or <code>EReference</code>) defined in the related metamodel.</p>
     * 
     * <p>If the script result is:
     * <ul>
     * <li><code>null</code>: <code>0</code> is returned.</li>
     * <li>a <code>java.lang.Number</code>, its <code>float</code> value is returned.
     * <li>a <code>java.lang.String</code>, <code>Float.parseFloat(String)</code> is used to convert the <code>String</code> to a <code>float</code>.
     * <li>anything else, an exception is thrown.</li>
     * </ul>
     * </p>
     * 
     * @param scriptName the name of the script to evaluate (which must not expect any parameter).
     * @return a <code>float</code> representation of the script result.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed,
     *      or the script result can't be converted to a <code>float</code>.
     * @see Float#parseFloat(java.lang.String)
     */
    public float toFloat(String scriptName);
    
    /**
     * Invokes the specified script and returns a <code>float</code> representation of the evaluation result.
     * 
     * <p>If the script result is:
     * <ul>
     * <li><code>null</code>: <code>0</code> is returned.</li>
     * <li>a <code>java.lang.Number</code>, its <code>float</code> value is returned.
     * <li>a <code>java.lang.String</code>, <code>Float.parseFloat(String)</code> is used to convert the <code>String</code> to a <code>float</code>.
     * <li>anything else, an exception is thrown.</li>
     * </ul>
     * </p>
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate.
     * @param arguments the script arguments.
     * @return a <code>float</code> representation of the script result.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed,
     *      or the script result can't be converted to a <code>float</code>.
     * @see Float#parseFloat(java.lang.String)
     */
    public float toFloat(String scriptName, List<?> arguments);
    
    /**
     * Invokes the specified script and returns a <code>double</code> representation of the evaluation result.
     * 
     * <p>The specified name can reference a structural feature
     * (<code>EAttribute</code> or <code>EReference</code>) defined in the related metamodel.</p>
     * 
     * <p>If the script result is:
     * <ul>
     * <li><code>null</code>: <code>0</code> is returned.</li>
     * <li>a <code>java.lang.Number</code>, its <code>double</code> value is returned.
     * <li>a <code>java.lang.String</code>, <code>Double.parseDouble(String)</code> is used to convert the <code>String</code> to a <code>double</code>.
     * <li>anything else, an exception is thrown.</li>
     * </ul>
     * </p>
     * 
     * @param scriptName the name of the script to evaluate (which must not expect any parameter).
     * @return a <code>double</code> representation of the script result.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed,
     *      or the script result can't be converted to a <code>double</code>.
     * @see Double#parseDouble(java.lang.String)
     */
    public double toDouble(String scriptName);
    
    /**
     * Invokes the specified script and returns a <code>double</code> representation of the evaluation result.
     * 
     * <p>If the script result is:
     * <ul>
     * <li><code>null</code>: <code>0</code> is returned.</li>
     * <li>a <code>java.lang.Number</code>, its <code>double</code> value is returned.
     * <li>a <code>java.lang.String</code>, <code>Double.parseDouble(String)</code> is used to convert the <code>String</code> to a <code>double</code>.
     * <li>anything else, an exception is thrown.</li>
     * </ul>
     * </p>
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate.
     * @param arguments the script arguments.
     * @return a <code>double</code> representation of the script result.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed,
     *      or the script result can't be converted to a <code>double</code>.
     * @see Double#parseDouble(java.lang.String)
     */
    public double toDouble(String scriptName, List<?> arguments);
    
    /**
     * Invokes the specified script and returns a <code>byte</code> representation of the evaluation result.
     * 
     * <p>The specified name can reference a structural feature
     * (<code>EAttribute</code> or <code>EReference</code>) defined in the related metamodel.</p>
     * 
     * <p>If the script result is:
     * <ul>
     * <li><code>null</code>: <code>0</code> is returned.</li>
     * <li>a <code>java.lang.Number</code>, its <code>byte</code> value is returned.
     * <li>a <code>java.lang.String</code>, <code>Byte.parseByte(String)</code> is used to convert the <code>String</code> to a <code>byte</code>.
     * <li>anything else, an exception is thrown.</li>
     * </ul>
     * </p>
     * 
     * @param scriptName the name of the script to evaluate (which must not expect any parameter).
     * @return a <code>byte</code> representation of the script result.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed,
     *      or the script result can't be converted to a <code>byte</code>.
     * @see Byte#parseByte(java.lang.String)
     */
    public byte toByte(String scriptName);
    
    /**
     * Invokes the specified script and returns a <code>byte</code> representation of the evaluation result.
     * 
     * <p>If the script result is:
     * <ul>
     * <li><code>null</code>: <code>0</code> is returned.</li>
     * <li>a <code>java.lang.Number</code>, its <code>byte</code> value is returned.
     * <li>a <code>java.lang.String</code>, <code>Byte.parseByte(String)</code> is used to convert the <code>String</code> to a <code>byte</code>.
     * <li>anything else, an exception is thrown.</li>
     * </ul>
     * </p>
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate.
     * @param arguments the script arguments.
     * @return a <code>byte</code> representation of the script result.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed,
     *      or the script result can't be converted to a <code>byte</code>.
     * @see Byte#parseByte(java.lang.String)
     */
    public byte toByte(String scriptName, List<?> arguments);
    
    /**
     * Invokes the specified script and returns a <code>char</code> representation of the evaluation result.
     * 
     * <p>The specified name can reference a structural feature
     * (<code>EAttribute</code> or <code>EReference</code>) defined in the related metamodel.</p>
     * 
     * <p>If the script result is:
     * <ul>
     * <li>a <code>java.lang.Character</code>, its <code>char</code> value is returned.
     * <li>a <code>java.lang.String</code> of one character, its unique character is returned.
     * <li>anything else, an exception is thrown.</li>
     * </ul>
     * </p>
     * 
     * @param scriptName the name of the script to evaluate (which must not expect any parameter).
     * @return a <code>byte</code> representation of the script result.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed,
     *      or the script result can't be converted to a <code>char</code>.
     */
    public char toChar(String scriptName);
    
    /**
     * Invokes the specified script and returns a <code>char</code> representation of the evaluation result.
     * 
     * <p>If the script result is:
     * <ul>
     * <li>a <code>java.lang.Character</code>, its <code>char</code> value is returned.
     * <li>a <code>java.lang.String</code> of one character, its unique character is returned.
     * <li>anything else, an exception is thrown.</li>
     * </ul>
     * </p>
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate.
     * @param arguments the script arguments.
     * @return a <code>byte</code> representation of the script result.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed,
     *      or the script result can't be converted to a <code>char</code>.
     */
    public char toChar(String scriptName, List<?> arguments);
    
    /**
     * Invokes the specified script and returns a <code>MDWSet</code> representation of the evaluation result.
     * <code>MDWSet</code> provides facilities to invoke scripts on each <code>MDWObject</code> element of the Set.
     * 
     * <p>The specified name can reference a structural feature
     * (<code>EAttribute</code> or <code>EReference</code>) defined in the related metamodel.</p>
     * 
     * <p>If the script result is:
     * <ul>
     * <li><code>null</code>, an empty <code>MDWSet</code> is returned.
     * <li>a collection-like (<code>Collection</code>, <code>Map</code>, <code>Iterator</code> or <code>array</code>), 
     * a <code>MDWSet</code> is created and wraps these elements.</li>
     * <li>anything else, a singleton <code>MDWSet</code> is created containing just this element.</li>
     * </ul>
     * </p>
     * 
     * @param scriptName the name of the script to evaluate (which must not expect any parameter).
     * @return a modifiable <code>MDWSet</code> representation of the script result.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed.
     * @since MDWorkbench 3.0.0
     */
    public <T> MDWSet<T> toSet(String scriptName);
    
    /**
     * Invokes the specified script and returns a <code>MDWSet</code> representation of the evaluation result.
     * <code>MDWSet</code> provides facilities to invoke scripts on each <code>MDWObject</code> element of the Set.
     * 
     * <p>If the script result is:
     * <ul>
     * <li><code>null</code>, an empty <code>MDWSet</code> is returned.
     * <li>a collection-like (<code>Collection</code>, <code>Map</code>, <code>Iterator</code> or <code>array</code>), 
     * a <code>MDWSet</code> is created and wraps these elements.</li>
     * <li>anything else, a singleton <code>MDWSet</code> is created containing just this element.</li>
     * </ul>
     * </p>
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate.
     * @param arguments the script arguments.
     * @return a modifiable <code>MDWSet</code> representation of the script result.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed.
     * @since MDWorkbench 3.0.0
     */
    public <T> MDWSet<T> toSet(String scriptName, List<?> arguments);
    
    /**
     * Invokes the specified script and returns a <code>MDWList</code> representation of the evaluation result.
     * <code>MDWList</code> provides facilities to invoke scripts on each <code>MDWObject</code> element of the list.
     * 
     * <p>The specified name can reference a structural feature
     * (<code>EAttribute</code> or <code>EReference</code>) defined in the related metamodel.</p>
     * 
     * <p>If the script result is:
     * <ul>
     * <li><code>null</code>, an empty <code>MDWList</code> is returned.
     * <li>a collection-like (<code>Collection</code>, <code>Map</code>, <code>Iterator</code> or <code>array</code>), 
     * a <code>MDWList</code> is created and wraps these elements.</li>
     * <li>anything else, a singleton <code>MDWList</code> is created containing just this element.</li>
     * </ul>
     * </p>
     * 
     * @param scriptName the name of the script to evaluate (which must not expect any parameter).
     * @return a modifiable <code>MDWList</code> representation of the script result.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed.
     */
    public <T> MDWList<T> toList(String scriptName);
    
    /**
     * Invokes the specified script and returns a <code>MDWList</code> representation of the evaluation result.
     * <code>MDWList</code> provides facilities to invoke scripts on each <code>MDWObject</code> element of the list.
     * 
     * <p>If the script result is:
     * <ul>
     * <li><code>null</code>, an empty <code>MDWList</code> is returned.
     * <li>a collection-like (<code>Collection</code>, <code>Map</code>, <code>Iterator</code> or <code>array</code>), 
     * a <code>MDWList</code> is created and wraps these elements.</li>
     * <li>anything else, a singleton <code>MDWList</code> is created containing just this element.</li>
     * </ul>
     * </p>
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate.
     * @param arguments the script arguments.
     * @return a modifiable <code>MDWList</code> representation of the script result.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed.
     */
    public <T> MDWList<T> toList(String scriptName, List<?> arguments);
    
    /**
     * Converts the specified collection to a new modifiable <code>MDWList</code>.
     * <code>MDWList</code> provides facilities to invoke scripts on each <code>MDWObject</code> element of the list.
     * 
     * @return a modifiable <code>MDWList</code> containing each element of the specified collection.
     */
    public <E> MDWList<E> toList(Collection<E> values);
    
    /**
     * Converts the specified iterator to a new modifiable <code>MDWList</code>.
     * <code>MDWList</code> provides facilities to invoke scripts on each <code>MDWObject</code> element of the list.
     * 
     * @return a modifiable <code>MDWList</code> containing each element of the specified iterator.
     */
    public <E> MDWList<E> toList(Iterator<E> values);
    
    /**
     * Converts the specified array to a new modifiable <code>MDWList</code>.
     * <code>MDWList</code> provides facilities to invoke scripts on each <code>MDWObject</code> element of the list.
     * 
     * @return a modifiable <code>MDWList</code> containing each element of the specified array.
     */
    public <E> MDWList<E> toList(E[] values);
    
    /**
     * Invokes the specified script defined on the super-type.
     * 
     * <p>This is useful when overriding a script: when you redefine a script on a subtype,
     * you're still able to call the super-type script and refine its behavior.</p>
     * 
     * <p>If the related type has many super-types (for example in UML, <code>Classifier</code> 
     * inherits from <code>GeneralizableElement</code> and from <code>Namespace</code>), 
     * the specified script may be defined on more than one super-type, and the call can be ambiguous.  
     * To resolve this ambiguity, the expected script can be prefixed by its defining super-type name. 
     * For example, from a <code>Classifier</code> element, you can call <code>self.superScript("Namespace.myScript")</code>.</p>
     * 
     * @param scriptName the name of the script to evaluate (which must not expect any parameter).
     * @return the script result.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed.
     * @throws NoSuchTypeException if the super-type (if specified) does not exists
     */
    public Object superScript(String scriptName);
    
    /**
     * Invokes the specified script defined on the super-type.
     * 
     * <p>This is useful when overriding a script: when you redefine a script on a subtype,
     * you're still able to call the super-type script and refine its behavior.</p>
     * 
     * <p>If the related type has many super-types (for example in UML, <code>Classifier</code> 
     * inherits from <code>GeneralizableElement</code> and from <code>Namespace</code>), 
     * the specified script may be defined on more than one super-type, and the call can be ambiguous.  
     * To resolve this ambiguity, the expected script can be prefixed by its defining super-type name. 
     * For example, from a <code>Classifier</code> element, you can call <code>self.superScript("Namespace.myScript")</code>.</p>
     * 
     * <p>The number and type of arguments must match the parameters of the script.</p>
     * 
     * @param scriptName the name of the script to evaluate.
     * @param arguments the script arguments.
     * @return the script result.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if the script does not exist or is not visible.
     * @throws com.sodius.mdw.core.eval.EvaluationException if the evaluation of the script failed.
     * @throws NoSuchTypeException if the super-type (if specified) does not exists
     */
    public Object superScript(String scriptName, List<?> arguments);

    /**
     * Returns the type which gives reflective access to structural features.
     * 
     * @return the type which gives reflective access to structural features.
     * @since MDWorkbench 3.0.0
     */
    public ExtendedClass eExtendedClass();
}
