package com.sodius.mdw.core.model;

import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EPackage;

import com.sodius.mdw.core.CoreException;
import com.sodius.mdw.core.model.io.ConnectorDescriptor;

/**
 * Defines a metamodel ("UML 2.1" for example), from which you can manage models.
 * 
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 * 
 * @see Model#getMetamodel()
 * @see MDWObject#eMetamodel()
 * @see MetamodelManager#getMetamodel(String)
 */
public interface Metamodel extends Comparable<Object> {

    /**
     * Returns the manager to which the metamodel is registered.
     * 
     * @return the metamodel manager.
     */
    public MetamodelManager getManager();
    
    /**
     * Returns the configuration from which this metamodel originates.
     * The configuration can be used to retrieve metamodel related resources.
     * 
     * @return the metamodel configuration.
     */
    public MetamodelConfiguration getConfiguration();
    
    /**
     * Returns the name of this metamodel ("UML" for example).
     * 
     * @return the name of the metamodel
     */
    public String getName(); 
    
    /**
     * Returns the version of this metamodel ("2.1" for example), if any.
     * This version is for example used during XMI serialization. 
     * 
     * @return the version of the metamodel, an empty <code>String</code> if no version is specified.
     */
    public String getVersion(); 
    
    /**
     * Returns the display name of this metamodel, which may include a name and 
     * a metamodel version ("UML 2.1" for example).
     * 
     * @return the display name of the metamodel
     */
    public String getPresentationName(); 
    
    /**
     * Returns the ID which uniquely identifies this metamodel.
     * This ID will be used in templates to refer to the metamodel.
     * 
     * @return the unique ID of the metamodel.
     */
    public String getUniqueID();
    
    /**
     * Creates an empty model based on this metamodel.
     * 
     * @return an empty model.
     */
    public Model createModel();
    
    /**
     * Creates and populates a model using the specified connector information.
     * 
     * <p>The reader name must match a connector registered on this metamodel.
     * The uri is used to locate a resource from where the connector should read information.
     * If the connector expects a file, the uri must be a valid file path. 
     * Otherwise, it can be any connector specific resource locator, 
     * including <code>null</code> when the connector does not expect any uri.</p>
     * 
     * <p>This method is a facility and acts as a strict equivalent of calling {@link #createModel()}
     * and then {@link Model#read(String, String)}.</p>
     * 
     * @param readerName the connector name to be used.
     * @param uri an additional resource locator used by the connector to read the model (can be <code>null</code>)
     * @throws CoreException if the model could not be loaded. Reasons include:
     * <ul>
     * <li>The reader name does not match any registered connector.</li>
     * <li>The uri is not valid.</li>
     * <li>The connector failed to read the resource located by the uri.</li>
     * </ul>
     * @see Model#read(String, String)
     * @see #getModelReaderDescriptors()
     * @see ConnectorDescriptor#getKind()
     */
    public Model readModel(String readerName, String uri) throws CoreException;
    
    /**
     * Returns the registered connectors that can be used to read a model.
     * 
     * <p>The connectors are returned in the order they are declared in the <code>metamodels.xml</code> file
     * of this metamodel. Additional connectors (declared outside of the metamodel descriptor) 
     * are appended in an unpredictable order.</p> 
     * 
     * @return an unmodifiable list of registered connectors that can be used to read a model.
     */
    public List<ConnectorDescriptor> getModelReaderDescriptors();
    
    /**
     * Returns the connector, used to read a model, that matches the specified name.
     * 
     * @param name the name of the connector to retrieve.
     * @return the reader connector that matches the specified name
     * @throws CoreException if no reader connector matches this name
     * @see ConnectorDescriptor#getName() 
     */
    public ConnectorDescriptor getModelReaderDescriptor(String name) throws CoreException;

    
    /**
     * Returns the registered connectors that can be used to write a model.
     * 
     * <p>The connectors are returned in the order they are declared in the <code>metamodels.xml</code> file
     * of this metamodel. Additional connectors (declared outside of the metamodel descriptor) 
     * are appended in an unpredictable order.</p> 
     * 
     * @return an unmodifiable list of registered connectors that can be used to write a model.
     */
    public List<ConnectorDescriptor> getModelWriterDescriptors();
    
    /**
     * Returns the connector, used to write a model, that matches the specified name.
     * 
     * @param name the name of the connector to retrieve.
     * @return the writer connector that matches the specified name
     * @throws CoreException if no writer connector matches this name
     * @see ConnectorDescriptor#getName() 
     */
    public ConnectorDescriptor getModelWriterDescriptor(String name) throws CoreException;
    
    /**
     * Compare the metamodels based on their presentation name.
     * 
     * @see #getPresentationName()
     */
    public int compareTo(Object o);
    
    /**
     * Returns the list of <code>EClassifier</code>s defined in this metamodel.
     * 
     * @return the list of <code>EClassifier</code>s defined in this metamodel.
     * @since MDWorkbench 3.0.0
     */
    public EList<EClassifier> getEClassifiers();
    
    /**
     * Returns the <code>EClassifier</code> of specified name defined in this metamodel. 
     * 
     * @param name the name of the classifier to look for
     * @return the <code>EClassifier</code> of specified name defined in this metamodel, <code>null</code> if none.
     * @since MDWorkbench 3.0.0
     */
    public EClassifier getEClassifier(String name);

    /**
     * Returns the <code>EPackage</code> declaring this metamodel.
     * 
     * @return the <code>EPackage</code> declaring this metamodel.
     * @since MDWorkbench 3.0.0
     */
    public EPackage getEPackage();
    
    /**
     * Determines if the metamodel represented by this <code>Metamodel</code> object 
     * is the same as the specified <code>Metamodel</code> parameter,
     * and if the applied metamodel extensions are the same or a subset of the ones applied on the <code>Metamodel</code> parameter.
     * 
     * @param metamodel the metamodel to be checked
     * @return <code>true</code> if the specified metamodel is the same as the receiver and if the applied metamodel extensions are compatible.
     * @since MDWorkbench 3.0.0
     */
    public boolean isAssignableFrom(Metamodel metamodel);
}
