package com.sodius.mdw.core.model;

import java.util.Collection;
import java.util.Map;

import org.eclipse.emf.ecore.EClass;

import com.sodius.mdw.core.CoreException;
import com.sodius.mdw.core.model.diagram.DiagramImage;
import com.sodius.mdw.core.model.diagram.DiagramOptions;
import com.sodius.mdw.core.model.io.ConnectorDescriptor;

/**
 * A <code>Model</code> holds <code>MDWObject</code> instances.
 * A model can be:
 * <ul>
 * <li>loaded from a resource using {@link #read(String, String)}</li>
 * <li>persisted to a resource using {@link #write(String, String)}</li>
 * <li>queried for its contents using {@link #getInstances(String)}</li>
 * </ul>
 * 
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 * 
 * @see MDWObject#eModel()
 * @see Metamodel#readModel(String, String)
 */
public interface Model {
    
    /**
     * Returns the metamodel this model is based on.
     * 
     * @return the metamodel.
     */
    public Metamodel getMetamodel();
    
    /**
     * Returns the metamodel extensions currently applied on this model.
     * 
     * @return the metamodel extensions currently applied on this model.
     * @since MDWorkbench 3.0.0
     */
    public Collection<MetamodelExtension> getAppliedMetamodelExtensions();
    
    /**
     * Returns all metamodel extensions that can be applied on this model.
     * This list contains extensions that may already be applied on this model.
     * 
     * @return all metamodel extensions that can be applied on this model.
     * @since MDWorkbench 3.0.0
     */
    public Collection<MetamodelExtension> getApplicableMetamodelExtensions();
    
    /**
     * Applies the specified metamodel extensions on this model.
     * If those extensions require other extensions, those extensions are applied as well.
     * It should be noted that once an extension has been applied, it cannot be unapplied.
     * 
     * @param extensions the metamodel extensions to apply on the model.
     * @since MDWorkbench 3.0.0
     */
    public void applyMetamodelExtensions(Collection<MetamodelExtension> extensions);

    /**
     * Clear this model and then populates its contents using the specified connector information.
     * 
     * <p>The reader name must match a connector registered on the related metamodel.
     * The uri is used to locate a resource from where the connector should read information.
     * If the connector expects a file, the uri must be a valid file path. 
     * Otherwise, it can be any connector specific resource locator, 
     * including <code>null</code> when the connector does not expect any uri.</p>
     * 
     * @param readerName the connector name to be used.
     * @param uri an additional resource locator used by the connector to read the model (can be <code>null</code>)
     * @throws CoreException if the model could not be loaded. Reasons include:
     * <ul>
     * <li>The reader name does not match any registered connector.</li>
     * <li>The uri is not valid.</li>
     * <li>The connector failed to read the resource located by the uri.</li>
     * </ul>
     * @see Metamodel#getModelReaderDescriptors()
     * @see ConnectorDescriptor#getKind()
     */
    public void read(String readerName, String uri) throws CoreException;
    
    /**
     * Clear this model and then populates its contents using the specified connector information.
     * 
     * <p>The reader name must match a connector registered on the related metamodel.
     * The uri is used to locate a resource from where the connector should read information.
     * If the connector expects a file, the uri must be a valid file path. 
     * Otherwise, it can be any connector specific resource locator, 
     * including <code>null</code> when the connector does not expect any uri.</p>
     * 
     * @param readerName the connector name to be used.
     * @param uri an additional resource locator used by the connector to read the model (can be <code>null</code>)
     * @param options properties to configure the model reader.
     * @throws CoreException if the model could not be loaded. Reasons include:
     * <ul>
     * <li>The reader name does not match any registered connector.</li>
     * <li>The uri is not valid.</li>
     * <li>The connector failed to read the resource located by the uri.</li>
     * </ul>
     * @see Metamodel#getModelReaderDescriptors()
     * @see ConnectorDescriptor#getKind()
     * @since 3.1.0
     */
    public void read(String readerName, String uri, Map<String, ?> options) throws CoreException;
    
    /**
     * Returns the last connector used to read this model.
     * 
     * @return the last connector used to read this model, 
     * <code>null</code> if the model has never been loaded from a resource.
     */
    public ConnectorDescriptor getReader();
    
    /**
     * Returns the last uri used to read this model.
     * 
     * @return the last connector used to read this model, 
     * <code>null</code> if the model has never been loaded from a resource or if the uri used was <code>null</code>.
     */
    public String getReadURI();
    
    /**
     * Writes the model contents using the specified connector information.
     * 
     * <p>The writer name must match a connector registered on the related metamodel.
     * The uri is used to locate a resource where the connector should write information.
     * If the connector expects a file, the uri must be a valid file path. 
     * Otherwise, it can be any connector specific resource locator, 
     * including <code>null</code> when the connector does not expect any uri.</p>
     * 
     * @param writerName the connector name to be used.
     * @param uri an additional resource locator used by the connector to write the model (can be <code>null</code>)
     * @throws CoreException if the model could not be written. Reasons include:
     * <ul>
     * <li>The writer name does not match any registered connector.</li>
     * <li>The uri is not valid.</li>
     * <li>The connector failed to write the resource located by the uri.</li>
     * </ul>
     * @see Metamodel#getModelWriterDescriptors()
     * @see ConnectorDescriptor#getKind()
     */
    public void write(String writerName, String uri) throws CoreException;
    
    /**
     * Writes the model contents using the specified connector information.
     * 
     * <p>The writer name must match a connector registered on the related metamodel.
     * The uri is used to locate a resource where the connector should write information.
     * If the connector expects a file, the uri must be a valid file path. 
     * Otherwise, it can be any connector specific resource locator, 
     * including <code>null</code> when the connector does not expect any uri.</p>
     * 
     * @param writerName the connector name to be used.
     * @param uri an additional resource locator used by the connector to write the model (can be <code>null</code>)
     * @param options properties to configure the model writer.
     * @throws CoreException if the model could not be written. Reasons include:
     * <ul>
     * <li>The writer name does not match any registered connector.</li>
     * <li>The uri is not valid.</li>
     * <li>The connector failed to write the resource located by the uri.</li>
     * </ul>
     * @see Metamodel#getModelWriterDescriptors()
     * @see ConnectorDescriptor#getKind()
     */
    public void write(String writerName, String uri, Map<String, ?> options) throws CoreException;
    
    /**
     * Returns the last connector used to write this model.
     * 
     * @return the last connector used to write this model, 
     * <code>null</code> if the model has never been written to a resource.
     */
    public ConnectorDescriptor getWriter();
    
    /**
     * Returns the last uri used to write this model.
     * 
     * @return the last connector used to write this model, 
     * <code>null</code> if the model has never been written to a resource or if the uri used was <code>null</code>.
     */
    public String getWriteURI();
    
    /**
     * Returns all model elements attached to this model.
     * 
     * <p>The returned list is a <code>MDWList</code> so you can directly invoke scripts on the model elements.
     * This list has no predictable order.</p>
     * 
     * @return an unmodifiable list of <code>MDWObject</code> contained in this model.
     * @throws UnsupportedOperationException if this method is not supported by a particular model implementation
     * @see MDWObject
     */
    public <T extends MDWObject> MDWList<T> getInstances();
    
    /**
     * Returns all model elements of the specified type attached to this model.
     * This method does not consider sub types of the specified type.
     * 
     * <p>The specified type name can be a fully qualified class (<code>"com.sodius.mdw.metamodel.uml21.Class"</code> for example),
     * or a simple name (<code>"Class"</code>), if this name is unique in the metamodel.</p>
     * 
     * <p>The returned list is a <code>MDWList</code> so you can directly invoke scripts on the model elements.
     * These elements are ordered the way they were created and added to this model.</p>
     * 
     * @param typeName the type name of model elements to retrieve.
     * @return an unmodifiable list of <code>MDWObject</code> contained in this model that matches the specified type.
     * @throws UnsupportedOperationException if this method is not supported by a particular model implementation
     * @see MDWObject
     * @see #getInstances(String, boolean)
     */
    public <T extends MDWObject> MDWList<T> getInstances(String typeName);
    
    /**
     * Returns all model elements of the specified type, or one of its sub types, attached to this model.
     * 
     * <p>The specified type name can be a fully qualified class (<code>"com.sodius.mdw.metamodel.uml21.Classifier"</code> for example),
     * or a simple name (<code>"Classifier"</code>), if this name is unique in the metamodel.</p>
     * 
     * <p>The returned list is a <code>MDWList</code> so you can directly invoke scripts on the model elements.
     * This list has no predictable order.</p>
     * 
     * @param typeName the type name of model elements to retrieve.
     * @param includeSubTypes determines wheter to retrieve elements based on sub types.
     * @return an unmodifiable list of <code>MDWObject</code> contained in this model that matches the specified type, or one of its sub types.
     * @throws UnsupportedOperationException if this method is not supported by a particular model implementation
     * @see MDWObject
     * @see #getInstances(String)
     */
    public <T extends MDWObject> MDWList<T> getInstances(String typeName, boolean includeSubTypes);
    
    /**
     * Returns all model elements of the specified type attached to this model.
     * This method does not consider sub types of the specified type.
     * 
     * <p>The returned list is a <code>MDWList</code> so you can directly invoke scripts on the model elements.
     * These elements are ordered the way they were created and added to this model.</p>
     * 
     * @param type the type of model elements to retrieve.
     * @return an unmodifiable list of <code>MDWObject</code> contained in this model that matches the specified type.
     * @throws UnsupportedOperationException if this method is not supported by a particular model implementation
     * @see MDWObject
     * @see #getInstances(EClass, boolean)
     * @since MDWorkbench 3.0.0
     */
    public <T extends MDWObject> MDWList<T> getInstances(EClass type);
    
    /**
     * Returns all model elements of the specified type, or one of its sub types, attached to this model.
     * 
     * <p>The returned list is a <code>MDWList</code> so you can directly invoke scripts on the model elements.
     * This list has no predictable order.</p>
     * 
     * @param type the type of model elements to retrieve.
     * @param includeSubTypes determines wheter to retrieve elements based on sub types.
     * @return an unmodifiable list of <code>MDWObject</code> contained in this model that matches the specified type, or one of its sub types.
     * @throws UnsupportedOperationException if this method is not supported by a particular model implementation
     * @see MDWObject
     * @see #getInstances(String)
     * @since MDWorkbench 3.0.0
     */
    public <T extends MDWObject> MDWList<T> getInstances(EClass type, boolean includeSubTypes);
    
    /**
     * Returns model elements of the specified type attached to this model, which match the specified profile name.
     * This method does not consider sub types of the specified type.
     * 
     * <p>The specified type name can be a fully qualified class (<code>"com.sodius.mdw.metamodel.uml21.Class"</code> for example),
     * or a simple name (<code>"Class"</code>), if this name is unique in the metamodel.</p>
     * 
     * <p>The returned list is a <code>MDWList</code> so you can directly invoke scripts on the model elements.
     * These elements are ordered the way they were created and added to this model.</p>
     * 
     * @param typeName the type name of model elements to retrieve.
     * @param profileName the name of the profile type the instances must match.
     * @return an unmodifiable list of <code>MDWObject</code> contained in this model that matches the specified type.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if this API is not called during an MDWorkbench evaluation.
     * @throws UnsupportedOperationException if this method is not supported by a particular model implementation
     * @see MDWObject
     * @see #getProfiledInstances(String, String, boolean)
     */
    public <T extends MDWObject> MDWList<T> getProfiledInstances(String typeName, String profileName);
    
    /**
     * Returns all model elements of the specified type, or one of its sub types, attached to this model, 
     * which match the specified profile name.
     * 
     * <p>The specified type name can be a fully qualified class (<code>"com.sodius.mdw.metamodel.uml21.Classifier"</code> for example),
     * or a simple name (<code>"Classifier"</code>), if this name is unique in the metamodel.</p>
     * 
     * <p>The returned list is a <code>MDWList</code> so you can directly invoke scripts on the model elements.
     * This list has no predictable order.</p>
     * 
     * @param typeName the type name of model elements to retrieve.
     * @param profileName the name of the profile type the instances must match.
     * @param includeSubTypes determines wheter to retrieve elements based on sub types.
     * @return an unmodifiable list of <code>MDWObject</code> contained in this model that matches the specified type, or one of its sub types.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if this API is not called during an MDWorkbench evaluation.
     * @throws UnsupportedOperationException if this method is not supported by a particular model implementation
     * @see MDWObject
     * @see #getProfiledInstances(String, String)
     */
    public <T extends MDWObject> MDWList<T> getProfiledInstances(String typeName, String profileName, boolean includeSubTypes);
    
    /**
     * Returns model elements of the specified type attached to this model, which match the specified profile name.
     * This method does not consider sub types of the specified type.
     * 
     * <p>The returned list is a <code>MDWList</code> so you can directly invoke scripts on the model elements.
     * These elements are ordered the way they were created and added to this model.</p>
     * 
     * @param type the type of model elements to retrieve.
     * @param profileName the name of the profile type the instances must match.
     * @return an unmodifiable list of <code>MDWObject</code> contained in this model that matches the specified type.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if this API is not called during an MDWorkbench evaluation.
     * @throws UnsupportedOperationException if this method is not supported by a particular model implementation
     * @see MDWObject
     * @see #getProfiledInstances(EClass, String, boolean)
     * @since MDWorkbench 3.0.0
     */
    public <T extends MDWObject> MDWList<T> getProfiledInstances(EClass type, String profileName);
    
    /**
     * Returns all model elements of the specified type, or one of its sub types, attached to this model, 
     * which match the specified profile name.
     * 
     * <p>The returned list is a <code>MDWList</code> so you can directly invoke scripts on the model elements.
     * This list has no predictable order.</p>
     * 
     * @param type the type of model elements to retrieve.
     * @param profileName the name of the profile type the instances must match.
     * @param includeSubTypes determines wheter to retrieve elements based on sub types.
     * @return an unmodifiable list of <code>MDWObject</code> contained in this model that matches the specified type, or one of its sub types.
     * @throws com.sodius.mdw.core.eval.InvalidCallException if this API is not called during an MDWorkbench evaluation.
     * @throws UnsupportedOperationException if this method is not supported by a particular model implementation
     * @see MDWObject
     * @see #getProfiledInstances(EClass, String)
     * @since MDWorkbench 3.0.0
     */
    public <T extends MDWObject> MDWList<T> getProfiledInstances(EClass type, String profileName, boolean includeSubTypes);
    
    /**
     * Creates a new instance of a <code>MDWObject</code> based on the specified type.
     * The created element is automaticaly registered to this model, there's no need to call the <code>add(MDWObject)</code> method.
     * 
     * <p>The specified type name can be a fully qualified class (<code>"com.sodius.mdw.metamodel.uml21.Class"</code> for example),
     * or a simple name (<code>"Class"</code>), if this name is unique in the metamodel. This must be a concrete (i.e. instantiable) type.</p>
     * 
     * <p>If this model is based on EMF, the EMF factory registered to the <code>EPackage</code> of the related metamodel is used
     * to instantiate the model element.</p>
     * 
     * @throws UnsupportedOperationException if this method is not supported by a particular model implementation
     */
    public <T extends MDWObject> T create(String typeName);
    
    /**
     * Creates a new instance of a <code>MDWObject</code> based on the specified type.
     * The created element is automaticaly registered to this model, there's no need to call the <code>add(MDWObject)</code> method.
     * 
     * <p>If this model is based on EMF, the EMF factory registered to the <code>EPackage</code> of the related metamodel is used
     * to instantiate the model element.</p>
     * 
     * @throws UnsupportedOperationException if this method is not supported by a particular model implementation
     * @since MDWorkbench 3.0.0
     */
    public <T extends MDWObject> T create(EClass type);
    
    /**
     * Register this model element to the model. 
     * Has no effect if the element is already part of this model.
     * 
     * <p>This method is only to be used if the element has been created outside of a <code>Model</code>,
     * for example using an EMF factory directly. If the model element has been created using 
     * the {@link #create(String)} method, the element is already part of the model.</p>
     * 
     * @param object the model element to be registered.
     */
    public void add(MDWObject object);
    
    /**
     * Unregister this model element from the model.
     * Has no effect if the model element is not part of this model.
     * 
     * <p>This method only detach the model element from this model. 
     * To completely destroy the object and ensure it's not accessible from other elements, 
     * you should rather use the <code>eRemove()</code> method on this object.</p>
     * 
     * @see MDWObject#eRemove()
     */
    public void remove(MDWObject object);
    
    /**
     * Clears the contents of this model.
     */
    public void clear();
    
    /**
     * Get the diagram image for a specific model element.
     * 
     * <p>This method uses the diagram provider of the model reader that loaded this model.
     * If this model is not loaded by a model reader, or if the model reader does not define
     * a diagram provider, this method throws a <code>CoreException</code>.</p>
     * 
     * <p>Please note that many model readers won't be able to support diagrams (XMI reader for example).
     * To support diagrams, a model reader should generally have a live connection with a CASE tool,
     * through COM APIs for example, to be able to dynamically retrieve images.</p>
     * 
     * @param object a model element which is a logical diagram, or which has an attached diagram.
     * @param options some options used to retrieve the diagram image (may be <code>null</code>).
     * @return the diagram image.
     * @throws CoreException if no diagram image could be retrieved.
     * 
     * @see com.sodius.mdw.core.model.diagram.DiagramProvider
     * @since 2.1.0
     */
    public DiagramImage getDiagram(MDWObject object, DiagramOptions options) throws CoreException;
    
}
