package com.sodius.mdw.core.model;

import java.io.StringReader;
import java.util.Collections;
import java.util.Map;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.sodius.mdw.core.OperationCanceledException;
import com.sodius.mdw.core.util.io.XHTMLDefaultHandler;
import com.sodius.mdw.corext.util.io.SAXParserPool;
import com.sodius.mdw.internal.core.model.richtext.RichTextConverter;
import com.sodius.mdw.internal.core.model.richtext.RichTextSimplifier;

/**
 * <p>Holds a rich text representation, whose underlying format can either be XHTML or RTF.
 * Conversion methods are provided to convert from one format to another (eg. RTF to XHTML).</p>
 * 
 * <p>It is worth noting it is not intended to be a general RTF-XHTML converter.
 * Only a subset of the RTF and XHTML specifications are covered here.
 * The converters have been designed to convert only what can be edited in the Model Editor.
 * Any non-handled RTF or XHTML construct is simply ignored during the conversion process.</p>
 * 
 * @since MDWorkbench 3.1.0
 */
public final class RichText {
    
    private static final RichText EMPTY_PLAIN_TEXT = new RichText("", RichTextType.PLAIN_TEXT_LITERAL);
    private static final RichText EMPTY_XHTML = new RichText("", RichTextType.XHTML_LITERAL);
    private static final RichText EMPTY_RTF = new RichText("", RichTextType.RTF_LITERAL);
    
    /**
     * Options to specify the XHTML doctype to use. 
     * Default value is Transitional.
     */
    public static final String OPTION_XHTML_DOCTYPE = "XHTML_DOCTYPE";
    
    /**
     * Doctype value to tell the converter to ignore the doctype.
     */
    public static final String DOCTYPE_OMIT = "OMIT";
    
    /**
     * XHTML Strict doctype.
     */
    public static final String DOCTYPE_STRICT = "DOCTYPE_STRICT";
    
    /**
     * XHTML Transitional doctype
     */
    public static final String DOCTYPE_TRANSITIONAL = "TRANSITIONAL";
    
    /**
     * Option to determine if the converter should only output the contents of the body tag as an HTML fragment.
     * Useful for incorporating text fragments as a portion of another page.
     * Default value is <code>false</code>.
     */
    public static final String OPTION_XHTML_SHOW_BODY_ONLY = "XHTML_SHOW_BODY_ONLY";
    
    /**
     * Option to specify the <code>String</code> used to prefix in plain text an ordered list item.
     * Default value is <code>.</code>
     */
    public static final String OPTION_PLAIN_ORDERED_LIST_STRING = "PLAIN_ORDERED_LIST_STRING";
    
    /**
     * Option to specify the <code>String</code> used to prefix in plain text an unordered list item.
     * Default value is <code>-</code>
     */
    public static final String OPTION_PLAIN_UNORDERED_LIST_STRING = "PLAIN_UNORDERED_LIST_STRING";

    /**
     * Creates a new RichText instance wrapping the specified text.
     * This method will try to guess the underlying format of the text.
     * 
     * @param text the text to wrap as RichText
     * @return a new instance of RichText
     * @see #valueOf(String, RichTextType)
     */
    public static RichText valueOf(String text) {
        if (text == null || text.length() == 0)
            return valueOf(text, RichTextType.PLAIN_TEXT_LITERAL);
        else if (isRTF(text))
            return valueOf(text, RichTextType.RTF_LITERAL);
        else if (isXHTML(text))
            return valueOf(text, RichTextType.XHTML_LITERAL);
        else
            return valueOf(text, RichTextType.PLAIN_TEXT_LITERAL); 
    }
    
    /**
     * Creates a new RichText instance wrapping the specified text,
     * whose underlying format is known by the caller.
     * 
     * @param text the text to wrap as RichText
     * @param type the underlying format of the text
     * @return a new instance of RichText
     * @see #valueOf(String)
     */
    public static RichText valueOf(String text, RichTextType type) {
        
        // rtf?
        if (type == RichTextType.RTF_LITERAL) {
            if (text == null || text.length() == 0)
                return EMPTY_RTF;
            else
                return new RichText(text, type);
        }
        
        // xhtml?
        else if (type == RichTextType.XHTML_LITERAL) {
            if (text == null || text.length() == 0)
                return EMPTY_XHTML;
            else
                return new RichText(text, type);
        }
        
        // plain text?
        else {
            if (text == null || text.length() == 0)
                return EMPTY_PLAIN_TEXT;
            else
                return new RichText(text, type);
        }
    }
    
    private static boolean isXHTML(String value) {
        
        // quick guess
        if (value.startsWith("<html>") || value.startsWith("<html "))
            return true;
        if (value.indexOf("<!DOCTYPE html") >= 0)
            return true;
        
        // parse
        else if (value.startsWith("<?xml")) {
            final boolean[] isXHTML = new boolean[1];
            try {
                SAXParserPool.getInstance().parse(new InputSource(new StringReader(value)), new XHTMLDefaultHandler() {
                    @Override
                    public void startElement(String uri, String localName, String name, Attributes attributes) throws SAXException {
                        isXHTML[0] = "html".equals(name);
                        throw new SAXException(new OperationCanceledException());
                    }
                });
            }
            catch (Throwable e) {
                // ignore
            }
            return isXHTML[0];
        }
        
        else
            return false;
    }
    
    private static final boolean isRTF(String value) {
        return value.startsWith("{\\rtf");
    }

    private String value;
    private RichTextType type;
    private String plainTextCache;
    
    private RichText(String value, RichTextType type) {
        this.value = value;
        this.type = type;
    }
    
    /**
     * Returns the wrapped text as is.
     * 
     * @return the wrapped text
     */
    public String getValue() {
        return value;
    }
    
    @Override
    public int hashCode() {
        return getValue().hashCode();
    }
    
    /**
     * Return <code>true</code> if the argument is a RichText of same type wrapping the same text. 
     */
    @Override
    public boolean equals(Object obj) {
        if (obj == this)
            return true;
        else if (obj instanceof RichText)
            return getClass() == obj.getClass() && getValue().equals(((RichText) obj).getValue());
        else
            return false;
    }
    
    /**
     * Return the format of the underlying text.
     * 
     * @return the format of the underlying text
     */
    public RichTextType getType() {
        return type;
    }
    
    /**
     * Returns <code>true</code> if the underlying value length is <code>0</code>.
     * 
     *  @return <code>true</code> if the underlying value length is <code>0</code>, <code>false</code> otherwise.
     */
    public boolean isEmpty() {
        return getValue().length() == 0;
    }
    
    /**
     * Converts the wrapped text into a plain text format.
     * 
     * @return the plain text conversion result
     */
    public String toPlainText() {
        return convertValue(RichTextType.PLAIN_TEXT_LITERAL, null);
    }
    
    /**
     * Converts the wrapped text into a plain text format.
     * 
     * @param options Options that configure the conversion process
     * @return the plain text conversion result
     */
    public String toPlainText(Map<String, ?> options) {
        return convertValue(RichTextType.PLAIN_TEXT_LITERAL, options);
    }
    
    /**
     * Converts the wrapped text into a RTF format.
     * 
     * @return the RTF conversion result
     */
    public final String toRTF() {
        return convertValue(RichTextType.RTF_LITERAL, null);
    }
    
    /**
     * Converts the wrapped text into a RTF format.
     * 
     * @param options Options that configure the conversion process
     * @return the RTF conversion result
     */
    public String toRTF(Map<String, ?> options) {
        return convertValue(RichTextType.RTF_LITERAL, options);
    }
    
    /**
     * Converts the wrapped text into an XHTML format.
     * 
     * @return the XHTML conversion result
     */
    public String toXHTML() {
        return convertValue(RichTextType.XHTML_LITERAL, null);
    }

    /**
     * Converts the wrapped text into an XHTML format.
     * 
     * @param options Options that configure the conversion process
     * @return the XHTML conversion result
     */
    public String toXHTML(Map<String, ?> options) {
        return convertValue(RichTextType.XHTML_LITERAL, options);
    }

    /**
     * Converts the wrapped text into the specified format.
     * 
     * @return the conversion result wrapped as a new RichText instance.
     */
    public RichText convert(RichTextType type) {
        return convert(type, null);
    }

    /**
     * Converts the wrapped text into the specified format.
     * 
     * @param options Options that configure the conversion process
     * @return the conversion result wrapped as a new RichText instance.
     */
    public final RichText convert(RichTextType type, Map<String, ?> options) {
        return RichText.valueOf(convertValue(type, options));
    }
    
    private String convertValue(RichTextType type, Map<String, ?> options) {
        if (getValue().length() == 0)
            return "";
        
        if (options == null)
            options = Collections.emptyMap();
        
        // plain text -> use cache
        if (type == RichTextType.PLAIN_TEXT_LITERAL && options.isEmpty()) {
            if (plainTextCache == null) {
                plainTextCache = RichTextConverter.getInstance(getType(), type).convert(getValue(), options);
                
                // try to optimize plain text cache
                if (plainTextCache != value) {
                    if (plainTextCache.equals(value))
                        plainTextCache = value;
                    else {
                        int index = value.indexOf(plainTextCache);
                        if (index >= 0)
                            plainTextCache = value.substring(index, index + plainTextCache.length());
                    }
                }
            }
            return plainTextCache;
        }
        else {
            return RichTextConverter.getInstance(getType(), type).convert(getValue(), options);
        }
    }
    
    /**
     * <p>Returns the most lightweight type of rich text that can represents the underlying value.</p>
     * <p>If the underlying value if a non formatted RTF of XHTML text, it returns the plain text version.
     * If the underlying value is a formatted RTF text, it returns the XHTML version.</p>
     * 
     * @return the most lightweight type of rich text that can represents the underlying value.
     */
    public RichText simplify() {
        if (getValue().length() == 0)
            return this;
        else
            return RichText.valueOf(RichTextSimplifier.getInstance(getType()).simplify(getValue()));
    }
    
    /**
     * Returns the plain text version of the underlying text.
     * 
     * @return the plain text version of the underlying text.
     */
    public final String toString() {
        return toPlainText();
    }
}
