package com.sodius.mdw.core.model.emf;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.Resource.IOWrappedException;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.FeatureNotFoundException;
import org.eclipse.emf.ecore.xmi.IllegalValueException;
import org.eclipse.emf.ecore.xmi.XMIException;

import com.sodius.mdw.core.CoreException;
import com.sodius.mdw.core.OperationCanceledException;
import com.sodius.mdw.core.model.MDWObject;
import com.sodius.mdw.core.model.MetamodelExtension;
import com.sodius.mdw.core.model.Model;
import com.sodius.mdw.core.model.io.ConnectorContext;
import com.sodius.mdw.core.model.io.ModelReader;
import com.sodius.mdw.core.util.ClassUtils;
import com.sodius.mdw.core.util.io.IOUtils;
import com.sodius.mdw.corext.nls.MessageUtils;
import com.sodius.mdw.corext.nls.Messages;
import com.sodius.mdw.internal.core.model.AbstractModel;

/**
 * Provides a skeletal implementation for EMF based model readers. 
 * 
 * <p>This class acts as a bridge between EMF <code>Resource</code> and MDWorkbench <code>ModelReader</code>.
 * Subclasses are required to implement the <code>createResource</code> method. 
 * This class will delegate the loading to the created resource.
 * Then it will attach to the model every created EMF model element.</p>
 * 
 * <p>A default implementation is available to handle XMI uris.</p> 
 * 
 * <p>Clients may subclass this class.</p>
 */
public abstract class EMFModelReader implements ModelReader {
    
    /**
     * Option that gives access to the connector context.
     * A resource <code>load</code> method can retrieve the connector context this way :<br>
     * <code>ConnectorContext context = (ConnectorContext) options.get(EMFModelReader.MDW_CONNECTOR_CONTEXT);</code>
     */
    public static final String MDW_CONNECTOR_CONTEXT = "MDW_CONNECTOR_CONTEXT"; //$NON-NLS-1$
    
    public static final String OPTION_DISABLE_AUTO_RESOLVE_PROXIES = "DISABLE_AUTO_RESOLVE_PROXIES"; //$NON-NLS-1$
    
    /**
     * Defines a resource set that delegates the creation of a resource to the <code>EMFModelReader</code> implementation.
     */
    private class ModelResourceSet extends ResourceSetImpl {
        
        private Model model;
        private ConnectorContext context;
        
        ModelResourceSet(Model model, ConnectorContext context) {
            this.model = model;
            this.context = context;
        }
        
        /**
         * Delegates the creation of the resource.
         * 
         * @param uri the URI of the resource to create.
         * @return the created resource.
         * @see EMFModelReader#createResource(URI, ConnectorContext)
         */
        public Resource createResource(URI uri) {
            Resource resource = EMFModelReader.this.createResource(uri, context);
            getResources().add(resource);
            return resource;
        }
        
        /*
         * in EMF 2.4, this method does not call anymore createResource(uri)
         * bug id=2652: [Junit] UML2 Junit Test fail with Eclipse 3.4 
         */
        protected Resource demandCreateResource(URI uri) {
            return createResource(uri);
        }
        
        protected void demandLoad(Resource resource) throws IOException {
            
            // load resource
            super.demandLoad(resource);
            
            // add elements to model (only if truly on demand loader)
            Object option = this.context.getConnectorDescriptor().getProperty(OPTION_DISABLE_AUTO_RESOLVE_PROXIES);
            if (Boolean.TRUE.equals(option) || Boolean.TRUE.toString().equals(option)) {
                for (Iterator<EObject> i = resource.getAllContents(); i.hasNext(); ){
                    Object object = i.next();
                    if (object instanceof MDWObject)
                        this.model.add((MDWObject) object);
                }
            }
        }
    }

    /**
     * Reads the specified uri. This implementation reads a model in four steps :
     * <ul>
     * <li>creates a <code>Resource</code>.</li>
     * <li>creates options to pass to the resource.</li>
     * <li>calls resource <code>load</code> method.</li>
     * <li>attach resource model elements to the specified model.</li>
     * </ul>
     * 
     * <p>The context can be used to access a variery of properties and is to be used to check for cancellation.</p>
     * 
     * @param filepath the location of a resource to read.
     * @param model the model in which model elements must be created.
     * @param context the context in which this reader runs. 
     * @throws CoreException if anything prevents the connector to read the model.
     */
    public void read(String filepath, Model model, ConnectorContext context) throws CoreException {
        
        // create resource
        ResourceSet resourceSet = createResourceSet(model, context);
        URI uri = IOUtils.createURI(filepath);
        Resource resource = resourceSet.createResource(uri);
        
        try {
            // create options
            Map<String, Object> options = createOptions(context);
            resourceSet.getLoadOptions().putAll(options);
            
            // load resource
            resource.load(options);
        }
        catch (IOWrappedException e) {
            Throwable wrapped = e.getCause();
            List<Resource.Diagnostic> errors = resource.getErrors();
            Exception first = null;
            Throwable firstWrapped = null;
            if (! errors.isEmpty()) {
                first = (Exception) errors.get(0);
                if (first instanceof XMIException) 
                    firstWrapped = ((XMIException) first).getCause();
                else
                    firstWrapped = first.getCause();
            }
                
            
            // only throw this exception if this is not the first one in the known errors
            if (! wrapped.equals(first) && ! wrapped.equals(firstWrapped)) {
                if (wrapped instanceof OperationCanceledException)
                    throw (OperationCanceledException) wrapped;
                else if (wrapped instanceof CoreException)
                    throw (CoreException) wrapped;
                else
                    throw new CoreException(wrapped);
            }
        }
        catch (IOException e) {
            if (e.getCause() instanceof CoreException)
                throw (CoreException) e.getCause();
            else
                throw new CoreException(e);
        }
        finally {
            
            // add elements to model
            addModelElements(model, resourceSet);
            
            // log
            logErrors(context, resourceSet);
        }
        
    }
    
    private void addModelElements(Model model, ResourceSet resourceSet) {
        
        // compute available extensions
        Map<EPackage, MetamodelExtension> extensions = new HashMap<EPackage, MetamodelExtension>();
        Set<MetamodelExtension> appliedExtensions = new HashSet<MetamodelExtension>();
        for (Iterator<MetamodelExtension> i = model.getApplicableMetamodelExtensions().iterator(); i.hasNext(); ) {
            MetamodelExtension extension = i.next();
            extensions.put(extension.getEPackage(), extension);
        }
        
        // compute extensions and model elements to attach
        List<MDWObject> objectsToAdd = new ArrayList<MDWObject>();
        for (Iterator<Resource> i = new ArrayList<Resource>(resourceSet.getResources()).iterator(); i.hasNext(); ) {
            Resource resource = i.next();
            
            for (Iterator<EObject> j = resource.getAllContents(); j.hasNext(); ){
                Object object = j.next();
                
                // model element?
                if (object instanceof MDWObject) {
                    objectsToAdd.add((MDWObject) object);
                }
                
                // decoration
                else if (object instanceof EObject) {
                    EPackage ePackage = ((EObject) object).eClass().getEPackage();
                    if (extensions.containsKey(ePackage)) {
                        appliedExtensions.add(extensions.get(ePackage));
                    }
                }
            }
        }
        
        // apply extension
        ((AbstractModel) model).setAppliedMetamodelExtensions(appliedExtensions);
        
        // attach model elements now that extensions are known
        for (Iterator<MDWObject> i = objectsToAdd.iterator(); i.hasNext(); )
            model.add(i.next());
    }
    
    /**
     * Creates a resource set based on this connector context
     * 
     * @param model the model in which model elements must be created.
     * @param context the connector context to use.
     * @return a resource set used to create a resource.
     */
    protected ResourceSet createResourceSet(Model model, ConnectorContext context) {
        ResourceSet resourceSet = new ModelResourceSet(model, context);
        
        // Bug id=323
        // Force the implementation of this EPackage in the registry:
        // ATL defines a KM3 metamodel, so two KM3 metamodels shares the same uri. We force the uri to resolve to our EPackage.
        EPackage ePackage = context.getConnectorDescriptor().getMetamodel().getEPackage();
        resourceSet.getPackageRegistry().put(ePackage.getNsURI(), ePackage);
        
        return resourceSet;
    }

    /**
     * Creates a resource that is able to handle the specified uri.
     * 
     * @param uri the URI to load.
     * @param context the context in which the connector runs
     * @return the created resource.
     */
    protected abstract Resource createResource(URI uri, ConnectorContext context);
    
    /**
     * Create options to pass to the created resource.
     * The default implementation set as option every property defined on the connector context,
     * and add <code>MDW_CONNECTOR_CONTEXT</code> to give access to the context.
     * 
     * @param context the context in which the connector runs
     * @return the options to pass to the resource.
     */
    protected Map<String, Object> createOptions(ConnectorContext context) {
        Map<String, Object> options = new HashMap<String, Object>();
        options.put(MDW_CONNECTOR_CONTEXT,context);
        
        for (Iterator<String> i = context.getPropertyNames().iterator(); i.hasNext(); ) {
            String name = i.next();
            options.put(name, context.getProperty(name));
           
        }
        
        return options;
    }
    
    private void logErrors(ConnectorContext context, ResourceSet resourceSet) {
        for (Iterator<Resource> i = resourceSet.getResources().iterator(); i.hasNext(); ) {
            Resource resource = i.next();
            
            for (Iterator<Resource.Diagnostic> j = resource.getErrors().iterator(); j.hasNext(); ) {
                Resource.Diagnostic error = (Resource.Diagnostic) j.next();
                logError(context, error);
            }
        }
    }
    
    /**
     * Logs the specified diagnostic to the registered logger.
     */
    protected void logError(ConnectorContext context, Resource.Diagnostic error) {
        
        // format message
        String message = error.getMessage();
        int index = message.indexOf("(file:"); //$NON-NLS-1$
        if (index > 0)
            message = message.substring(0, index);
        
        // format XMI messages
        if (error instanceof FeatureNotFoundException) {
            FeatureNotFoundException e = (FeatureNotFoundException) error;
            message = MessageUtils.format(Messages.EMFModelReader_messageFeatureNotFound, 
                    e.getName(), 
                    e.getObject().eClass().getName());
        }
        else if (error instanceof IllegalValueException) {
            IllegalValueException e = (IllegalValueException) error;
            StringBuffer buffer = new StringBuffer();
            
            String typeName = ""; //$NON-NLS-1$
            if (e.getValue() != null) {
                if (e.getValue() instanceof EObject)
                    typeName = ((EObject) e.getValue()).eClass().getName();
                else
                    typeName = ClassUtils.getSimpleName(e.getValue().getClass());
            }
            buffer.append(MessageUtils.format(Messages.EMFModelReader_messageIllegalValue, 
                    String.valueOf(e.getValue()),
                    typeName));
            
            
            
            if (e.getObject() != null) {
                buffer.append(" "); //$NON-NLS-1$
                buffer.append(MessageUtils.format(Messages.EMFModelReader_messageIllegalValueSuffix, 
                        e.getFeature() != null ? e.getFeature().getName() : "",  //$NON-NLS-1$
                            e.getObject().toString(),
                            e.getObject().eClass().getName()));
            }
            
            message = buffer.toString();
        }
        
        // format location
        String location = error.getLocation();
        if (location == null)
            location = ""; //$NON-NLS-1$
        if (location.startsWith("file:")) //$NON-NLS-1$
            location = URI.createURI(location).toFileString();
        
        // log
        context.getLogger().error(message, location, error.getLine(), error.getColumn());
    }

    
}
