package com.sodius.mdw.core.model.emf;

import java.io.IOException;
import java.text.Collator;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;

import com.sodius.mdw.core.CoreException;
import com.sodius.mdw.core.model.MDWObject;
import com.sodius.mdw.core.model.Model;
import com.sodius.mdw.core.model.io.ConnectorContext;
import com.sodius.mdw.core.model.io.ModelWriter;
import com.sodius.mdw.core.util.io.IOUtils;

/**
 * Provides a skeletal implementation for EMF based model writers. 
 * 
 * <p>This class acts as a bridge between EMF <code>Resource</code> and MDWorkbench <code>ModelWriter</code>.
 * Subclasses are required to implement the <code>createResource</code> method. 
 * This class will delegate the writing to the created resource.</p>
 * 
 * <p>A default implementation is available to handle XMI uris.</p> 
 * 
 * <p>Clients may subclass this class.</p>
 */
public abstract class EMFModelWriter implements ModelWriter {

    /**
     * Option that gives access to the connector context.
     * A resource <code>save</code> method can retrieve the connector context this way :<br>
     * <code>ConnectorContext context = (ConnectorContext) options.get(EMFModelWriter.MDW_CONNECTOR_CONTEXT);</code>
     */
    public static final String MDW_CONNECTOR_CONTEXT = "MDW_CONNECTOR_CONTEXT"; //$NON-NLS-1$
    
    /**
     * Writes the specified model to the specified uri. This implementation writes a model in three steps :
     * <ul>
     * <li>creates a <code>Resource</code>.</li>
     * <li>attach model elements to this resource.</li>
     * <li>calls resource <code>save</code> method.</li>
     * </ul>
     * 
     * <p>The context can be used to access a variery of properties and is to be used to check for cancellation.</p>
     * 
     * @param filepath the location of a resource to write.
     * @param model the model to be persisted.
     * @param context the context in which this writer runs. 
     * @throws CoreException if anything prevents the connector to write the model.
     */
    public void write(String filepath, Model model, ConnectorContext context) throws CoreException {
        
        // create resource
        URI uri = IOUtils.createURI(filepath);
        Resource resource = createResource(uri, context);
        
        // register resource contents
        resource.getContents().addAll(getDirectContents(model, context));
        
        // save
        try {
            resource.save(createOptions(context));
        }
        catch (IOException e) {
            if (e.getCause() instanceof CoreException)
                throw (CoreException) e.getCause();
            else
                throw new CoreException(e);
        }
        finally {
            logErrors(context, resource);
        }
    }
    
    /**
     * Creates a resource that is able to handle the specified uri.
     * 
     * @param uri the URI to write.
     * @param context the context in which the connector runs
     * @return the created resource.
     */
    protected abstract Resource createResource(URI uri, ConnectorContext context);
    
    /**
     * Returns the model elements to attach to the create resource.
     * The default implementation returns every <code>EObject</code> that has no container. 
     * 
     * @param model the model to be persisted.
     * @param context the context in which this writer runs. 
     * @return the model elements to attach to the create resource.
     */
    protected Collection<EObject> getDirectContents(Model model, ConnectorContext context) {
        ArrayList<EObject> contents = new ArrayList<EObject>();
        
        for (Iterator<MDWObject> i = model.getInstances().iterator(); i.hasNext(); ) {
            EObject object = i.next();
            if (object.eContainer() == null)
                contents.add(object);
        }
        
        Collections.sort(contents, new Comparator<EObject>() {
        
            public int compare(EObject o1, EObject o2) {
                EClass c1 = o1.eClass();
                EClass c2 = o2.eClass();
                return Collator.getInstance().compare(c1.getName(), c2.getName());
            }
        
        });
        
        return contents;
    }
 
    /**
     * Create options to pass to the created resource.
     * The default implementation set as option every property defined on the connector context,
     * and add <code>MDW_CONNECTOR_CONTEXT</code> to give access to the context.
     * 
     * @param context the context in which the connector runs
     * @return the options to pass to the resource.
     */
    protected Map<String, Object> createOptions(ConnectorContext context) {
        Map<String, Object> options = new HashMap<String, Object>();
        options.put(MDW_CONNECTOR_CONTEXT,context);
        
        for (Iterator<String> i = context.getPropertyNames().iterator(); i.hasNext(); ) {
            String name = i.next();
            options.put(name, context.getProperty(name));
        }
        
        return options;
    }
    
    private void logErrors(ConnectorContext context, Resource resource) {
        for (Iterator<Resource.Diagnostic> i = resource.getErrors().iterator(); i.hasNext(); ) {
            Resource.Diagnostic error = i.next();
            logError(context, error);
        }
    }
    
    /**
     * Logs the specified diagnostic to the registered logger.
     */
    protected void logError(ConnectorContext context, Resource.Diagnostic error) {
        
        // format message
        String message = error.getMessage();
        int index = message.indexOf("(file:"); //$NON-NLS-1$
        if (index > 0)
            message = message.substring(0, index);
        
        // format location
        String location = error.getLocation();
        if (location == null)
            location = ""; //$NON-NLS-1$
        if (location.startsWith("file:")) //$NON-NLS-1$
            location = URI.createURI(location).toFileString();
        
        // log
        context.getLogger().error(message, location, error.getLine(), error.getColumn());
    }
}
