package com.sodius.mdw.core.model.emf;

import java.util.Map;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import com.sodius.mdw.core.model.io.ConnectorContext;
import com.sodius.mdw.corext.model.io.xmi.XMI1Resource;
import com.sodius.mdw.corext.model.io.xmi.XMI2ResourceImpl;
import com.sodius.mdw.corext.model.io.xmi.XMIHeader;
import com.sodius.mdw.corext.model.io.xmi.XMILogger;
import com.sodius.mdw.corext.model.io.xmi.XMIVersion;

/**
 * Default EMF based implementation to handle XMI uris.
 * This implementation is able to deal with XMI versions from 1.0 to 2.1.
 */
public class EMFXMIModelWriter extends EMFModelWriter {
    
    /**
     * Name of the property that specifies the expected XMI version.
     * This property is mandatory. 
     */
    public static final String PROPERTY_XMI_VERSION = "xmiVersion"; //$NON-NLS-1$

    /**
     * This implementation creates a resource to handle XMI based on the XMI version specified
     * in the connector descriptor properties.
     * This method fails if the XMI version property is not set.
     * 
     * @see #PROPERTY_XMI_VERSION
     */
    protected Resource createResource(URI uri, ConnectorContext context) {
        String versionName = (String) context.getConnectorDescriptor().getProperty(PROPERTY_XMI_VERSION);
        if (versionName == null || versionName.length() == 0)
            throw new RuntimeException("Missing property " + PROPERTY_XMI_VERSION); //$NON-NLS-1$
        
        XMIVersion version = XMIVersion.getVersion(versionName);
        if (version == null)
            throw new IllegalArgumentException("Unsupported XMI version " + versionName); //$NON-NLS-1$

        // XMI 2.X ?
        if (version.equals(XMIVersion.XMI_2_0) || version.equals(XMIVersion.XMI_2_1)) {
            Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put("xmi", new XMIResourceFactoryImpl()); //$NON-NLS-1$
            XMI2ResourceImpl resource = new XMI2ResourceImpl(uri);
            resource.setConnectorContext(context);
            setXMIVersion(resource, version);
            return resource;
        }
        
        // XML 1.x
        else {
            XMIHeader header = new XMIHeader(version);
            return new XMI1Resource(uri, context, new XMILogger(context.getLogger()), header);
        }
    }
    
    private void setXMIVersion(XMIResource resource, XMIVersion version) {
        // XMI 2.0 is the default version
        if (version.equals(XMIVersion.XMI_2_0))
            return;
        
        resource.setXMIVersion(version.getVersionName());
    }
    
    /**
     * Registers UTF-8 as the default encoding to use.
     */
    protected Map<String,Object> createOptions(ConnectorContext context) {
        Map<String,Object> options =  super.createOptions(context);
        
        // encoding (check not already specified in connector properties)
        if (! options.containsKey(XMLResource.OPTION_ENCODING))
            options.put(XMLResource.OPTION_ENCODING, "UTF-8"); //$NON-NLS-1$
        
        options.put(XMLResource.OPTION_PROCESS_DANGLING_HREF,  XMLResource.OPTION_PROCESS_DANGLING_HREF_RECORD);
        
        return options;
    }

}
