package com.sodius.mdw.core.model.io;

import com.sodius.mdw.core.MDWorkbench;
import com.sodius.mdw.core.util.PropertySet;
import com.sodius.mdw.core.util.log.Logger;
import com.sodius.mdw.core.util.progress.ProgressMonitor;

/**
 * Describes the context in which a connector runs.
 * This interface provides access to various elements which help to implement a model reader/writer.
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 */
public interface ConnectorContext extends PropertySet {

    /**
     * Returns the descriptor which details the properties of this connector.
     * 
     * <p>The connector descriptor can be helpful to access connector specific properties and to load related resources.</p>
     * 
     * @return the connector descriptor.
     * @see ConnectorDescriptor#getProperty(String)
     * @see ConnectorDescriptor#getConfiguration()
     */
    public ConnectorDescriptor getConnectorDescriptor();
    
    /**
     * Returns the monitor used to report progress and to check for cancellation.
     * 
     * <p>A connector implementation is expected to check regularly whether the process is canceled,
     * in which case it should abort as soon as possible its activity. 
     * You may throw an <code>OperationCanceledException</code> to interrupt the reader/writer.</p>
     * 
     * @return the progress monitor.
     * @see com.sodius.mdw.core.util.progress.ProgressMonitor#isCanceled()
     * @see com.sodius.mdw.core.OperationCanceledException
     */
    public ProgressMonitor getProgressMonitor();
    
    /**
     * Returns the logger used to report errors and warnings, as well as debugging information. 
     * 
     * @return the logger.
     */
    public Logger getLogger();
    
    /**
     * Returns the workbench, which may be used to access configuration properties.
     * The workbench configuration provides for example access to the command line that launched the workbench, 
     * and can determine whether the workbench runs in headless (no user interface) mode. 
     * 
     * @return the workbench.
     * @see com.sodius.mdw.core.WorkbenchConfiguration#getCommandLine()
     * @see com.sodius.mdw.core.WorkbenchConfiguration#isHeadless()
     */
    public MDWorkbench getWorkbench();
    
}
