package com.sodius.mdw.core.model.io;

import java.util.List;
import java.util.Set;

import com.sodius.mdw.core.model.Metamodel;
import com.sodius.mdw.core.model.MetamodelConfiguration;
import com.sodius.mdw.core.util.PropertySet;

/**
 * Describes the properties of a model connector.
 * 
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 * 
 * @see com.sodius.mdw.core.model.Model#read(String, String)
 * @see com.sodius.mdw.core.model.Model#write(String, String)
 */
public interface ConnectorDescriptor extends PropertySet {
    
    /**
     * Boolean property that can be used to tell MDWorkbench the underlying connector does not support multi-threaded environments.
     * In MDWorkbench UI, this means the model should be loaded in the UI event dispatch thread.
     * Default value is false.
     */
    public static final String PROPERTY_SINGLE_THREAD = "mdw.singleThread"; //$NON-NLS-1$
    
    /**
     * String property that can be used to support diagrams in doc templates.
     * This optional property must define the qualified name of a subclass of <code>DiagramProvider</code>.
     * 
     * @see com.sodius.mdw.core.model.diagram.DiagramProvider 
     * @since 2.1.0
     */
    public static final String PROPERTY_DIAGRAM_PROVIDER = "mdw.diagramProvider"; //$NON-NLS-1$
    
    /**
     * Returns the metamodel to which this connector is registered.
     * The connector is available for any model based on this metamodel.
     * 
     * @return the connector metamodel.
     */
    public Metamodel getMetamodel();
    
    /**
     * Returns the configuration from which this connector originates.
     * The configuration can be used to retrieve connector related resources.
     * 
     * <p>This configuration may differ from the metamodel's one if the connector is declared externally.
     * If connector related resources are required, always use this configuration rather than the metamodel's one,
     * as you don't necessary have the knowledge of how this connector is declared.</p>
     * 
     * @return the connector configuration.
     */
    public MetamodelConfiguration getConfiguration();
    
    /**
     * Returns the connector name. 
     * The connector is referenced by its name when reading and writing models.
     * 
     * <p>This name is expected to be unique for a particular metamodel.</p>
     * 
     * @return the connector name.
     * @see com.sodius.mdw.core.model.Model#read(String, String)
     * @see com.sodius.mdw.core.model.Model#write(String, String)
     */
    public String getName();
    
    /**
     * Returns the implementation class name, subclass of <code>ModelReader</code> or <code>ModelWriter</code>, 
     * which is used to do the job of reading/writing the model.
     * 
     * <p>Note : this is an advanced facility that is not needed in regular cases
     * You should not have to consider this property in typical uses, since the model framework 
     * will instantiate and configure by itself the implementation class as requested.</p>
     * 
     * @return the connector implementation class name.
     */
    public String getClassName();
    
    /**
     * Returns the connector kind, which describes the type of uri (resource locator) this connector expects.
     * 
     * @return the connector kind.
     */
    public ConnectorKind getKind();
    
    /**
     * Determines wheter this connector is meant to read models.
     * If this is a reader, then its implementation class name must be a subclass of <code>ModelReader</code>.
     * A connector descriptor is either a reader or a writer, but not both.
     * 
     * @return <code>true</code> if this connector descriptor is a model reader, <code>false</code> otherwise. 
     */
    public boolean isReader();
    
    /**
     * Determines wheter this connector is meant to write models.
     * If this is a writer, then its implementation class name must be a subclass of <code>ModelWriter</code>.
     * A connector descriptor is either a reader or a writer, but not both.
     * 
     * @return <code>true</code> if this connector descriptor is a model writer, <code>false</code> otherwise. 
     */
    public boolean isWriter();

    /**
     * Returns the registered file extension descriptors.
     * These are the file extensions the underlying connector is able to deal with.
     * 
     * <p>Typical use of these extensions is to provide a comprehensive file selection dialog box to the end-user.
     * The file uri passed to the underlying connector is not guaranteed to have one of these extensions.</p>
     * 
     * <p>If the connector does not deal with files, the returned list is likely to be empty but this is not required.
     * Always check the {@link #getKind()} to determine the type of uri this connector expects.</p>
     * 
     * @return an unmodifiable list of registered <code>FileExtensionDescriptor</code>,
     * an empty list if no extensions is registered.
     * @see FileExtensionDescriptor 
     */
    public List<FileExtensionDescriptor> getFileExtensions();
    
    /**
     * Returns the default file extension this connector expects to deal with.
     * 
     * <p>Typical use is to select this extension as the default one in a file selection dialog box.</p>
     * 
     * @return the default file extension descriptor, <code>null</code> if not specified.
     */
    public FileExtensionDescriptor getDefaultFileExtension();
    
    /**
     * Returns the property names registered to this connector.
     * 
     * <p>These properties may comes from the connector descriptor declaration file. 
     * They also can be set dynamically using {@link #setProperty(String, Object)}. 
     * As the underlying <code>ModelReader</code> or <code>ModelWriter</code> has access to this descriptor,
     * this can be a facility to pass extra parameters to configure the reader/writer.</p>
     * 
     * @return an unmodifiable collection of registered property names. 
     */
    public Set<String> getPropertyNames();
    
    /**
     * Returns the value mapped to the specified property name. 
     * Returns <code>null</code> if the property is not defined or if it maps to <code>null</code>.
     * 
     * @param name the property name
     * @return the property value
     */
    public Object getProperty(String name);
    
    /**
     * Associates the specified value with the specified property name.
     * 
     * @param name the property name.
     * @param value the property value (can be <code>null</code>)
     */
    public void setProperty(String name, Object value);

}
