package com.sodius.mdw.core.model.io;

/**
 * Enumeration of known connector kinds, which describe the kind of uri (resource locator) a connector expects.
 * 
 * <p>
 * This class cannot be instantiated or subclassed by clients.
 * </p>
 * @see com.sodius.mdw.core.model.io.ConnectorDescriptor#getKind()
 * @see com.sodius.mdw.core.model.Model#read(String, String)
 * @see com.sodius.mdw.core.model.Model#write(String, String)
 */
public final class ConnectorKind {

    /**
     * File kind : the uri passed to a connector must be a file path.
     */
    public static final ConnectorKind FILE = new ConnectorKind("file"); //$NON-NLS-1$
    
    /**
     * Directory kind : the uri passed to a connector must be a directory path.
     */
    public static final ConnectorKind DIRECTORY = new ConnectorKind("directory"); //$NON-NLS-1$
    
    /**
     * No uri kind : the connector does not need any uri, the uri parameter can be <code>null</code>.
     */
    public static final ConnectorKind NO_URI = new ConnectorKind("noURI"); //$NON-NLS-1$
    
    /**
     * Unspecified kind : the uri can be any arbitrary context specific <code>String</code> value, 
     * in which case the model framework has no specific knowlegde on the uri form to pass to the connector.
     */
    public static final ConnectorKind UNSPECIFIED = new ConnectorKind("unspecified"); //$NON-NLS-1$
    
    /**
     * Determines the connector kind represented as a <code>String</code> value.
     * 
     * @return the corresponding connector kind, <code>UNSPECIFIED</code> if unrecognized.
     */
    public static final ConnectorKind valueOf(String value) {
        if (FILE.name.equals(value))
            return FILE;
        
        else if (DIRECTORY.name.equals(value))
            return DIRECTORY;
        
        else if (NO_URI.name.equals(value))
            return NO_URI;
        
        else
            return UNSPECIFIED;
    }
    
    private String name;
    
    private ConnectorKind(String name) {
        this.name = name;
    }
    
    /**
     * Determines wheter this is a file connector kind.
     * 
     * @return <code>true</code> if this connector is <code>FILE</code>.
     */
    public boolean isFile() {
        return this == FILE;
    }
    
    /**
     * Determines wheter this is a directory connector kind.
     * 
     * @return <code>true</code> if this connector is <code>DIRECTORY</code>.
     */
    public boolean isDirectory() {
        return this == DIRECTORY;
    }
    
    /**
     * Determines wheter this is a no uri connector kind.
     * 
     * @return <code>true</code> if this connector is <code>NO_URI</code>.
     */
    public boolean isNoURI() {
        return this == NO_URI;
    }
    
    /**
     * Determines wheter this is a unspecified connector kind.
     * 
     * @return <code>true</code> if this connector is <code>UNSPECIFIED</code>.
     */
    public boolean isUnspecified() {
        return this == UNSPECIFIED;
    }
    
    /**
     * Returns a <code>String</code> representation of this connector kind.
     * 
     * @return a <code>String</code> representation of this connector kind.
     */
    public String toString() {
        return name;
    }

}
