package com.sodius.mdw.core.model.io;

import com.sodius.mdw.core.CoreException;
import com.sodius.mdw.core.model.Model;

/**
 * Base interface to be implemented by a connector that read models. 
 * 
 * <p>The implementation class specified by the connector descriptor must :
 * <ul>
 * <li>be public</li>
 * <li>implement this interface</li>
 * <li>define a public zero-args constructor</li>
 * </ul>
 * </p>
 * 
 * <p>If you intent to read XML documents, the framework provides facilities you should consider.</p>
 * 
 * <p>Clients may implement this interface.</p>
 * 
 * @see com.sodius.mdw.core.util.io.XMLUtils#newParser()
 * @see com.sodius.mdw.core.util.io.XMLDefaultHandler
 */
public interface ModelReader {
    
    /**
     * Reads a model from the specified uri (resource locator).
     * 
     * <p>The uri denotes a resource location to handle. 
     * The form of the uri depends on the connector kind. 
     * If this is a file connector, this uri is expected to be a valid file path.
     * If this is a "no uri" connector, this uri is expected to be <code>null</code> or an empty <code>String</code>.</p>
     * 
     * <p>The specified model is empty when the reader starts. 
     * The implementation is supposed to read the specified uri resource and to create model elements.
     * Model elements will generally be created using the model {@link Model#create(String)} method.
     * If these elements are created outside of the model (using an EMF factory directly for example), 
     * they have to be attached to the model using the {@link Model#add(MDWObject)} method.</p>
     * 
     * <p>The context can be used to access a variety of properties and is to be used to check for cancellation.</p>
     * 
     * @param uri the location of a resource to read (can be <code>null</code>).
     * @param model the model in which model elements must be created.
     * @param context the context in which this reader runs. 
     * @see ConnectorDescriptor#getKind()
     * @throws CoreException if anything prevents the connector to read the model.
     */
    public void read(String uri, Model model, ConnectorContext context) throws CoreException;

}
