package com.sodius.mdw.core.util;

import java.util.Set;

/**
 * <p>Represents a set of properties.</p>
 * 
 * <p>
 * Clients may implement this interface.
 * </p>
 * 
 * @since 3.1.0
 */
public interface PropertySet {
    
    /**
     * Returns the actual property names defined in this set.
     * 
     * @return an unmodifiable set of registered property names. 
     */
    public Set<String> getPropertyNames();
    
    /**
     * Returns the value mapped to the specified property name. 
     * Returns <code>null</code> if the property is not defined or if it maps to <code>null</code>.
     * 
     * @param name the property name
     * @return the property value
     */
    public Object getProperty(String name);
    
    /**
     * Returns the value mapped to the specified property name,
     * or the specified default value if the property is not found or if it cannot be treated as a <code>String</code>.
     * 
     * @param name the property name
     * @param defaultValue the default value to use if the property is not found
     * @return the property value
     */
    public String getProperty(String name, String defaultValue);
    
    /**
     * Returns the value mapped to the specified property name,
     * or the specified default value if the property is not found or if it cannot be treated as a <code>int</code>.
     * 
     * @param name the property name
     * @param defaultValue the default value to use if the property is not found
     * @return the property value
     */
    public int getProperty(String name, int defaultValue);
    
    /**
     * Returns the value mapped to the specified property name,
     * or the specified default value if the property is not found or if it cannot be treated as a <code>boolean</code>.
     * 
     * @param name the property name
     * @param defaultValue the default value to use if the property is not found
     * @return the property value
     */
    public boolean getProperty(String name, boolean defaultValue);
    
    /**
     * Returns the value mapped to the specified property name,
     * or the specified default value if the property is not found or if it cannot be treated as a <code>float</code>.
     * 
     * @param name the property name
     * @param defaultValue the default value to use if the property is not found
     * @return the property value
     */
    public float getProperty(String name, float defaultValue);
    
    /**
     * Associates the specified value with a property name.
     * 
     * @param name the property name.
     * @param value the property value (can be <code>null</code>)
     */
    public void setProperty(String name, Object value);
    
    /**
     * Associates the specified value with a property name.
     * 
     * @param name the property name.
     * @param value the property value (can be <code>null</code>)
     */
    public void setProperty(String name, String value);
    
    /**
     * Associates the specified value with a property name.
     * 
     * @param name the property name.
     * @param value the property value 
     */
    public void setProperty(String name, int value);
    
    /**
     * Associates the specified value with a property name.
     * 
     * @param name the property name.
     * @param value the property value 
     */
    public void setProperty(String name, boolean value);
    
    /**
     * Associates the specified value with a property name.
     * 
     * @param name the property name.
     * @param value the property value 
     */
    public void setProperty(String name, float value);
}
