package com.sodius.mdw.core.util.io;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.Reader;
import java.io.Writer;

import org.eclipse.emf.common.util.URI;

/**
 * General I/O stream manipulation utilities.
 */
public final class IOUtils {
    
    /**
     * Returns the extension of the specified <code>File</code>.
     * 
     * <p>
     * The file extension portion is defined as the string
     * following the last period (".") character in the name.
     * If there is no period in the name, the path has no
     * file extension portion. If the name ends in a period,
     * the file extension portion is the empty string.
     * </p>
     * 
     * @return the file extension, <code>null</code> if the file has no extension.
     */
    public static String getFileExtension(File file) {
        String name = file.getName();
        int dotIndex = name.lastIndexOf('.');
        if (dotIndex < 0)
            return null;
        else if (dotIndex == name.length() - 1)
            return ""; //$NON-NLS-1$
        else
            return name.substring(dotIndex + 1);
    }
    
    /**
     * Returns the extension of the specified file name.
     * 
     * <p>
     * The file extension portion is defined as the string
     * following the last period (".") character in the name.
     * If there is no period in the name, the path has no
     * file extension portion. If the name ends in a period,
     * the file extension portion is the empty string.
     * </p>
     * 
     * @return the file extension, <code>null</code> if the file has no extension.
     */
    public static String getFileExtension(String filename) {
        return getFileExtension(new File(filename));
    }
    
    /**
     * Returns the absolute form of the specified relative file,
     * using the specified directory to compute this absolute path.
     * 
     * <p>This method does the same job as <code>{@link java.io.File#getAbsoluteFile}</code>,
     * but uses the specified directory to compute the absolute file 
     * instead of the current user directory.
     * 
     * @param relativeFile the relative file.
     * @param directory the reference directory (to use in place of the current user directory).
     * @return the absolute file.
     * @since 2.1.0
     */
    public static File getAbsoluteFile(File relativeFile, File directory) {
        if (relativeFile.isAbsolute())
            return relativeFile;
        
        String userDirectory = System.getProperty("user.dir"); //$NON-NLS-1$
        try {
            String path = null;
            if (directory.exists() && ! directory.isDirectory()) {
                // existing file -> use parent directory 
                path = directory.getParent();
            }
            else {
                path = directory.getPath();
            }
            
            if (path != null)
                System.setProperty("user.dir", path); //$NON-NLS-1$
            
            return relativeFile.getAbsoluteFile();
        }
        finally {
            System.setProperty("user.dir", userDirectory); //$NON-NLS-1$
        }
    }

    /**
     * Get the contents of a <code>File</code> as a <code>String</code> using the specified character encoding.
     * 
     * @param file the file to read from.
     * @param charsetName the encoding to use, <code>null</code> means platform default.
     * @return the file contents as a <code>String</code>.
     * @throws IOException if an I/O error occurs.
     */
    public static String toString(File file, String charsetName) throws IOException {
        FileInputStream is = null;
        Reader reader = null;
        try {
            is = new FileInputStream(file);
            if (charsetName != null)
                reader = new InputStreamReader(is, charsetName);
            else
                reader = new InputStreamReader(is);
        
            StringBuffer buffer = new StringBuffer();
            char[] charBuffer = new char[2048];
            int count;
            while ((count = reader.read(charBuffer)) > 0)
                buffer.append(charBuffer, 0, count);
            return buffer.toString();
        }
        finally {
            IOUtils.closeQuietly(reader);
            IOUtils.closeQuietly(is);
        }
    }
    
    /**
     * Get the contents of a <code>File</code> as a <code>byte</code> array.
     * 
     * @param file the file to read from.
     * @return the file contents as a <code>byte</code> array.
     * @throws IOException if an I/O error occurs.
     */
    public static byte[] toByteArray(File file) throws IOException {
        InputStream is = new FileInputStream(file);
        try {
            return toByteArray(is);
        }
        finally {
            closeQuietly(is);
        }
    }
        
    /**
     * Get the contents of an <code>InputStream</code> as a <code>byte</code> array.
     * 
     * @param input the stream to read from.
     * @return the stream contents as a <code>byte</code> array.
     * @throws IOException if an I/O error occurs.
     */
    public static byte[] toByteArray(InputStream input) throws IOException {
        
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        byte[] byteBuffer = new byte[2048];
        int count;
        while ((count = input.read(byteBuffer)) > 0)
            os.write(byteBuffer, 0, count);
        return os.toByteArray();
    }
    
    /**
     * Copy bytes from an <code>InputStream</code> to an <code>OutputStream</code>.
     * 
     * <p>This method buffers the input internally, so there is no need to use a <code>BufferedInputStream</code>.</p>
     * 
     * @throws IOException if an I/O error occurs
     * @since 2.1.0
     */
    public static void copy(InputStream input, OutputStream output) throws IOException {
        if (! (input instanceof BufferedInputStream))
            input = new BufferedInputStream(input);
        
        if (! (output instanceof BufferedOutputStream))
            output = new BufferedOutputStream(output);
        
        byte[] buffer = new byte[2048];
        int count = 0;
        while ((count = input.read(buffer)) > 0) {
            output.write(buffer, 0, count);
        }
        
        output.flush();
    }
    
    /**
     * Copy bytes from one <code>File</code> to another <code>File</code>.
     * 
     * @throws IOException if an I/O error occurs
     * @throws IllegalArgumentException if input or output is a directory
     * @since MDWorkbench 3.0.0
     */
    public static void copy(File input, File output) throws IOException {
        if (input.isDirectory())
            throw new IllegalArgumentException(input + " is a directory"); //$NON-NLS-1$
        if (output.isDirectory())
            throw new IllegalArgumentException(output + " is a directory"); //$NON-NLS-1$
        
        InputStream is = new BufferedInputStream(new FileInputStream(input));
        try {
            OutputStream os = new BufferedOutputStream(new FileOutputStream(output));
            try {
                copy(is, os);
            }
            finally {
                closeQuietly(os);
            }
        }
        finally {
            closeQuietly(is);
        }
    }
    
    /**
     * Copy characters from a <code>Reader</code> to a <code>Writer</code>.
     * 
     * <p>This method buffers the input internally, so there is no need to use a <code>BufferedReader</code>.</p>
     * 
     * @throws IOException if an I/O error occurs
     * @since MDWorkbench 3.0.0
     */
    public static void copy(Reader reader, Writer writer) throws IOException {
        if (! (reader instanceof BufferedReader))
            reader = new BufferedReader(reader);
        
        if (! (writer instanceof BufferedWriter))
            writer = new BufferedWriter(writer);
        
        char[] buffer = new char[2048];
        int count = 0;
        while ((count = reader.read(buffer)) > 0) {
            writer.write(buffer, 0, count);
        }
        
        writer.flush();
    }
    
    /**
     * Unconditionally close an <code>InputStream</code>.
     * 
     * <p>Equivalent to <code>InputStream.close()</code>, except any exceptions will be ignored. 
     * This is typically used in finally blocks.</p>
     * 
     * @param input the <code>InputStream</code> to close, may be <code>null</code> or already closed
     */
    public static void closeQuietly(InputStream input) {
        try {
            if (input != null)
                input.close();
        }
        catch (IOException e) {
            // ignore
        }
    }
    
    /**
     * Unconditionally close an <code>OutputStream</code>.
     * 
     * <p>Equivalent to <code>OutputStream.close()</code>, except any exceptions will be ignored. 
     * This is typically used in finally blocks.</p>
     * 
     * @param output the <code>OutputStream</code> to close, may be <code>null</code> or already closed
     */
    public static void closeQuietly(OutputStream output) {
        try {
            if (output != null)
                output.close();
        }
        catch (IOException e) {
            // ignore
        }
    }
    
    /**
     * Unconditionally close a <code>Reader</code>.
     * 
     * <p>Equivalent to <code>Reader.close()</code>, except any exceptions will be ignored. 
     * This is typically used in finally blocks.</p>
     * 
     * @param reader the <code>Reader</code> to close, may be <code>null</code> or already closed
     * @since MDWorkbench 3.0.0
     */
    public static void closeQuietly(Reader reader) {
        try {
            if (reader != null)
                reader.close();
        }
        catch (IOException e) {
            // ignore
        }
    }
    
    /**
     * Unconditionally close a <code>Writer</code>.
     * 
     * <p>Equivalent to <code>Writer.close()</code>, except any exceptions will be ignored. 
     * This is typically used in finally blocks.</p>
     * 
     * @param writer the <code>Writer</code> to close, may be <code>null</code> or already closed
     * @since MDWorkbench 3.0.0
     */
    public static void closeQuietly(Writer writer) {
        try {
            if (writer != null)
                writer.close();
        }
        catch (IOException e) {
            // ignore
        }
    }
    
    static boolean isValidScheme(URI uri) {
        return 
            uri.isFile() || 
            URI.isArchiveScheme(uri.scheme()) || // uri.isArchive() available only with EMF >= 2.2
            "platform".equals(uri.scheme()) || //$NON-NLS-1$
            "pathmap".equals(uri.scheme()); //$NON-NLS-1$
    }

    /**
     * Parses an URI string. 
     * This method first checks if the path resolves to an existing file (in terms of <code>File.exists()</code>).
     * Otherwise <code>URI.createURI(String)</code> is called.
     * 
     * @see URI#createURI(String)
     * @since MDWorkbench 3.0.0
     */
    public static URI createURI(String uri) {
        // check File
        File file = new File(uri);
        if (file.exists())
            return URI.createFileURI(uri);
        
        // file: archive: platform: ?
        try {
            URI result = URI.createURI(uri);
            
            // validate the scheme (-> what is understood by URIConverterImpl)
            if (isValidScheme(result)) {
                
                // bug id=3916: Cannot save an Excel XLSX model on Mac 
                if (result.isFile() && result.scheme() == null) {
                    result = URI.createFileURI(uri);
                }
                
                return result;
            }
        }
        catch (Exception e) {
            // ignore
        }
        
        // default to file
        return URI.createFileURI(uri);
    }

    private IOUtils() {
    }

}
