package com.sodius.mdw.core.util.io;

import java.io.ByteArrayInputStream;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import com.sodius.mdw.internal.core.util.StringIntern;

/**
 * Default base class for SAX2 event handlers.
 * 
 * <p>This handler provides two features :
 * <ul>
 * <li>The ability to intern strings, to reduce memory usage.</li>
 * <li>The ability to ignore XML Doctype declaration.</li>
 * </ul>
 * </p>
 * 
 * <p>If you intent to provide an XML connector reader, you should consider to subclass this implementation.</p>
 * 
 * <p>Clients may subclass this class.</p>
 * 
 * @see com.sodius.mdw.core.model.io.ModelReader
 * @see com.sodius.mdw.core.util.io.XMLUtils#newParser()
 */
public class XMLDefaultHandler extends DefaultHandler {

    private StringIntern intern;
    
    /**
     * This implementation ignores external entities. 
     * This will avoid to load the DTD referenced by the XMl document,
     * and particulary avoid to get an exception if this DTD is not available. 
     * 
     * @return always returns an empty input source to ignore this external entity.
     */
    public InputSource resolveEntity(String publicId, String systemId) throws SAXException {
        return new InputSource(new ByteArrayInputStream(new byte[0]));
    }
    
    /**
     * Returns a canonical representation for the specified string object.
     * This can help to reduce memory usage.
     * 
     * <p>Note : the interned pool of strings is local to this handler.
     * Once this handler is garbaged, the whole pool of strings is garbaged as well.</p>
     * 
     * @param value the string to intern.
     * @return a string that has the same contents as this string, but is
     * guaranteed to be from a pool of unique strings.
     * @see String#intern()
     */
    public final String intern(String value) {
        if (intern == null)
            intern = StringIntern.newInstance();
        return intern.intern(value);
    }
}
