package com.sodius.mdw.core.util.io;

import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.io.Writer;
import java.util.Map;

import javax.xml.parsers.SAXParser;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import com.sodius.mdw.core.CoreException;
import com.sodius.mdw.corext.nls.Messages;
import com.sodius.mdw.internal.core.util.io.InternalXMLUtils;

/**
 * General XML related operations.
 * 
 * <p>
 * This class cannot be instantiated or subclassed by clients.
 * </p>
 */
public final class XMLUtils {
    
    /**
     * Options to specify the XML encoding to use. 
     * Default value is UTF-8.
     * 
     * @since MDWorkbench 3.1.0
     */
    public static final String OPTION_ENCODING = "ENCODING";
    
    /**
     * Option to specify the line separator to use when serializing XML streams.
     * Default value is the System line separator, except for XHTML plain text serialization where it is <code>\n</code>.
     * 
     * @since MDWorkbench 3.1.0
     */
    public static final String OPTION_NEW_LINE = "NEW_LINE";
    
    /**
     * Option to determine whether to format an XML serialization.
     * Default value is true.
     * 
     * @since MDWorkbench 3.1.0
     */
    public static final String OPTION_FORMATTED = "OPTION_FORMATTED";
    
    /**
     * Option to determine whether to put the XML declaration on top of the serialized XML stream.
     * Default value is true.
     * 
     * @since MDWorkbench 3.1.0
     */
    public static final String OPTION_DECLARE_XML = "DECLARE_XML";
    
    /**
     * Option to specify the indentation String to use.
     * 
     * @since MDWorkbench 3.1.0
     */
    public static final String OPTION_INDENT_STRING = "INDENT_STRING";
    
    /**
     * Option to determine whether to use a buffered stream when serializing an XML stream.
     * Default value is true.
     * 
     * @since MDWorkbench 3.1.0
     */
    public static final String OPTION_USE_BUFFERED_WRITER = "USE_BUFFERED_WRITER";
    
    /**
     * The XML default encoding to use, which is <code>"UTF-8"</code>.
     */
    public static final String DEFAULT_ENCODING = "UTF-8"; //$NON-NLS-1$
    
    /**
     * Creates a new <code>SAXParser</code> parser instance.
     * 
     * <p>You should then register to this parser a subclass of <code>XMLDefaultHandler</code> as content handler.</p>
     * 
     * @return the new XML parser instance.
     * @throws SAXException if an XML parser can't be provided.
     * @see XMLDefaultHandler
     */
    public static SAXParser newParser() throws SAXException {
        return InternalXMLUtils.newParser();
    }
    
    /**
     * Creates a new <code>XMLWriter</code> instance for the specified file, using the default XML encoding.
     * 
     * @param file the output location where to write XML data.
     * @return a new XML writer.
     * @throws IOException if the file could not be opened for write.
     * @see #DEFAULT_ENCODING
     */
    public static XMLWriter createWriter(File file) throws IOException {
        return InternalXMLUtils.createWriter(file);
    }
    
    /**
     * Creates a new <code>XMLWriter</code> instance for the specified output stream, using the default XML encoding.
     * 
     * <p>You can specify whether the platform must use buffering on top of the provided output stream.
     * It is recommended to use buffering, unless the specified output stream internally already uses buffers.</p>
     * 
     * @param os the output location where to write XML data.
     * @param useBuffer <code>true</code> if the buffering must be used.
     * @return a new XML writer.
     * @throws IOException if the file could not be opened for write.
     * @see #DEFAULT_ENCODING
     */
    public static XMLWriter createWriter(OutputStream os, boolean useBuffer) throws IOException {
        return InternalXMLUtils.createWriter(os, useBuffer);
    }
    
    /**
     * Creates a new <code>XMLWriter</code> instance for the specified output stream, using the serialization options.
     * 
     * @param os the output location where to write XML data.
     * @return a new XML writer.
     * @throws IOException if the file could not be opened for write.
     * @since MDWorkbench 3.1.0
     */
    public static XMLWriter createWriter(OutputStream os, Map<?, ?> options) throws IOException {
        return InternalXMLUtils.createWriter(os, options);
    }
    
    /**
     * Creates a new <code>XMLWriter</code> instance for the specified writer.
     * 
     * <p>You can specify whether the platform must use buffering on top of the provided writer.
     * It is recommended to use buffering, unless the specified writer internally already uses buffers.</p>
     * 
     * @param writer the writer where to write XML data.
     * @param useBuffer <code>true</code> if the buffering must be used.
     * @return a new XML writer.
     * @throws IOException if the file could not be opened for write.
     * @since 2.1.0
     */
    public static XMLWriter createWriter(Writer writer, boolean useBuffer) throws IOException {
        return InternalXMLUtils.createWriter(writer, useBuffer);
    }
    
    /**
     * Creates a new <code>XMLWriter</code> instance for the specified writer, using the serialization options.
     * 
     * @param writer the writer where to write XML data.
     * @return a new XML writer.
     * @throws IOException if the file could not be opened for write.
     * @since MDWorkbench 3.1.0
     */
    public static XMLWriter createWriter(Writer writer, Map<?, ?> options) throws IOException {
        return InternalXMLUtils.createWriter(writer, options);
    }
    
    /**
     * Creates a <code>CoreException</code> based on the specified XML exception.
     * 
     * @param e the exception thrown by the XML parser or writer.
     * @return a new <code>CoreException</code>.
     */
    public static CoreException createCoreException(SAXException e) {
        if (e instanceof SAXParseException) {
            SAXParseException pe = (SAXParseException) e;
            
            StringBuffer buffer = new StringBuffer(pe.getMessage());
            if (pe.getLineNumber() >= 0) {
                buffer.append(" ["); //$NON-NLS-1$
                buffer.append(Messages.XMLUtils_line);
                buffer.append(" ").append(pe.getLineNumber()); //$NON-NLS-1$
                if (pe.getColumnNumber() >= 0) {
                    buffer.append(", "); //$NON-NLS-1$
                    buffer.append(Messages.XMLUtils_column);
                    buffer.append(pe.getColumnNumber());
                }
                buffer.append("]"); //$NON-NLS-1$
            }
            
            return new CoreException(buffer.toString(), e);
        }
        else {
            Throwable cause = e.getException();
            if (cause == null)
                cause = e.getCause();
            if (cause == null)
                return new CoreException(e);
            else if (cause instanceof CoreException)
                return (CoreException) cause;
            else
                return new CoreException(cause);
        }
        
    }

    private XMLUtils() {
    }

}
