package com.sodius.mdw.core.util.io;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

/**
 * Receives notification of the logical content of a document to be written out.
 * 
 * <p>This writer outputs well-formated XML data and encodes text data as necessary.</p>
 * 
 * <p>
 * This interface is not intended to be implemented by clients.
 * </p>
 * 
 * @see com.sodius.mdw.core.util.io.XMLUtils#createWriter(File)
 */
public interface XMLWriter extends ContentHandler {
    
    /**
     * Creates a singleton XML <code>Attributes</code> for the specified name and value.
     * 
     * @param name the attribute name.
     * @param value the attribute value.
     * @return a singleton XML <code>Attributes</code>.
     */
    public Attributes createAttributes(String name, String value);
    
    /**
     * Creates an XML <code>Attributes</code> for the specified names and values.
     * 
     * <p>The specified array must contains pairs of names and values
     * (<code>["name", aNameValue, "id", anID]</code> for example).
     * <code>null</code> values are ignored and won't appear in the XML document.</p>
     * 
     * @param nameAndValues the attributes name and values.
     * @return an XML <code>Attributes</code>.
     */
    public Attributes createAttributes(String[] nameAndValues);
    
    /**
     * Receives notification of character data.
     * 
     * @param text the text data to write.
     * @throws SAXException any SAX exception, possibly wrapping another exception.
     */
    public void characters(String text) throws SAXException;
    
    /**
     * Receives notification of the element with data information but whithout attributes and child.
     * 
     * <p>This method can be used if the XML document does not contain namespaces.</p>
     * 
     * @param qName the qualified name (with prefix).
     * @param data the element text information.
     * @throws SAXException any SAX exception, possibly wrapping another exception.
     */
    public void dataElement(String qName, String data) throws SAXException;
    
    /**
     * Receives notification of the element with data information but whithout attributes and child.
     * 
     * <p>This method can be used if the XML document contains namespaces.</p>
     * 
     * @param uri the Namespace URI.
     * @param localName the local name (without prefix).
     * @param data the element text information.
     * @throws SAXException any SAX exception, possibly wrapping another exception.
     */
    public void dataElement(String uri, String localName, String data) throws SAXException;
    
    /**
     * Receives notification of the element without data information and whithout child.
     * 
     * <p>This method can be used if the XML document does not contain namespaces.</p>
     * 
     * @param qName the qualified name (with prefix).
     * @param attrs the attributes attached to the element.
     * @throws SAXException any SAX exception, possibly wrapping another exception.
     */
    public void emptyElement(String qName, Attributes attrs) throws SAXException;
    
    /**
     * Receives notification of the element without data information and whithout child.
     * 
     * <p>This method can be used if the XML document contains namespaces.</p>
     * 
     * @param uri the Namespace URI.
     * @param localName the local name (without prefix).
     * @param attrs the attributes attached to the element.
     * @throws SAXException any SAX exception, possibly wrapping another exception.
     */
    public void emptyElement(String uri, String localName, Attributes attrs) throws SAXException;
    
    /**
     * Receives notification of the beginning of an element.
     * 
     * <p>This method can be used if the XML document does not contain namespaces
     * and if the XML element has no attribute.</p>
     * 
     * @param qName the qualified name (with prefix).
     * @throws SAXException any SAX exception, possibly wrapping another exception.
     */
    public void startElement(String qName) throws SAXException;
    
    /**
     * Receives notification of the beginning of an element containing attributes.
     * 
     * <p>This method can be used if the XML document does not contain namespaces.</p>
     * 
     * @param qName the qualified name (with prefix).
     * @param attrs the attributes attached to the element.
     * @throws SAXException any SAX exception, possibly wrapping another exception.
     */
    public void startElement(String qName, Attributes attrs) throws SAXException;
    
    /**
     * Receives notification of the beginning of an element.
     * 
     * <p>This method can be used if the XML document contains namespaces
     * and if the XML element has no attribute.</p>
     * 
     * @param uri the Namespace URI.
     * @param localName the local name (without prefix).
     * @throws SAXException any SAX exception, possibly wrapping another exception.
     */
    public void startElement(String uri, String localName) throws SAXException;
    
    /**
     * Receives notification of the beginning of an element containing attributes.
     * 
     * <p>This method can be used if the XML document contains namespaces.</p>
     * 
     * @param uri the Namespace URI.
     * @param localName the local name (without prefix).
     * @param attrs the attributes attached to the element.
     * @throws SAXException any SAX exception, possibly wrapping another exception.
     */
    public void startElement(String uri, String localName, Attributes attrs) throws SAXException;
    
    /**
     * Receives notification of the end of an element.
     * 
     * <p>This method can be used if the XML document does not contain namespaces.</p>
     * 
     * @param qName the qualified name (with prefix).
     * @throws SAXException any SAX exception, possibly wrapping another exception.
     */
    public void endElement(String qName) throws SAXException;
    
    /**
     * Receives notification of the end of an element.
     * 
     * <p>This method can be used if the XML document contains namespaces.</p>
     * 
     * @param uri the Namespace URI.
     * @param localName the local name (without prefix).
     * @throws SAXException any SAX exception, possibly wrapping another exception.
     */
    public void endElement(String uri, String localName) throws SAXException;

}
