package com.sodius.mdw.core.util.log;

import java.io.File;

import com.sodius.mdw.corext.nls.Messages;

/**
 * Skeletal implementation of the <code>Logger</code> interface.
 * Subclasses should generally only have to implement the <code>log()</code> method, 
 * as any other logging facility (<code>debug()</code>, <code>error()</code>, etc.) 
 * is redirected to this method using the corresponding <code>Level</code> information.
 * 
 * <p>
 * Clients may subclass this class.
 * </p>
 */
public abstract class AbstractLogger implements Logger {
    
    private Level level;
    
    /*
     * Javadoc dopied
     */
    public Level getLevel() {
        if (level == null)
            level = Level.INFO;
        return level;
    }
    
    /*
     * Javadoc dopied
     */
    public void setLevel(Level level) {
        this.level = level;
    }
    
    /**
     * Creates a log string based on the specified message and file location.
     * 
     * @param message the information to log (usually a <code>String</code>)
     * @param filename the current file from which this message originates.
     * @param line the current file line number.
     * @param column the current file column number.
     * @return a log information.
     */
    protected String createMessage(Object message, String filename, int line, int column) {
        StringBuffer buffer = new StringBuffer();
        
        if (filename != null && filename.length() > 0) {
            buffer.append("["); //$NON-NLS-1$
            buffer.append(new File(filename).getName());
            
            if (line >= 0) {
                buffer.append(", "); //$NON-NLS-1$
                buffer.append(Messages.XMLUtils_line);
                buffer.append(" ").append(line); //$NON-NLS-1$
                if (column >= 0) {
                    buffer.append(", "); //$NON-NLS-1$
                    buffer.append(Messages.XMLUtils_column);
                    buffer.append(" ").append(column); //$NON-NLS-1$
                }
            }
            
            buffer.append("] "); //$NON-NLS-1$
        }
        
        buffer.append(message);
        
        return buffer.toString();
    }
    
    /**
     * Ask to log the specified message and error.
     * Implementors can safely assume the level has be checked to be loggable at this stage.
     * 
     * @param level the log level (known to be loggable).
     * @param message the information to log (usually a <code>String</code>), maybe <code>null</code>.
     * @param error an exception to log, maybe <code>null</code>.
     */
    protected abstract void log(Level level, Object message, Throwable error);
    
    /*
     * Javadoc dopied
     */
    public boolean isLoggable(Level level) {
        return getLevel().compareTo(level) <= 0;
    }
        
    /*
     * Javadoc dopied
     */
    public void debug(Object message) {
        if (isLoggable(Level.DEBUG))
            log(Level.DEBUG, message, null);
    }
    
    /*
     * Javadoc dopied
     */
    public void debug(Object message, String filename, int lineNumber, int columnNumber) {
        if (isLoggable(Level.DEBUG))
            log(Level.DEBUG, createMessage(message, filename, lineNumber, columnNumber), null);
    }

    /*
     * Javadoc dopied
     */
    public void debug(Object message, Throwable error) {
        if (isLoggable(Level.DEBUG))
            log(Level.DEBUG, message, error);
    }

    /*
     * Javadoc dopied
     */
    public void info(Object message) {
        if (isLoggable(Level.INFO))
            log(Level.INFO, message, null);
    }

    /*
     * Javadoc dopied
     */
    public void info(Object message, String filename, int lineNumber, int columnNumber) {
        if (isLoggable(Level.INFO))
            log(Level.INFO, createMessage(message, filename, lineNumber, columnNumber), null);
    }

    /*
     * Javadoc dopied
     */
    public void info(Object message, Throwable error) {
        if (isLoggable(Level.INFO))
            log(Level.INFO, message, error);
    }

    /*
     * Javadoc dopied
     */
    public void warning(Object message) {
        if (isLoggable(Level.WARNING))
            log(Level.WARNING, message, null);
    }

    /*
     * Javadoc dopied
     */
    public void warning(Object message, String filename, int lineNumber, int columnNumber) {
        if (isLoggable(Level.WARNING))
            log(Level.WARNING, createMessage(message, filename, lineNumber, columnNumber), null);
    }

    /*
     * Javadoc dopied
     */
    public void warning(Object message, Throwable error) {
        if (isLoggable(Level.WARNING))
            log(Level.WARNING, message, error);
    }

    /*
     * Javadoc dopied
     */
    public void error(Object message) {
        if (isLoggable(Level.ERROR))
            log(Level.ERROR, message, null);
    }

    /*
     * Javadoc dopied
     */
    public void error(Object message, String filename, int lineNumber, int columnNumber) {
        if (isLoggable(Level.ERROR))
            log(Level.ERROR, createMessage(message, filename, lineNumber, columnNumber), null);
    }

    /*
     * Javadoc dopied
     */
    public void error(Object message, Throwable error) {
        if (isLoggable(Level.ERROR))
            log(Level.ERROR, message, error);
    }

}
