package com.sodius.mdw.core.util.log;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;

import com.sodius.mdw.core.util.io.IOUtils;

/**
 * Outputs log information into a file.
 * 
 * <p>
 * Clients may instantiate or subclass this class.
 * </p>
 */
public class FileLogger extends AbstractLogger {
    
    private static final byte[] NEW_LINE = System.getProperty("line.separator").getBytes(); //$NON-NLS-1$

    private File file;
    private boolean append;
    
    /**
     * Constructs a logger which outputs log information into the specified file.
     * 
     * <p>If the file exists but is a directory rather than a regular file, 
     * does not exist but cannot be created, or cannot be opened for any other reason 
     * then a <code>FileNotFoundException</code> is thrown.</p>
     * 
     * @param file the file where to log information.
     * @throws FileNotFoundException if the file exists but is a directory rather than a regular file, 
     * does not exist but cannot be created, or cannot be opened for any other reason.
     */
    public FileLogger(File file) throws FileNotFoundException {
        this(file, false);
    }
    
    /**
     * Constructs a logger which outputs log information into the specified file.
     * 
     * <p>If the file exists but is a directory rather than a regular file, 
     * does not exist but cannot be created, or cannot be opened for any other reason 
     * then a <code>FileNotFoundException</code> is thrown.</p>
     * 
     * @param file the file where to log information.
     * @param append if <code>true</code>, then information will be written to the end of the file rather than the beginning.
     * @throws FileNotFoundException if the file exists but is a directory rather than a regular file, 
     * does not exist but cannot be created, or cannot be opened for any other reason.
     */
    public FileLogger(File file, boolean append) throws FileNotFoundException {
        this.file = file;
        this.append = append;
        
        // check can write
        FileOutputStream os = new FileOutputStream(this.file, append);
        IOUtils.closeQuietly(os);
    }
    
    /**
     * Appends the specified message and error to the underlying file.
     * This default implementation always output information the same way, whatever the log level is.
     * 
     * @param level the log level (not considered by the implementation).
     * @param message the information to log (usually a <code>String</code>), maybe <code>null</code>.
     * @param error an exception to log, maybe <code>null</code>.
     */
    protected void log(Level level, Object message, Throwable error) {
        if (message == null && error == null)
            return;
        
        try {
            FileOutputStream os = new FileOutputStream(file, this.append);
            this.append = true; // next log must be appended !
            
            try {
                // message
                if (message != null) {
                    os.write(message.toString().getBytes());
                    os.write(NEW_LINE);
                }
                
                // error
                if (error != null) {
                    PrintStream ps = new PrintStream(os);
                    error.printStackTrace(ps);
                    ps.close();
                }
            }
            finally {
                IOUtils.closeQuietly(os);
            }
        }
        catch (IOException e) {
            // ignore
        }
    }
}
