package com.sodius.mdw.core.util.log;

/**
 * The <code>Level</code> class defines a set of standard logging levels that can be used to control logging output. 
 * Enabling logging at a given level also enables logging at all higher levels.
 * 
 * <p>
 * The levels in descending order are:
 * <ul>
 * <li>ERROR</Li>
 * <li>WARNING</Li>
 * <li>INFO</Li>
 * <li>DEBUG</Li>
 * </ul>
 * </p>
 * 
 * <p>
 * This class cannot be instantiated or subclassed by clients.
 * </p>
 * 
 * @see com.sodius.mdw.core.util.log.Logger#setLevel(Level)
 */
public final class Level {
    
    /**
     * <code>DEBUG</code> level is used to assist in debugging problems that may be associated with
     * particular configurations.
     */
    public static final Level DEBUG = new Level("debug", 0); //$NON-NLS-1$
    
    /**
     * <code>INFO</code> level should only be used for reasonably significant messages that
     * will make sense to end users and system admins.
     */
    public static final Level INFO = new Level("info", 1); //$NON-NLS-1$
    
    /**
     * <code>WARNING</code> level should describe events that will be of interest to end users or system managers, 
     * or which indicate potential problems.
     */
    public static final Level WARNING = new Level("warning", 2); //$NON-NLS-1$
    
    /**
     * <code>ERROR</code> level should describe events that are of considerable importance and which will prevent normal program execution.
     */
    public static final Level ERROR = new Level("error", 3); //$NON-NLS-1$

    private String name;
    private int level;
    
    private Level(String name, int level) {
        this.name = name;
        this.level = level;
    }
    
    /**
     * <p>Compares this level with the specified level for order.  Returns a
     * negative integer, zero, or a positive integer as this level is less
     * than, equal to, or greater than the specified level.</p>
     * 
     * <p>The lowest level is <code>DEBUG</code>, the highest is <code>ERROR</code></p>
     * 
     * @param level the level to compare with.
     * @return a negative integer, zero, or a positive integer as this level is less
     * than, equal to, or greater than the specified level.
     */
    public int compareTo(Level level) {
        return new Integer(this.level).compareTo(new Integer(level.level));
    }
    
    /**
     * Returns <code>true</code> if this level is <code>DEBUG</code>.
     * 
     * @return <code>true</code> if this level is <code>DEBUG</code>, <code>false</code> otherwise.
     */
    public boolean isDebug() {
        return this == DEBUG; 
    }
    
    /**
     * Returns <code>true</code> if this level is <code>INFO</code>.
     * 
     * @return <code>true</code> if this level is <code>INFO</code>, <code>false</code> otherwise.
     */
    public boolean isInfo() {
        return this == INFO; 
    }
    
    /**
     * Returns <code>true</code> if this level is <code>WARNING</code>.
     * 
     * @return <code>true</code> if this level is <code>WARNING</code>, <code>false</code> otherwise.
     */
    public boolean isWarning() {
        return this == WARNING; 
    }
    
    /**
     * Returns <code>true</code> if this level is <code>ERROR</code>.
     * 
     * @return <code>true</code> if this level is <code>ERROR</code>, <code>false</code> otherwise.
     */
    public boolean isError() {
        return this == ERROR; 
    }
    
    /**
     * Returns a <code>String</code> representation of this level.
     * 
     * @return the level as a <code>String</code>.
     */
    public String toString() {
        return name;
    }

}
