package com.sodius.mdw.core.util.log;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Creates records of logged information.
 * 
 * <p>
 * Clients may instantiate or subclass this class.
 * </p>
 */
public class LogRecorder extends AbstractLogger {
    
    /**
     * A <code>Record</code> keeps track of a logged information.
     */
    public static final class Record {
        
        private Object message;
        private Throwable error;
        private Level level;
        
        private Record(Object message, Throwable error, Level level) {
            this.message = message;
            this.error = error;
            this.level = level;
        }
        
        /**
         * Returns the logged message.
         * 
         * @return the logged message, <code>null</code> if no logged message.
         */
        public Object getMessage() {
            return message;
        }
        
        /**
         * Returns the logged error.
         * 
         * @return the logged error, <code>null</code> if no logged error.
         */
        public Throwable getError() {
            return error;
        }
        
        /**
         * Returns the level of the logged information.
         * 
         * @return the level of the logged information.
         */
        public Level getLevel() {
            return level;
        }
    }

    private List<Record> records;
    
    /**
     * Return the list of recorded logged information.
     * 
     * @return an unmodifiable list of <code>Record</code> elements. 
     */
    public List<Record> getRecords() {
        if (records != null)
            return Collections.unmodifiableList(records);
        else
            return Collections.emptyList();
    }
    
    /**
     * Removes all records from this logger.
     */
    public void clear() {
        if (records != null)
            records.clear();
    }
    
    /**
     * Creates a record with the specified log information.
     * 
     * @param level the log level (known to be loggable).
     * @param message the information to log (usually a <code>String</code>), maybe <code>null</code>.
     * @param error an exception to log, maybe <code>null</code>.
     */
    protected void log(Level level, Object message, Throwable error) {
        if (records == null)
            records = new ArrayList<Record>();
        
        Record record = new Record(message, error, level);
        records.add(record);
        recorded(record);
    }
    
    /**
     * Called when a record is created.
     * This default implementation does nothing.
     * 
     * @param record the created record
     */
    protected void recorded(Record record) {
        // nothing
    }
}
