package com.sodius.mdw.core.util.log;

/**
 * Provides facilities to log errors and warnings, as well as debugging information.
 * 
 * <p>One can customize the loggers used by evaluation and model loading/writing, by subclassing this interface
 * and registering the implementation to the workbench configuration.</p>
 * 
 * <p>
 * Clients may implement this interface.
 * </p>
 * 
 * @see com.sodius.mdw.core.WorkbenchConfiguration#getLogger()
 */
public interface Logger {
    
    /**
     * Get the log Level that has been specified for this logger.
     * The default level is <code>INFO</code>.
     * 
     * @return the log Level that has been specified for this logger (never <code>null</code>).
     */
    public Level getLevel();
    
    /**
     * Set the log level specifying which message levels will be logged by this logger. 
     * Message levels lower than this value will be discarded. 
     * 
     * @param level the new level to use.
     */
    public void setLevel(Level level);
    
    /**
     * Check if a message of the given level would actually be logged by this logger. 
     * This check is based on the logger effective level.
     * 
     * @param level a message logging level.
     * @return <code>true</code> if the given message level is currently being logged.
     */
    public boolean isLoggable(Level level);
    
    /**
     * Logs a debug level message.
     * 
     * <p>Debug messages are used to assist in debugging problems that may be associated with
     * particular configurations.</p>
     * 
     * @param message the information to log (usually a <code>String</code>)
     */
    public void debug(Object message);
    
    /**
     * Logs a debug level message.
     * 
     * <p>This method is typically used when parsing a model file and 
     * when a parsing debug information must be logged.</p>
     * 
     * <p>Debug messages are used to assist in debugging problems that may be associated with
     * particular configurations.</p>
     * 
     * @param message the information to log (usually a <code>String</code>)
     * @param filename the current file from which this message originates.
     * @param lineNumber the current file line number.
     * @param columnNumber the current file column number.
     */
    public void debug(Object message, String filename, int lineNumber, int columnNumber);
    
    /**
     * Logs a debug level message.
     * 
     * <p>Debug messages are used to assist in debugging problems that may be associated with
     * particular configurations.</p>
     * 
     * @param message the information to log (usually a <code>String</code>), maybe <code>null</code>
     * @param error an exception to log.
     */
    public void debug(Object message, Throwable error);
    
    /**
     * Logs an info level message.
     * 
     * <p>Info messages should only be used for reasonably significant messages that
     * will make sense to end users and system admins.</p>
     * 
     * @param message the information to log (usually a <code>String</code>)
     */
    public void info(Object message);
    
    /**
     * Logs an info level message, with the specified file location.
     * 
     * <p>This method is typically used when parsing a model file and 
     * when a parsing information must be logged.</p>
     * 
     * <p>Info messages should only be used for reasonably significant messages that
     * will make sense to end users and system admins.</p>
     * 
     * @param message the information to log (usually a <code>String</code>)
     * @param filename the current file from which this message originates.
     * @param lineNumber the current file line number.
     * @param columnNumber the current file column number.
     */
    public void info(Object message, String filename, int lineNumber, int columnNumber);
    
    /**
     * Logs an info level message.
     * 
     * <p>Info messages should only be used for reasonably significant messages that
     * will make sense to end users and system admins.</p>
     * 
     * @param message the information to log (usually a <code>String</code>), maybe <code>null</code>
     * @param error an exception to log.
     */
    public void info(Object message, Throwable error);
    
    /**
     * Logs a warning level message.
     * 
     * <p>In general, warning messages should describe events that will be of interest to end users or system managers, 
     * or which indicate potential problems.</p>
     * 
     * @param message the information to log (usually a <code>String</code>)
     */
    public void warning(Object message);
    
    /**
     * Logs a warning level message.
     * 
     * <p>This method is typically used when parsing a model file and 
     * when a parsing warning must be logged.</p>
     * 
     * <p>In general, warning messages should describe events that will be of interest to end users or system managers, 
     * or which indicate potential problems.</p>
     * 
     * @param message the information to log (usually a <code>String</code>)
     * @param filename the current file from which this message originates.
     * @param lineNumber the current file line number.
     * @param columnNumber the current file column number.
     */
    public void warning(Object message, String filename, int lineNumber, int columnNumber);
    
    /**
     * Logs a warning level message.
     * 
     * <p>In general, warning messages should describe events that will be of interest to end users or system managers, 
     * or which indicate potential problems.</p>
     * 
     * @param message the information to log (usually a <code>String</code>), maybe <code>null</code>
     * @param error an exception to log.
     */
    public void warning(Object message, Throwable error);
    
    /**
     * Logs an error level message.
     * 
     * <p>In general, error messages should describe events that are of considerable importance and which will prevent normal program execution.
     * They should be reasonably intelligible to end users and to system administrators.</p>
     * 
     * @param message the information to log (usually a <code>String</code>)
     */
    public void error(Object message);
    
    /**
     * Logs an error level message.
     * 
     * <p>This method is typically used when parsing a model file and 
     * when a parsing recoverable error must be logged.</p>
     * 
     * <p>In general, error messages should describe events that are of considerable importance and which will prevent normal program execution.
     * They should be reasonably intelligible to end users and to system administrators.</p>
     * 
     * @param message the information to log (usually a <code>String</code>)
     * @param filename the current file from which this message originates.
     * @param lineNumber the current file line number.
     * @param columnNumber the current file column number.
     */
    public void error(Object message, String filename, int lineNumber, int columnNumber);
    
    /**
     * Logs an error level message.
     * 
     * <p>In general, error messages should describe events that are of considerable importance and which will prevent normal program execution.
     * They should be reasonably intelligible to end users and to system administrators.</p>
     * 
     * @param message the information to log (usually a <code>String</code>), maybe <code>null</code>
     * @param error an exception to log.
     */
    public void error(Object message, Throwable error);

}
