package com.sodius.mdw.core.util.log;

/**
 * Dispatch log information to a set of registered loggers.
 * 
 * <p>When a logging facility (<code>debug()</code>, <code>error()</code>, etc.) is called,
 * this implementation just redirects the call to every registered logger.</p>
 * 
 * <p>This logger can be particulary useful for example to log information into a file and also to System streams :</p>
 * <p><code>
 * workbench.getConfiguration().setLogger(new LoggerSet(new Logger[] {<br>
 * &nbsp;&nbsp;new FileLogger(new File("myFile.log")),<br>
 * &nbsp;&nbsp;new SystemStreamLogger()<br>
 * }));
 * </p></code>
 * 
 * <p>
 * Clients may instantiate this class.
 * </p>
 */
public final class LoggerSet implements Logger {

    private Logger[] loggers;
    private int length;
    
    /**
     * Constructs a new <code>LoggerSet</code> that will dispatch log information to the specified loggers.
     * 
     * @param loggers the sub loggers this logger is in charge of.
     */
    public LoggerSet(Logger[] loggers) {
        this.length = loggers.length;
        this.loggers = new Logger[length];
        System.arraycopy(loggers, 0, this.loggers, 0, length);
    }

    /*
     * Javadoc dopied
     */
    public Level getLevel() {
        Level result = Level.ERROR;
        
        for (int i = 0; i < length; i++) {
            Level level = loggers[i].getLevel();
            if (level.compareTo(result) < 0)
                result = level;
        }
        
        return result;
    }
    
    /*
     * Javadoc dopied
     */
    public void setLevel(Level level) {
        for (int i = 0; i < length; i++)
            loggers[i].setLevel(level);
    }
    
    /*
     * Javadoc dopied
     */
    public boolean isLoggable(Level level) {
        for (int i = 0; i < length; i++) {
            if (loggers[i].isLoggable(level))
                return true;
        }
        
        return false;
    }

    /*
     * Javadoc dopied
     */
    public void debug(Object message) {
        for (int i = 0; i < length; i++)
            loggers[i].debug(message);
    }

    /*
     * Javadoc dopied
     */
    public void debug(Object message, String filename, int lineNumber, int columnNumber) {
        for (int i = 0; i < length; i++)
            loggers[i].debug(message, filename, lineNumber, columnNumber);
    }

    /*
     * Javadoc dopied
     */
    public void debug(Object message, Throwable error) {
        for (int i = 0; i < length; i++)
            loggers[i].debug(message, error);
    }

    /*
     * Javadoc dopied
     */
    public void info(Object message) {
        for (int i = 0; i < length; i++)
            loggers[i].info(message);
    }
    
    /*
     * Javadoc dopied
     */
    public void info(Object message, String filename, int lineNumber, int columnNumber) {
        for (int i = 0; i < length; i++)
            loggers[i].info(message, filename, lineNumber, columnNumber);
    }

    /*
     * Javadoc dopied
     */
    public void info(Object message, Throwable error) {
        for (int i = 0; i < length; i++)
            loggers[i].info(message, error);
    }

    /*
     * Javadoc dopied
     */
    public void warning(Object message) {
        for (int i = 0; i < length; i++)
            loggers[i].warning(message);
    }

    /*
     * Javadoc dopied
     */
    public void warning(Object message, String filename, int lineNumber, int columnNumber) {
        for (int i = 0; i < length; i++)
            loggers[i].warning(message, filename, lineNumber, columnNumber);
    }

    /*
     * Javadoc dopied
     */
    public void warning(Object message, Throwable error) {
        for (int i = 0; i < length; i++)
            loggers[i].warning(message, error);
    }

    /*
     * Javadoc dopied
     */
    public void error(Object message) {
        for (int i = 0; i < length; i++)
            loggers[i].error(message);
    }

    /*
     * Javadoc dopied
     */
    public void error(Object message, String filename, int lineNumber, int columnNumber) {
        for (int i = 0; i < length; i++)
            loggers[i].error(message, filename, lineNumber, columnNumber);
    }

    /*
     * Javadoc dopied
     */
    public void error(Object message, Throwable error) {
        for (int i = 0; i < length; i++)
            loggers[i].error(message, error);
    }

}
