package com.sodius.mdw.core.util.log;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;

/**
 * Provides methods to redirect <code>System.out</code> and <code>System.err</code> streams to a logger.
 * 
 * <p>For example, you can redirect System stream messages into a file using the following code : </p>
 * <p><code>
 * FileLogger fileLogger = new FileLogger(new File("myFile.log"));<br>
 * SystemStream.connect(fileLogger);<br>
 * workbench.getConfiguration().setLogger(fileLogger);
 * </code></p>
 * 
 * <p>
 * This class cannot be instantiated or subclassed by clients.
 * </p>
 */
public final class SystemStream {

    static final PrintStream SYSTEM_OUT = System.out;
    static final PrintStream SYSTEM_ERR = System.err;

    /**
     * Redirects <code>System.out</code> and <code>System.err</code> streams to the specified logger.
     * 
     * <p>When a logger is connected to system streams, 
     * any call to <code>System.out.print()</code> will be redirected to the logger <code>info()</code> method, 
     * and any call to <code>System.err.print()</code> will be redirected to the logger <code>error()</code> method.
     * Additionally, the messages will still be written into the original <code>System.out</code> and <code>System.err</code> streams</p>.
     * 
     * @param logger the logger which should receive any information written in <code>System.out</code> and <code>System.err</code> streams.
     */
    public static void connect(Logger logger) {
        System.setOut(new PrintStreamLogger(logger, SYSTEM_OUT));
        System.setErr(new PrintStreamLogger(logger, SYSTEM_ERR));
    }
    
    /**
     * Restores the original streams to <code>System.out</code> and <code>System.err</code>.
     */
    public static void disconnect() {
        System.setOut(SYSTEM_OUT);
        System.setErr(SYSTEM_ERR);
    }
    
    private static final OutputStream NULL_STREAM = new OutputStream() {
        
        public void write(int b) {
            // ignore
        }
    
        public void write(byte[] b) {
            // ignore
        }
    
        public void write(byte[] b, int off, int len) {
            // ignore
        }
    
        public void flush() {
            // ignore
        }
    
        public void close() {
            // ignore
        }
    };
    
    private static class PrintStreamLogger extends PrintStream {
        
        private Logger logger;
        private PrintStream systemStream;
        private boolean isErr;
        
        private PrintStreamLogger(Logger logger, PrintStream systemStream) {
            super(NULL_STREAM);
            this.logger = logger;
            this.systemStream = systemStream;
            this.isErr = (systemStream == SYSTEM_ERR);
        }
        
        public void close() {
            this.systemStream.close();
        }

        public void flush() {
            this.systemStream.flush();
        }

        public void print(boolean b) {
            this.systemStream.print(b);
            log(String.valueOf(b));
        }

        public void print(char c) {
            this.systemStream.print(c);
            log(String.valueOf(c));
        }

        public void print(char[] s) {
            this.systemStream.print(s);
            log(String.valueOf(s));
        }

        public void print(double d) {
            this.systemStream.print(d);
            log(String.valueOf(d));
        }

        public void print(float f) {
            this.systemStream.print(f);
            log(String.valueOf(f));
        }

        public void print(int i) {
            this.systemStream.print(i);
            log(String.valueOf(i));
        }

        public void print(long l) {
            this.systemStream.print(l);
            log(String.valueOf(l));
        }

        public void print(Object obj) {
            this.systemStream.print(obj);
            log(obj);
        }

        public void print(String s) {
            this.systemStream.print(s);
            log(s);
        }

        public void println(String x) {
            this.systemStream.println(x);
            log(x);
        }
        
        public void println() {
            this.systemStream.println();
            log(""); //$NON-NLS-1$
        }
        
        public void println(boolean x) {
            this.systemStream.println(x);
            log(String.valueOf(x));
        }

        public void println(char x) {
            this.systemStream.println(x);
            log(String.valueOf(x));
        }

        public void println(char[] x) {
            this.systemStream.println(x);
            log(String.valueOf(x));
        }

        public void println(double x) {
            this.systemStream.println(x);
            log(String.valueOf(x));
        }

        public void println(float x) {
            this.systemStream.println(x);
            log(String.valueOf(x));
        }

        public void println(int x) {
            this.systemStream.println(x);
            log(String.valueOf(x));
        }

        public void println(long x) {
            this.systemStream.println(x);
            log(String.valueOf(x));
        }

        public void println(Object x) {
            this.systemStream.println(x);
            log(String.valueOf(x));
        }

        public void write(byte[] b) throws IOException {
            this.systemStream.write(b);
            log(new String(b));
        }
        
        public void write(byte[] buf, int off, int len) {
            this.systemStream.write(buf, off, len);
            log(new String(buf, off, len));
        }
        
        public void write(int b) {
            this.systemStream.write(b);
            log(new Integer(b));
        }
        
        private void log(Object message) {
            if (this.isErr)
                this.logger.error(message);
            else
                this.logger.info(message);
        }

    }
    
    private SystemStream() {
    }

}
