package com.sodius.mdw.core.util.progress;

/**
 * A progress monitor that uses a given amount of work ticks
 * from a parent monitor.
 *  
 * <p>
 * This class may be instantiated or subclassed by clients.
 * </p>
 */
public class SubProgressMonitor extends ProgressMonitorWrapper {

    /**
     * Style constant indicating that calls to <code>subTask</code>
     * should not have any effect.
     *
     * @see #SubProgressMonitor(ProgressMonitor, int, int)
     */
    public static final int SUPPRESS_SUBTASK_LABEL = 1;
    
    /**
     * Style constant indicating that the main task label 
     * should be prepended to the subtask label.
     *
     * @see #SubProgressMonitor(ProgressMonitor, int, int)
     */
    public static final int PREPEND_MAIN_LABEL_TO_SUBTASK = 2;

    private int parentTicks = 0;
    private double sentToParent = 0.0;
    private double scale = 0.0;
    private int nestedBeginTasks = 0;
    private boolean usedUp = false;
    private int style;
    private String mainTaskLabel;

    /**
     * Creates a new sub-progress monitor for the given monitor. The sub 
     * progress monitor uses the given number of work ticks from its parent monitor.
     *
     * @param monitor the parent progress monitor
     * @param ticks the number of work ticks allocated from the parent monitor
     */
    public SubProgressMonitor(ProgressMonitor monitor, int ticks) {
        this(monitor, ticks, 0);
    }

    /**
     * Creates a new sub-progress monitor for the given monitor. The sub 
     * progress monitor uses the given number of work ticks from its parent monitor.
     *
     * @param monitor the parent progress monitor
     * @param ticks the number of work ticks allocated from the parent monitor
     * @param style one of
     *    <ul>
     *    <li> <code>SUPPRESS_SUBTASK_LABEL</code> </li>
     *    <li> <code>PREPEND_MAIN_LABEL_TO_SUBTASK</code> </li>
     *    </ul>
     * @see #SUPPRESS_SUBTASK_LABEL
     * @see #PREPEND_MAIN_LABEL_TO_SUBTASK
     */
    public SubProgressMonitor(ProgressMonitor monitor, int ticks, int style) {
        super(monitor);
        this.parentTicks = ticks;
        this.style = style;
    }

    /*
     * Javadoc copied
     */
    public void beginTask(String name, int totalWork) {
        nestedBeginTasks++;
        if (nestedBeginTasks > 1) {
            return;
        }
        
        scale = totalWork <= 0 ? 0 : (double) parentTicks / (double) totalWork;
        if (style == PREPEND_MAIN_LABEL_TO_SUBTASK) {
            mainTaskLabel = name;
        }
    }
    
    /*
     * Javadoc copied
     */
    public void done() {
        if (nestedBeginTasks == 0 || --nestedBeginTasks > 0)
            return;
        
        double remaining = parentTicks - sentToParent;
        if (remaining > 0)
            super.internalWorked(remaining);
        
        subTask(""); //$NON-NLS-1$
        sentToParent = 0;
    }

    /*
     * Javadoc copied
     */
    public void internalWorked(double work) {
        if (usedUp || nestedBeginTasks != 1) {
            return;
        }

        double realWork = scale * work;
        super.internalWorked(realWork);
        sentToParent += realWork;
        if (sentToParent >= parentTicks)
            usedUp = true;
    }

    /*
     * Javadoc copied
     */
    public void subTask(String name) {
        if (style == SUPPRESS_SUBTASK_LABEL)
            return;

        String label = name;
        if (style == PREPEND_MAIN_LABEL_TO_SUBTASK && mainTaskLabel != null && mainTaskLabel.length() > 0)
            label = mainTaskLabel + ' ' + label;
            
        super.subTask(label);
    }

    /*
     * Javadoc copied
     */
    public void worked(int work) {
        internalWorked(work);
    }
}
