
//Licensed Materials - Property of IBM 
// Copyright IBM Corporation 2006, 2008. All Rights Reserved.

package com.telelogic.rhapsody.core;


 /**
  * The IRPModelElement interface represents an element in a Rational Rhapsody model, and its methods reflect the behavior shared by the various types of model elements. The specific types of elements in a model are derived from this interface.
  */
public interface IRPModelElement {
	/**
	 * method addAssociation
	 * @throws RhapsodyRuntimeException
	 */
	public IRPAssociationClass addAssociation(IRPRelation end1, IRPRelation end2, String name);
	/**
	 * method addDependency
	 * @throws RhapsodyRuntimeException
	 */
	public IRPDependency addDependency(String dependsOnName, String dependsOnType);
	/**
 	 * Adds a dependency upon another model element.
 	 * @param element the model element that this element depends upon
 	 * @return the dependency created
 	 */
	public IRPDependency addDependencyTo(IRPModelElement element);
	/**
	 * method addLinkToElement
	 * @throws RhapsodyRuntimeException
	 */
	public IRPLink addLinkToElement(IRPModelElement toElement, IRPRelation assoc, IRPModelElement fromPort, IRPModelElement toPort);
	/**
	 * method addNewAggr
	 * @throws RhapsodyRuntimeException
	 */
	public IRPModelElement addNewAggr(String metaType, String name);
	/**
 	 * Adds a new property to the model element and assigns a value to it. Note that this method does not have a user interface equivalent in the Features window.
 	 * @param propertyKey the name of the property to add. The syntax to use for this parameter is Subject.Metaclass.Property, for example, CG.Class.NewProperty
 	 * @param propertyType the property type. The strings that can be used for this parameter are: "Enum","Bool","String","Color","Int","Double","Font","File","Path", and "MultiLine". If you want to add a property of type Enum, you can specify the possible values using the following syntax for this parameter: "Enum,wood,plastic,metal".
 	 * @param propertyValue the value to assign to the new property. For boolean properties, use "True" or "False". 
 	 */
	public void addProperty(String propertyKey, String propertyType, String propertyValue);
	/**
 	 * Applies the specified stereotype to the model element.
 	 * @param stereotype the stereotype to apply to the model element 
 	 */
	public void addSpecificStereotype(IRPStereotype stereotype);
	/**
 	 * Applies the specified stereotype to the model element if the project contains a stereotype with the name specified and applicable to the metaclass specified. If the project does not yet contain such a stereotype, this method creates the stereotype in the package that owns the model element, and applies the new stereotype to the model element.
 	 * @param name the name of the stereotype to apply (or create and apply)
 	 * @param metaType the metaclass that the stereotype is applicable to
 	 * @return the stereotype applied (or created and applied) to the model element
 	 */
	public IRPStereotype addStereotype(String name, String metaType);
	/**
 	 * Makes the current model element a template instantiation of the specified template.
 	 * @param newVal the template to use for the instantiation
 	 */
	public void becomeTemplateInstantiationOf(IRPModelElement newVal);
	/**
	 * method changeTo
	 * @throws RhapsodyRuntimeException
	 */
	public IRPModelElement changeTo(String metaClass);
	/**
 	 * Clones a model element.
 	 * @param name the name to use for the new element
 	 * @param newOwner the model element that should be the owner of the new element
 	 * @return the new model element that was created
 	 */
	public IRPModelElement clone(String name, IRPModelElement newOwner);
	/**
 	 * Deletes the specified dependency from the model.
 	 * @param dependency the dependency to be deleted
 	 */
	public void deleteDependency(IRPDependency dependency);
	/**
 	 * Deletes the current model element from the model.
 	 */
	public void deleteFromProject();
	/**
 	 * Returns the most recent error message.
 	 * @return the most recent error message
 	 */
	public String errorMessage();
	/**
 	 * Searches for the specified model element in the specified path under the current model element.
 	 * <pre> 
 	 * // this code gets the class Webcam in the package SpecializedCameras which is a subpackage of the top-level package Cameras
 	 * IRPProject currentProject = app.activeProject();
 	 * IRPClass classToFind = (IRPClass)currentProject.findElementsByFullName("Webcam in Cameras::SpecializedCameras", "Class");
 	 * System.out.println(classToFind.getFullPathName());
 	 * </pre> 
 	 * @param name the name of the element to search for and the relative path to the element starting at the current element. This argument must use the format "Class in Package::Subpackage"
 	 * @param metaClass the metaclass of the element you are looking for. The strings to use for this parameter should be taken from the file metaclasses.txt in the Doc directory of the Rhapsody installation.
 	 * @return the model element that was specified
 	 */
	public IRPModelElement findElementsByFullName(String name, String metaClass);
	/**
 	 * Searches for the specified model element. This method only searches the first level of elements below the current element. To search all of the levels below the current element, use the method findNestedElementRecursive.
 	 * @param name the name of the element to search for
 	 * @param metaClass the metaclass of the element you are looking for. The strings to use for this parameter should be taken from the file metaclasses.txt in the Doc directory of the Rhapsody installation.
 	 * @return the model element that was specified. Note that the element is always returned as an object of type IRPModelElement. So you will usually have to use casting, for example, <br>IRPPackage packageToUse = (IRPPackage)prj.findNestedElement("GreeterPackage", "Package");
 	 */
	public IRPModelElement findNestedElement(String name, String metaClass);
	/**
 	 * Searches recursively for the specified model element. This method searches all of the levels below the current element. To search only the first level of elements below the current element, use the method findNestedElement.
 	 * @param name the name of the element to search for
 	 * @param metaClass the metaclass of the element you are looking for. The strings to use for this parameter should be taken from the file metaclasses.txt in the Doc directory of the Rhapsody installation.
 	 * @return the model element that was specified. Note that the element is always returned as an object of type IRPModelElement. So you will usually have to use casting, for example, <br>IRPPackage packageToUse = (IRPPackage)prj.findNestedElementRecursive("GreeterPackage", "Package");
 	 */
	public IRPModelElement findNestedElementRecursive(String name, String metaClass);
	/**
 	 * Returns a collection of all the element's tags.
 	 * @return a collection of IRPTag objects representing the element's tags
 	 */
	public IRPCollection getAllTags();
	/**
 	 * Returns all of the element's annotations. This includes comments, constraints, and requirements.
 	 * @return all of the element's annotations: comments, constraints, and requirements
 	 */
	public IRPCollection getAnnotations();
	/**
	 * get property associationClasses
	 * @throws RhapsodyRuntimeException
	 */
	public IRPCollection getAssociationClasses();
	/**
	 * get property BinaryID
	 * @throws RhapsodyRuntimeException
	 */
	public byte[] getBinaryID();
	/**
 	 * Returns all of the element's constraints.
 	 * @return all of the element's constraints
 	 */
	public IRPCollection getConstraints();
	/**
	 * get property constraintsByHim
	 * @throws RhapsodyRuntimeException
	 */
	public IRPCollection getConstraintsByHim();
	/**
 	 * Returns a collection of all the element's controlled files.
 	 * @return a collection of IRPControlledFile objects representing the element's controlled files
 	 */
	public IRPCollection getControlledFiles();
	/**
 	 * Returns all of the element's dependencies.
 	 * @return all of the element's dependencies
 	 */
	public IRPCollection getDependencies();
	/**
 	 * Returns the description defined for the element.
 	 * @return the description for the element
 	 */
	public String getDescription();
	/**
 	 * Returns HTML representation of the element description.
 	 * @return HTML representation of the element description
 	 */
	public String getDescriptionHTML();
	/**
	 * get property description in plain text format
	 * @throws RhapsodyRuntimeException
	 */
	public String getDescriptionPlainText();
	/**
 	 * Returns the description defined for the element in RTF format.
 	 * @return the description for the element in RTF format
 	 */
	public String getDescriptionRTF();
	/**
 	 * Returns the label of the model element.
 	 * @return the label of the model element
 	 */
	public String getDisplayName();
	/**
 	 * Returns the label of the model element as an RTF string.
 	 * @return the label of the model element as an RTF string.
 	 */
	public String getDisplayNameRTF();
	/**
 	 * Returns the most recent error message.
 	 * @return the most recent error message
 	 */
	public String getErrorMessage();
	/**
 	 * Returns the full path name of the model element. The format of the string returned is package::subpackage::class.
 	 * <pre> 
 	 *  // this code prints the full path name for each class in the Cameras package, including the classes in subpackages
 	 *	System.out.println("======================================================================");
 	 *	IRPCollection allClassesInCamerasPackage = cameraPackage.getNestedElementsByMetaClass("Class", 1);
 	 *	int numberOfClasses = allClassesInCamerasPackage.getCount();
 	 *	// note that when using getItem to get an item from an IRPCollection object, the index starts at 1, not 0
 	 *	IRPModelElement elementInCollection;
 	 *	for(int i = 1; i < numberOfClasses+1 ; i++) {
 	 *		elementInCollection = (IRPModelElement)allClassesInCamerasPackage.getItem(i);
 	 * 	System.out.println(elementInCollection.getFullPathName());
 	 * }
 	 * </pre>
 	 * @return the full path name of the model element. The format of the string returned is package::subpackage::class.
 	 */
	public String getFullPathName();
	/**
 	 * Retrieves the full path name of the element as a string in the following format: (class) in (package).
 	 * @return the full path name of the element in the format: (class) in (package)
 	 */
	public String getFullPathNameIn();
	/**
	 * get property GUID
	 * @throws RhapsodyRuntimeException
	 */
	public String getGUID();
	/**
 	 * Returns a collection of all the hyperlinks associated with the element.
 	 * @return a collection of IRPHyperLink objects representing the hyperlinks associated with the element
 	 */
	public IRPCollection getHyperLinks();
	/**
	 * Get the full path of the element's icon
	 * @throws RhapsodyRuntimeException
	 */
	public String getIconFileName();
	/**
 	 * Returns the name of the API interface corresponding to the current element, for example, IRPClass for a class element, IRPOperation for an operation element.
 	 * @return the name of the API interface corresponding to the current element
 	 */
	public String getInterfaceName();
	/**
	 * get property isExternal
	 * @throws RhapsodyRuntimeException
	 */
	public int getIsExternal();
	/**
 	 * Indicates whether the model element is based on the metaclass provided as a parameter.
 	 * @param metaClass The name of the metaclass to check for. The strings to use for this parameter should be taken from the file metaclasses.txt in the Doc directory of the Rhapsody installation.
 	 * @return indication of whether the model element is based on the metaclass specified. 1 means that the model element is based on the metaclass specified, 0 means it is not based on that metaclass.
 	 */
	public int getIsOfMetaClass(String metaClass);
	/**
 	 * Checks whether the model element is configured to have its label displayed instead of its name whenever it is included in a diagram. This behavior is controlled by the General::Graphics::ShowLabels property.
 	 * @return 1 if the element is configured to have its label displayed instead of its name in diagrams, 0 otherwise
 	 */
	public int getIsShowDisplayName();
	/**
 	 * Checks if the element is an element that can't be resolved by Rhapsody.
 	 * @return indication of whether the element is an unresolved element - 0 if the element can be resolved, 1 if the element is unresolved 
 	 */
	public int getIsUnresolved();
	/**
 	 * Returns a collection of the tags that were created locally for this model element.
 	 * @return the tags that were created locally for the model element (collection of IRPTag elements)
 	 */
	public IRPCollection getLocalTags();
	/**
 	 * Returns the "main" diagram for the element. This operation is valid only for packages, classes, actors, use cases, objects, and interfaces.
 	 * @return the "main" diagram for the element
 	 */
	public IRPDiagram getMainDiagram();
	/**
 	 * Gets the name of the metaclass on which the model element is based. Note that if the element is based on a New Term stereotype, the string returned here will be the metaclass on which it is based. To get the name of the New Term stereotype, use the method IRPModelElement.getUserDefinedMetaClass.
 	 * @return the name of the metaclasses on which the model element is based. The string returned will be one of the the metaclass names listed in the file metaclasses.txt in the Doc directory of the Rhapsody installation. For example, for an object of type IRPStereotype, the string "Stereotype" will be returned. 
 	 */
	public String getMetaClass();
	/**
 	 * Returns the name of the element.
 	 * @return the name of the element
 	 */
	public String getName();
	/**
 	 * Gets a collection of all the model elements that are directly under the current element. Note that if you call this method on a package, the returned collection will not include functions, global variables, or global objects contained in the package because these are actually contained in a class called TopLevel. To get the functions, global variables, or global objects contained in a package, use the following IRPPackage methods: getGlobalFunctions(), getGlobalVariables(), and getGlobalObjects().
 	 * @return a collection of IRPModelElement objects representing all the model elements that are directly under the current element 
 	 * <pre>
 	 * {@code
 	 * IRPProject prj = app.openProject("l:\\temp\\_sample_code\\Unit_Tricks.rpy");
 	 * IRPPackage vehiclePackage = prj.addPackage("Vehicles");
 	 * vehiclePackage.addClass("Car");
 	 * vehiclePackage.addClass("Jeep");
 	 * vehiclePackage.addClass("Convertible");
 	 * prj.save();
 	 * IRPCollection elementsInVehiclesPackage = vehiclePackage.getNestedElements();
 	 * IRPModelElement elementInCollection;
 	 * System.out.println("The Vehicles package contains:");
 	 * for (int i = 1; i <= elementsInVehiclesPackage.getCount(); i++) {
 	 * 	elementInCollection = (IRPModelElement)elementsInVehiclesPackage.getItem(i);
 	 * 	System.out.println("\t" + elementInCollection.getName());
 	 * }
 	 * }
 	 * </pre>
 	 */
	public IRPCollection getNestedElements();
	/**
 	 * Retrieves all of the model elements of the specified type below the current element. The second argument can be used to specify whether the retrieval should be recursive.
 	 * <pre> 
 	 *  // this code retrieves all the classes in the Cameras package (including classes in subpackages), and prints their type and name
 	 * 		IRPCollection allClassesInCamerasPackage = cameraPackage.getNestedElementsByMetaClass("Class", 1);
 	 * 		int numberOfClasses = allClassesInCamerasPackage.getCount();
 	 * 		// note that when using getItem to get an item from an IRPCollection object, the index starts at 1, not 0
 	 * 		IRPModelElement elementInCollection;
 	 * 		for(int i = 1; i < numberOfClasses+1 ; i++) {
 	 * 			elementInCollection = (IRPModelElement)allClassesInCamerasPackage.getItem(i);
 	 * 			System.out.println(elementInCollection.getMetaClass() + ": " + elementInCollection.getDisplayName());
 	 * 		}
 	 * </pre>
 	 * @param metaClass the type of elements that you want to retrieve. The strings to use for this parameter should be taken from the file metaclasses.txt in the Doc directory of the Rhapsody installation.
 	 * @param recursive Use 1 to specify that the retrieval should be recursive. Use 0 if you only want to retrieve the relevant elements from the first level below the current element.
 	 * @return a collection of the model elements of the specified type below the current element
 	 */
	public IRPCollection getNestedElementsByMetaClass(String metaClass, int recursive);
	/**
 	 * Retrieves all of the model elements below the current element.
 	 * <pre> 
 	 * // this code retrieves all the items in the Cameras package, and prints their type and name
 	 * IRPCollection allItemsInCameraPackage = cameraPackage.getNestedElementsRecursive();
 	 * int numberOfElements = allItemsInCameraPackage.getCount();
 	 * // note that when using getItem to get an item from an IRPCollection object, the index starts at 1, not 0
 	 * IRPModelElement elementInCollection;
 	 * for(int i = 1; i < numberOfElements+1 ; i++) {
 	 * 	elementInCollection = (IRPModelElement)allItemsInCameraPackage.getItem(i);
 	 * 	System.out.println(elementInCollection.getMetaClass() + ": " + elementInCollection.getDisplayName());
 	 * }
 	 * </pre>
 	 * @return a collection of all the model elements below the current element
 	 */
	public IRPCollection getNestedElementsRecursive();
	/**
	 * method getNewTermStereotype
	 * @throws RhapsodyRuntimeException
	 */
	public IRPStereotype getNewTermStereotype();
	/**
 	 * If the element is an instantiation of a template, this method returns the template that it instantiates.
 	 * @return the template that this model element instantiates
 	 */
	public IRPModelElement getOfTemplate();
	/**
	 * Get the full path of the element's overlay icon
	 * @throws RhapsodyRuntimeException
	 */
	public String getOverlayIconFileName();
	/**
	 * method getOverriddenProperties
	 * @throws RhapsodyRuntimeException
	 */
	public IRPCollection getOverriddenProperties(int recursive);
	/**
	 * method getOverriddenPropertiesByPattern
	 * @throws RhapsodyRuntimeException
	 */
	public IRPCollection getOverriddenPropertiesByPattern(String pattern, int localyOverridenOnly, int withDefaultValues);
	/**
 	 * Returns the model element that owns this model element.
 	 * @return the model element that owns this model element
 	 */
	public IRPModelElement getOwner();
	/**
 	 * Returns the project that the current element belongs to.
 	 * @return the project the current element belongs to
 	 */
	public IRPProject getProject();
	/**
 	 * Returns the value of the specified property for the model element.
 	 * @param propertyKey the property whose value should be returned. The syntax to use for this parameter is Subject.Metaclass.Property, for example, CG.Class.ActiveThreadName
 	 * @return the value of the specified property. If a value has not been set specifically for this element, the default value is returned (the value propagated from a higher level)
 	 */
	public String getPropertyValue(String propertyKey);
	/**
 	 * Returns the value of the specified property for the model element, taking into account the collection of tokens specified and the collection of token values specified. For more information on using tokens in property values, see "Conditional Properties" in the Rational Rhapsody help.
 	 * @param propertyKey the property whose value should be returned. The syntax to use for this parameter is Subject.Metaclass.Property, for example, CG.Class.ActiveThreadName
 	 * @param formalKey the collection of tokens to take into account (collection of strings)
 	 * @param actualValues the collection of token values to take into account (collection of strings)
 	 * @return the value of the specified property, taking into account the tokens and token values specified. If a value has not been set specifically for this element, the default value is returned (the value propagated from a higher level)
 	 */
	public String getPropertyValueConditional(String propertyKey, IRPCollection formalKey, IRPCollection actualValues);
	/**
 	 * Returns the value of the specified property for the model element, if the default value was overridden, taking into account the collection of tokens specified and the collection of token values specified. For more information on using tokens in property values, see "Conditional Properties" in the Rational Rhapsody help. If a value has not been set explicitly for the model element, the method will not return the default value (like the getPropertyValueConditional method does). Rather, it will throw an exception.
 	 * @param propertyKey the property whose value should be returned. The syntax to use for this parameter is Subject.Metaclass.Property, for example, CG.Class.ActiveThreadName
 	 * @param formalKey the collection of tokens to take into account (collection of strings)
 	 * @param actualValues the collection of token values to take into account (collection of strings)
 	 * @return the value that was explicitly set for the model element for the specified property, taking into account the tokens and token values specified
 	 */
	public String getPropertyValueConditionalExplicit(String propertyKey, IRPCollection formalKey, IRPCollection actualValues);
	/**
 	 * Returns the value of the specified property for the model element if the default value was overridden. If a value has not been set explicitly for the model element, it will not return the default value (like the getPropertyValue method does). Rather, it will throw an exception.
 	 * @param propertyKey the property whose value should be returned. The syntax to use for this parameter is Subject.Metaclass.Property, for example, CG.Class.ActiveThreadName
 	 * @return the value that was explicitly set for the model element for the specified property
 	 */
	public String getPropertyValueExplicit(String propertyKey);
	/**
 	 * Returns a collection of all the model elements that point to this model element.
 	 * @return all the model elements that point to this model element
 	 */
	public IRPCollection getReferences();
	/**
	 * get property requirementTraceabilityHandle
	 * @throws RhapsodyRuntimeException
	 */
	public int getRequirementTraceabilityHandle();
	/**
 	 * Returns the unit that the model element is saved in.
 	 * @return the unit that the element is saved in
 	 */
	public IRPUnit getSaveUnit();
	/**
 	 * @deprecated Since Rhapsody now allows multiple stereotypes to be applied to a model element, the getStereotypes() method should be used instead.
 	 */
	public IRPStereotype getStereotype();
	/**
 	 * Returns a collection of the stereotypes that have been applied to the element.
 	 * @return the stereotypes that have been applied to the element
 	 */
	public IRPCollection getStereotypes();
	/**
 	 * Returns the tag specified. This method can be used for both local tags and global tags.
 	 * @param name the name of the tag to return
 	 * @return the tag specified
 	 */
	public IRPTag getTag(String name);
	/**
 	 * For model elements that are templates, returns the template parameters.
 	 * @return the parameters of the template
 	 */
	public IRPCollection getTemplateParameters();
	/**
 	 * For model elements that are template instantiations, returns an object that contains the template instantiation parameters.
 	 * @return object that contains the template instantiation parameters
 	 */
	public IRPTemplateInstantiation getTi();
	/**
 	 * Gets the name of the New Term on which the model element is based.
 	 * @return the name of the New Term on which the model element is based. The string returned will be the name of the New Term stereotype that you defined. To get the name of the metaclass on which the New Term is based, use the method IRPModelElement.getMetaClass(). 
 	 */
	public String getUserDefinedMetaClass();
	/**
 	 * Checks whether the model element contains other elements.
 	 * @return 1 if the model element contains other elements, 0 otherwise
 	 */
	public int hasNestedElements();
	/**
	 * method hasPanelWidget
	 * @throws RhapsodyRuntimeException
	 */
	public int hasPanelWidget();
	/**
 	 * Locates the element in the Rhapsody browser, and highlights the element in the diagram where it appears. Note that the element will be highlighted in the diagram only if it is the kind of element that can appear in only one diagram, for example, a state.
 	 */
	public void highLightElement();
	/**
 	 * Checks whether the model element is a template.
 	 * @return 1 if the element is a template, 0 otherwise
 	 */
	public int isATemplate();
	/**
 	 * Checks whether the description for the element is in RTF format.
 	 * @return 1 if the description is in RTF format, 0 otherwise
 	 */
	public int isDescriptionRTF();
	/**
 	 * Checks whether the label of the element is in RTF format.
 	 * @return 1 if the label is in RTF format, 0 otherwise
 	 */
	public int isDisplayNameRTF();
	/**
	 * Check if the element is modified
	 * @throws RhapsodyRuntimeException
	 */
	public int isModified();
	/**
 	 * Locates the model element in the Rhapsody browser.
 	 * @return returns 1 if the element was located in the browser
 	 */
	public int locateInBrowser();
	/**
 	 * Displays the information for the element in the Features window. Depending on the value of the parameter provided, opens a new Features window or uses an already-open Features window.
 	 * @param newDialog Use 1 to specify that the element information should be displayed in a new Features window. Use 0 to specify that the information should be displayed in a Features window that is already open or in a new window if there is no open Features window.
 	 */
	public void openFeaturesDialog(int newDialog);
	/**
 	 * Removes the value that was set for the specified property. This is equivalent to the "un-override" option in the Features window.
 	 * @param propertyKey the property whose value should be removed. The syntax to use for this parameter is Subject.Metaclass.Property, for example, CG.Class.ActiveThreadName
 	 */
	public void removeProperty(String propertyKey);
	/**
 	 * Removes the specified stereotype from the element.
 	 * @param stereotype the stereotype to be removed from the element
 	 */
	public void removeStereotype(IRPStereotype stereotype);
	/**
 	 * Sets the specified string as the description of the element.
 	 * @param description the string to use as the description of the element
 	 */
	public void setDescription(String description);
	/**
 	 * Specifies an RTF string to use as the description for the element, and a collection of elements to which hyperlinks should be created.
 	 * @param rtfText the string to use for the element description - must be in RTF format
 	 * @param targets the collection of elements for which hyperlinks should be created
 	 */
	public void setDescriptionAndHyperlinks(String rtfText, IRPCollection targets);
	/**
 	 * Specifies the HTML text to use for the description of the model element.
 	 * @param descriptionHTML the HTML text to use for the description of the model element
 	 */
	public void setDescriptionHTML(String descriptionHTML);
	/**
 	 * Specifies the RTF string to use for the description of the model element.
 	 * @param descriptionRTF the RTF string to use for the description of the model element
 	 */
	public void setDescriptionRTF(String descriptionRTF);
	/**
 	 * Specifies the text to use for the label of the model element.
 	 * @param displayName the text to use for the label of the model element
 	 */
	public void setDisplayName(String displayName);
	/**
 	 * Specifies the RTF string to use for the label of the model element.
 	 * @param newVal the RTF string to use for the label of the model element
 	 */
	public void setDisplayNameRTF(String newVal);
	/**
	 * set property GUID
	 * @throws RhapsodyRuntimeException
	 */
	public void setGUID(String gUID);
	/**
 	 * Specifies whether the label of the element should be displayed instead of the element name whenever the element is used in a diagram. This method changes the value of the General::Graphics::ShowLabels property.
 	 * @param isShowDisplayName use 1 if you want the label of the element displayed, use 0 if you want the name of the element displayed
 	 */
	public void setIsShowDisplayName(int isShowDisplayName);
	/**
 	 * Specifies the "main" diagram for the element. This operation is valid only for packages, classes, actors, use cases, objects, and interfaces.
 	 * @param mainDiagram the diagram to use as the "main" diagram for the element
 	 */
	public void setMainDiagram(IRPDiagram mainDiagram);
	/**
 	 * Sets the specified string as the name of the element.
 	 * @param name the string to use as the name of the element
 	 */
	public void setName(String name);
	/**
 	 * Makes the current model element a template instantiation of the specified template.
 	 * @param ofTemplate the template to use for the instantiation
 	 */
	public void setOfTemplate(IRPModelElement ofTemplate);
	/**
 	 * Specifies the model element that should be the owner of this element.
 	 * @param owner the model element that should be the owner of this element
 	 */
	public void setOwner(IRPModelElement owner);
	/**
 	 * Sets the value of a property for the model element.
 	 * @param propertyKey the property whose value should be set. The syntax to use for this parameter is Subject.Metaclass.Property, for example, CG.Class.ActiveThreadName.
 	 * @param propertyValue the new value to use for the property. For boolean properties, use "True" or "False". 
 	 */
	public void setPropertyValue(String propertyKey, String propertyValue);
	/**
	 * set property requirementTraceabilityHandle
	 * @throws RhapsodyRuntimeException
	 */
	public void setRequirementTraceabilityHandle(int requirementTraceabilityHandle);
	/**
 	 * Applies the specified stereotype to the element.
 	 * @param stereotype the stereotype to be applied to the element
 	 */
	public void setStereotype(IRPStereotype stereotype);
	/**
	 * method setTagContextValue
	 * @throws RhapsodyRuntimeException
	 */
	public IRPTag setTagContextValue(IRPTag tag, IRPCollection elements, IRPCollection multiplicities);
	/**
 	 * Applies a tag whose type is a model element to the current element with the value specified. If the tag has already been applied to the current element, the method can be used to modify the value of the tag.
 	 * @param tag the tag to apply to the element
 	 * @param val the value to use for the tag applied
 	 * @return the tag applied to the model element
 	 */
	public IRPTag setTagElementValue(IRPTag tag, IRPModelElement val);
	/**
 	 * Applies the specified tag to the model element with the value specified. If the tag has already been applied to the model element, the method can be used to modify the value of the tag.
 	 * @param tag the tag to apply to the element
 	 * @param val the value to use for the tag applied
 	 * @return the tag applied to the model element
 	 */
	public IRPTag setTagValue(IRPTag tag, String val);
	/**
	 * set property template instantiation
	 * @throws RhapsodyRuntimeException
	 */
	public void setTi(IRPTemplateInstantiation ti);
	/**
	 * synchronizeTemplateInstantiation
	 * @throws RhapsodyRuntimeException
	 */
	public void synchronizeTemplateInstantiation();
}

